<?

 /*****************************************************************************
 * Filename: include/modules/categories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_categories extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Category";

   $this->data->defaultFields();

   $this->data->addField("ordernr", "nr", "all", "order", false, false, 0, "Order number", "Leave: 0 for automatic number");
   $this->data->addField("name", "text", "all", "title", 1, 25, false, "Name", "Name of the category");
   $this->data->addField("text", "text", "all", false, 0, 255, false, "Explanation", "Explanation about it's content");
   $this->data->addField("moduleLock", "boolean", "all", false, false, false, true, "Module lock", "Only allow selected modules in this category");
   $this->data->addField("module", "modules", "all", false, 0, 65536, false, "Module", "Allowed modules in this category");

   $this->data->addField("recurseCategories", "boolean", "", false, false, false, false, "Recurse categories", "Delete all underlying categories");
   $this->data->addField("recurseContent", "boolean", "", false, false, false, false, "Recurse content", "Delete all underlying content");

   $this->data->addUnique("uniqueInCategory", "catnr", "name");

   $this->data->sortOrder("ordernr", "name");

   $this->data->group("Modules", "moduleLock", "module");

   //rights manipulation
   $this->data->addActions("rights");
  }

  function postInit()
  {
   if($this->data->action=="rights")
    return $this->initRightsFields();

   return true;
  }

  function rightsPostGet()
  {
   if($this->data->action=="rights")
    return $this->setRightsFields();

   return true;
  }

  function addPostSubmit()
  {
   //copy rights from parent category
   $du=Data::cache("userRights", "view", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "view", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!$this->data->getValue("catnr", $c)) //skip top level category
     continue;

    //copy user rights
    $du->setAction("view");
    $du->setExtra("all");

    $du->clearValues();
    $du->setValue("catnr", $this->data->getValue("catnr", $c));

    if(!$du->get())
    {
     $this->data->e->add($du->e);
     return false;
    }

    if($du->vmax)
    {
     for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
      $du->setValue("catnr", $this->data->getValue("nr", $c), $du->vc);

     $du->setAction("add");
     $du->setExtra("item");

     //no checking, just add: should be ok
     for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
     {
      if(!$dus->add())
      {
       $this->data->e->add($dus->e);
       return false;
      }
     }
    }

    //copy group rights
    $dg->setAction("view");
    $dg->setExtra("all");

    $dg->clearValues();
    $dg->setValue("catnr", $this->data->getValue("catnr", $c));

    if(!$dg->get())
    {
     $this->data->e->add($dg->e);
     return false;
    }

    if($dg->vmax)
    {
     for($dg->vc=0;$dg->vc<$dg->vmax;$dg->vc++)
      $dg->setValue("catnr", $this->data->getValue("nr", $c), $dg->vc);

     $dg->setAction("add");
     $dg->setExtra("item");

     //no checking, just add: should be ok
     for($dg->vc=0;$dg->vc<$dg->vmax;$dg->vc++)
     {
      if(!$dgs->add())
      {
       $this->data->e->add($dgs->e);
       return false;
      }
     }
    }
   }

   return true;
  }

  function movePostSubmitErrorCheck()
  {
   //additional move checks
   $d=Data::cache("categories", "tree", "down");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!($newcatnr=$this->data->getValue("catnr", $c))) //no new category for this entry
     $newcatnr=$this->data->getValue("catnr"); //use new category of first entry

    if($this->data->getValue("nr", $c)==$newcatnr)
    {
     $this->data->e->add("Unable to move the category to itself", "error", "Entry: ".($c+1));
     return false;
    }

    //check if new category is a sub category of it's own
    $d->clearValues();
    $d->setShowFields(); //don't get any data, only id's
    $d->setValue("catnr", $this->data->getValue("nr", $c));

    if(!$d->get())
    {
     $this->data->e->add("Can't check tree for moving the category", "error", "Entry: ".($c+1));
     $this->data->e->add($d->e);
     return false;
    }

    while($d->next())
    {
     if($d->v["nr"]==$newcatnr)
     {
      $this->data->e->add("Unable to move the category to a subcategory of itself", "error", "Entry: ".($c+1));
      return false;
     }
    }
   }

   return true;
  }

  function deletePostSubmitErrorCheck()
  {
   //check if category has underlying data in case of no recursive delete
   $return=true;
   $d=Data::cache("categories", "view", "list");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //subcategories ?
    if(!$this->data->getValue("recurseCategories")||!$this->data->check("valuetype", "set", "recurseCategories"))
    {
     $d->setExtra("list");

     $d->clearValues();
     $d->setValue("catnr", $this->data->getValue("nr", $c));

     if(!$d->get(false, true))
     {
      $this->data->e->add($d->e);
      return false;
     }

     if($d->getValue("show_count")>0)
     {
      $this->data->e->add("Unable to delete category because it contains subcategories", "error", "Entry: ".($c+1));
      $return=false;
     }
    }

    //content for category referenced modules
    if(!$this->data->getValue("recurseContent")||!$this->data->check("valuetype", "set", "recurseContent"))
    {
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr", $c));
     $d->setShowFields("module");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     foreach($d->getValue("module") as $m)
     {
      $d2=Data::cache($m, "view", "list");

      if(!($catnr=$d2->getRightsReference()))
       continue;

      $d2->setValue($catnr, $this->data->getValue("nr", $c));

      if(!$d2->get(false, true))
      {
       $this->data->e->add($d2->e);
       return false;
      }

      if($d2->getValue("show_count")>0)
      {
       $this->data->e->add("Unable to delete category because it contains ".$d2->title." data", "error", "Entry: ".($c+1));
       $return=false;
      }
     }
    }
   }

   return $return;
  }

  function deletePreSubmit()
  {
   //delete underlying data in case of recursive delete
   $d=Data::cache("categories", "view", "list");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //subcategories ?
    if($this->data->getValue("recurseCategories")&&$this->data->check("valuetype", "set", "recurseCategories"))
    {
     $d->setAction("delete");
     $d->setExtra("list");

     $d->clearValues();
     $d->setValue("catnr", $this->data->getValue("nr", $c));
     $d->setShowFields("nr");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     //delete if any subcategories
     if($d->vmax)
     {
      //set recursive categories/content flag to underlying categories
      $d->setValue("recurseCategories", $this->data->getValue("recurseCategories"));
      $d->setValue("recurseContent", $this->data->getValue("recurseContent"));

      if(!$d->submit())
      {
       $this->data->e->add($d->e);
       return false;
      }
     }
    }

    //content for category referenced modules
    if($this->data->getValue("recurseContent")&&$this->data->check("valuetype", "set", "recurseContent"))
    {
     $d->setAction("view");
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr", $c));
     $d->setShowFields("module");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     foreach($d->getValue("module") as $m)
     {
      $d2=Data::cache($m, "delete", "list");

      if(!($catnr=$d2->getRightsReference()))
       continue;

      $d2->setValue($catnr, $this->data->getValue("nr", $c));
      $d2->setShowFields("nr");

      if(!$d2->get())
      {
       $this->data->e->add($d2->e);
       return false;
      }

      //delete if any content
      if($d2->vmax&&!$d2->submit())
      {
       $this->data->e->add($d2->e);
       return false;
      }
     }
    }
   }

   return true;
  }

  function deletePostSubmit()
  {
   //remove rights for category
   $du=Data::cache("userRights", "delete", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "delete", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //delete user rights
    $du->clearValues();
    $du->setValue("catnr", $this->data->getValue("nr", $c));

    $du->vc=0;
    if(!$dus->delete(true))
    {
     $this->data->e->add($dus->e);
     return false;
    }

    //delete group rights
    $dg->clearValues();
    $dg->setValue("catnr", $this->data->getValue("nr", $c));

    $dg->vc=0;
    if(!$dgs->delete(true))
    {
     $this->data->e->add($dgs->e);
     return false;
    }

    //delete rights from group
    $dg->clearValues();
    $dg->setValue("nr", $this->data->getValue("nr", $c));

    $dg->vc=0;
    if(!$dgs->delete(true))
    {
     $this->data->e->add($dgs->e);
     return false;
    }
   }


   //users module usersAdditionalGroups
   $users=Data::cache("users", "view", "item");
   if($users->check("reference", "categories", "usersAdditionalGroups"))
   {
    $da=$users->getReferenceData("usersAdditionalGroups");
    if(!($das=$da->storageObject()))
     return false;

    for($c=0;$c<$this->data->vmax;$c++)
    {
     //delete additional user rights
     $da->clearValues();
     $da->setValue("categories", $this->data->getValue("nr", $c));

     $du->vc=0;
     if(!$das->delete(true))
     {
      $this->data->e->add($das->e);
      return false;
     }
    }
   }

   return true;
  }


  /*
   * Rights
   */

  function initRightsFields()
  {
   global $_config;

   $this->data->addField("usernr", "list", "form", "reload", 0, false, false, "User");
   $this->data->addField("groupnr", "list", "form", "reload", 0, false, false, "Group");
   $this->data->addField("rights", "rights", "form", "", 0, false, "", "Rights", "Grant the selected rights to the user/group");

   $this->data->group("Rights", "usernr", "groupnr", "rights");

   //get users list
   $du=Data::cache($_config->get("usermodule"), "view", "all");
   $du->setShowFields($du->check("properties", "title"));

   $du->clearValues();
   $du->get();

   $users=Array("" => "");
   while($du->next())
    $users[$du->v("id")]=$du->v("title");

   $this->data->listOptions("usernr", $users);

   //get categories list
   $dg=Data::cache($this->data->module, "view", "all");
   $dg->setShowFields($dg->check("properties", "title"));

   //only categories where users module is set
   $dg->clearValues();
   $dg->setValue("module", "users");
   $dg->setOperator("module", "like");

   $dg->get();

   $groups=Array("" => "");
   while($dg->next())
    $groups[$dg->v("id")]=$dg->v("title");

   $this->data->listOptions("groupnr", $groups);

   //set values
   $this->data->setDefaultValue("usernr");
   $this->data->setDefaultValue("groupnr");
   $this->data->setDefaultValue("rights");

   $this->data->setRequestValue("usernr");
   $this->data->setRequestValue("groupnr");
   $this->data->setRequestValue("rights");

   return true;
  }

  function setRightsFields()
  {
   global $_config;

   $this->data->setDefaultValue("usernr");
   $this->data->setDefaultValue("groupnr");
   $this->data->setDefaultValue("rights");

   $this->data->setRequestValue("usernr");
   $this->data->setRequestValue("groupnr");
   $this->data->setRequestValue("rights");

   if($this->data->getValue("reload_trigger")=="usernr") //user selected
    $this->data->unsetValue("groupnr"); //clear group
   else if($this->data->getValue("reload_trigger")=="groupnr") //group selected
    $this->data->unsetValue("usernr"); //clear user

   if($this->data->extra=="item") //change single item rights
   {
    //get user rights
    $du=Data::cache("userRights", "view", "all");

    $du->clearValues();
    $du->setValue("catnr", $this->data->getValue("nr"));
    $du->get();

    //add current rights to users list
    while($du->next())
     if(isset($this->data->listOptions["usernr"][$du->v["nr"]]))
      $this->data->listOptions["usernr"][$du->v["nr"]].=": ".substr($du->v["rights"], 0, 6)." ".substr($du->v["rights"], 8, 6);

    //get group rights
    $dg=Data::cache("groupRights", "view", "all");

    $dg->clearValues();
    $dg->setValue("catnr", $this->data->getValue("nr"));
    $dg->get();

    //add current rights to groups list
    while($dg->next())
     if(isset($this->data->listOptions["groupnr"][$dg->v["nr"]]))
      $this->data->listOptions["groupnr"][$dg->v["nr"]].=": ".substr($dg->v["rights"], 0, 6)." ".substr($dg->v["rights"], 8, 6);

    //set current rights based on selected user/group
    if($this->data->getValue("reload_trigger")=="usernr") //user selected
    {
     $this->data->setValue("rights", $this->data->rights2bin(0));

     if($nr=$this->data->getValue("usernr"))
     {
      $du->reset();
      while($du->next())
      {
       if($du->v["nr"]==$nr)
       {
        $this->data->setValue("rights", $du->v["rights"]);
        break;
       }
      }
     }
    }
    else if($this->data->getValue("reload_trigger")=="groupnr") //group selected
    {
     $this->data->setValue("rights", $this->data->rights2bin(0));

     if($nr=$this->data->getValue("groupnr"))
     {
      $dg->reset();
      while($dg->next())
      {
       if($dg->v["nr"]==$nr)
       {
        $this->data->setValue("rights", $dg->v["rights"]);
        break;
       }
      }
     }
    }
   }

   return true;
  }

  function rightsPostSubmitErrorCheck()
  {
   global $_config;

   $this->data->checkField("usernr");
   $this->data->checkField("groupnr");
   $this->data->checkField("rights");

   $d=clone $this->data;
   $d->setShowFields("module");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("usernr", $c)==$_config->get("rootnr"))
     $this->data->e->add("Can't change root rights", "error", "Entry: ".($c+1));

    if($this->data->getValue("groupnr", $c)&&$this->data->sqlValue("rights", $c))
    {
     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("groupnr", $c));
     $d->get();

     $usercat=false;
     if($d->next())
     {
      if(is_array($d->v["module"])&&in_array($_config->get("usermodule"), $d->v["module"]))
       $usercat=true;
      else if(!is_array($d->v["module"])&&$_config->get("usermodule")==$d->v["module"])
       $usercat=true;
     }

//     if(!$usercat)
//      $this->data->e->add("Can't change rights for non user category", "error", "Entry: ".($c+1));
    }
   }

   if($this->data->e->any("fatal"))
    return false;

   return true;
  }

  function rightsPostSubmit()
  {
   $du=Data::cache("userRights", "view", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "view", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("usernr")) //user changed
    {
     $du->setAction("view");
     $du->setExtra("all");

     $du->clearValues();
     $du->setValue("catnr", $this->data->getValue("nr", $c));
     $du->setValue("nr", $this->data->getValue("usernr"));

     $du->get();

     if($du->next()) //already in database: update/delete/nothing
     {
      if(!$this->data->sqlValue("rights")) //delete
      {
       $du->setAction("delete");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr"));

       $du->vc=0;
       if(!$dus->delete(true))
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights revoked", "note");
      }
      else if($this->data->getValue("rights")!=$du->v["rights"]) //update
      {
       $du->setAction("edit");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr"));
       $du->setValue("rights", $this->data->getValue("rights"));
       $du->field["rights"]["valuetype"]="request";

       $du->vc=0;
       if(!$dus->edit(true))
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights granted", "note");
      }
     }
     else //not in database: add/nothing
     {
      if($this->data->sqlValue("rights")) //add
      {
       $du->setAction("add");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr"));
       $du->setValue("rights", $this->data->getValue("rights"));

       $du->vc=0;
       if(!$dus->add())
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights granted", "note");
      }
     }
    }

    if($this->data->getValue("groupnr")) //group changed
    {
     $dg->setAction("view");
     $dg->setExtra("all");

     $dg->clearValues();
     $dg->setValue("catnr", $this->data->getValue("nr", $c));
     $dg->setValue("nr", $this->data->getValue("groupnr"));
     $dg->get();

     if($dg->next()) //already in database: update/delete/nothing
     {
      if(!$this->data->sqlValue("rights")) //delete
      {
       $dg->setAction("delete");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr"));

       $dg->vc=0;
       if(!$dgs->delete(true))
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights revoked", "note");
      }
      else if($this->data->getValue("rights")!=$dg->v["rights"]) //update
      {
       $dg->setAction("edit");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr"));
       $dg->setValue("rights", $this->data->getValue("rights"));
       $dg->field["rights"]["valuetype"]="request";

       $dg->vc=0;
       if(!$dgs->edit(true))
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights granted", "note");
      }
     }
     else //not in database: add/nothing
     {
      if($this->data->sqlValue("rights")) //add
      {
       $dg->setAction("add");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr"));
       $dg->setValue("rights", $this->data->getValue("rights"));

       $dg->vc=0;
       if(!$dgs->add())
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights granted", "note");
      }
     }
    }
   }

   return true;
  }

 }
?>
