<?

 /*****************************************************************************
 * Filename: include/modules/dataset.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_dataset extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->filters=6;

   $this->data->title="Data set";

   $this->data->defaultFields();

   $this->data->addField("count", "nr", "form", "disabled", 0, false, 0, "Results count");
   $this->data->addField("name", "text", "all", "title", 1, 255, "", "Name");
   $this->data->addField("module", "module", "all", "reload;exists", 1, 255, "", "Module");


   $this->data->addField("extra", "enum", "all", "reload", false, false, "Item", "Extra");
   $this->data->enumOptions("extra", "Item", "List", "All");

   for($c=1;$c<=$this->filters;$c++)
   {
    $this->data->addField("filter_field_".$c, "list", "all", "reload", 0, 25, "", "Filter ".$c, "Custom allows you to use request variables and/or static values. Request variables have a \"\$\" prefix: \$m get the m var from the request, \$0 get the first \"p\" entry, \$1 the second entry, etc. For static values just leave out the \"\$\".");
    $this->data->addField("filter_operator_".$c, "list", "all", "reload", 0, 25, "", "Filter ".$c);
    $this->data->addField("filter_value_".$c, "text", "all", "reload", 0, 255, "", "Value");
    $this->data->addField("filter_type_".$c, "enum", "all", "reload", false, false, "Value", "Value");
    $this->data->enumOptions("filter_type_".$c, "", "Value", "Datetime", "Request", "Custom");
   }


   $this->data->field["aggregation_field"]["type"]="list";
   $this->data->field["aggregation_field"]["properties"]="reload";
   $this->data->field["aggregation_field"]["use"]="sql;show;form";
   $this->data->field["aggregation_type"]["properties"]="";
   $this->data->field["aggregation_type"]["use"]="sql;show;form";
   $this->data->field["aggregation_method"]["properties"]="";
   $this->data->field["aggregation_method"]["use"]="sql;show;form";
   $this->data->field["aggregation_interval"]["properties"]="reload";
   $this->data->field["aggregation_interval"]["use"]="sql;show;form";



   $this->data->addField("orderby1", "list", "all", "reload", false, 255, false, "Sort field 1");
   $this->data->addField("orderbysort1", "enum", "all", "", false, false, "Asc", "Sort field 1");
   $this->data->enumOptions("orderbysort1", "Asc", "Desc");

   $this->data->addField("orderby2", "list", "all", "reload", false, 255, false, "Sort field 2");
   $this->data->addField("orderbysort2", "enum", "all", "", false, false, "Asc", "Sort field 2");
   $this->data->enumOptions("orderbysort2", "Asc", "Desc");

   $this->data->addField("orderby3", "list", "all", "reload", false, 255, false, "Sort field 3");
   $this->data->addField("orderbysort3", "enum", "all", "", false, false, "Asc", "Sort field 3");
   $this->data->enumOptions("orderbysort3", "Asc", "Desc");


   $this->data->addField("data", "data", "", "", false, false, "", "Data set");

   $this->data->addField("showFields", "multilist", "", "", false, false, "", "Show fields");

   $this->data->addField("dataset_show_from", "nr", "form", "hidden;control", 0, false, 0); //start showing item from here
   $this->data->addField("dataset_show_total", "nr", "form", "hidden;control", 0, 100, 0); //number of items per page, 0 to show all


   $fields=Array("extra");
   for($c=1;$c<=$this->filters;$c++)
   {
    $fields[]="filter_field_".$c;
    $fields[]="filter_operator_".$c;
    $fields[]="filter_value_".$c;
    $fields[]="filter_type_".$c;
   }

   $this->data->group("Filters", $fields);
   $this->data->group("Aggregation", "aggregation_field", "aggregation_type", "aggregation_method", "aggregation_interval");
   $this->data->group("Sorting", "orderby1", "orderbysort1", "orderby2", "orderbysort2", "orderby3", "orderbysort3");

   $this->showFields=false;
  }


  function viewPostGet()
  {
   if(!$this->customizeFields())
    return false;

   if(!$this->setDataSet(false))
    return false;

   return true;
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   global $_datasetInitting;

   //dataset init endless loop protection
   if($_datasetInitting)
    return true;

   $_datasetInitting=true;

   //valid module set
   if(($module=$this->data->getValue("module"))&&!is_array($module)&&$module!=$this->data->module)
   {
    $this->data->resetField("count");
    $this->data->resetField("extra");

    //init module
    $d=Data::cache($module, "view", strtolower($this->data->getValue("extra")));

    //get valid fields
    $fields=Array("" => "");
    foreach(array_keys($d->field) as $k)
     if($d->check("use", "sql", $k)||$d->check("use", "show", $k))
      if($k!="r")
       $fields[$k]=$d->field[$k]["title"];

    asort($fields);

    //get filter fields
    if($d->extra=="item")
     $filterFields=Array("" => "", "nr" => $d->title("nr"));
    else if($d->extra=="list")
     $filterFields=Array("" => "", "catnr" => $d->title("catnr"));
    else
    {
     $filterFields=Array("" => "");
     foreach(array_keys($d->field) as $field)
      if($d->check("use", "sql", $field))
       $filterFields[$field]=$d->title($field);

     asort($filterFields);
    }

    $this->data->listOptions("showFields", array_keys($d->field));

    //loop through filters
    for($c=1;$c<=$this->filters;$c++)
    {
     if($c>1&&(!$this->data->getvalue("filter_field_".($c-1))||$d->extra=="list")) //only show set filters and next empty one (show only one for list view)
     {
      $this->data->field["filter_field_".$c]["properties"].=";hidden";
      $this->data->field["filter_operator_".$c]["properties"].=";hidden";
      $this->data->field["filter_value_".$c]["properties"].=";hidden";
      $this->data->field["filter_type_".$c]["properties"].=";hidden";

      $this->data->listOptions("filter_operator_".$c, $this->data->fieldOperators["default"]);
     }
     else //show filter fields
     {
      $this->data->resetField("filter_field_".$c);
      $this->data->listOptions("filter_field_".$c, $filterFields);

      //valid filter field set
      if(($field=$this->data->getValue("filter_field_".$c))&&isset($filterFields[$field]))
      {
       $this->data->resetField("filter_operator_".$c);
       $this->data->resetField("filter_value_".$c);
       $this->data->resetField("filter_type_".$c);

       //set operators
       if($d->extra=="item"||$d->extra=="list")
        $this->data->listOptions("filter_operator_".$c, Array("="));
       else if(isset($this->data->fieldOperators[$d->field[$field]["type"]]))
        $this->data->listOptions("filter_operator_".$c, $this->data->fieldOperators[$d->field[$field]["type"]]);
       else
        $this->data->listOptions("filter_operator_".$c, $this->data->fieldOperators["default"]);

       //set filter types
       if($d->field[$field]["type"]=="date"||$d->field[$field]["type"]=="datetime"||$d->field[$field]["type"]=="time") //date/time
        $this->data->enumOptions("filter_type_".$c, "Datetime", "Request", "Custom");
       else //other fields
        $this->data->enumOptions("filter_type_".$c, "Value", "Request", "Custom");

       //ensure only valid filter types are used
       if(!in_array($this->data->getValue("filter_type_".$c), $this->data->listOptions["filter_type_".$c]))
        $this->data->setValue("filter_type_".$c, $this->data->listOptions["filter_type_".$c][0]); //invalid type, use first (default)

       //set field properties per filter type
       if($this->data->getValue("filter_type_".$c)=="Value")
       {
        //set enum/list options
        if(isset($d->listOptions[$field]))
         $this->data->listOptions["filter_value_".$c]=$d->listOptions[$field];
        else
         $this->data->listOptions["filter_value_".$c]=Array();

        if(isset($d->listOptions[$field]))
         $this->data->listOptions["filter_value_".$c]=$d->listOptions[$field];
        else
         $this->data->listOptions["filter_value_".$c]=Array();

        //set field properties
        $this->data->field["filter_value_".$c]["type"]=$d->field[$field]["type"];
        $this->data->field["filter_value_".$c]["min"]=$d->field[$field]["min"];
        $this->data->field["filter_value_".$c]["max"]=$d->field[$field]["max"];
        $this->data->field["filter_value_".$c]["defaultvalue"]=$d->field[$field]["defaultvalue"];
        $this->data->field["filter_value_".$c]["format"]=$d->field[$field]["format"];

        if($d->getID()==$field) //id field, generate pick list
         $this->data->addReference("filter_value_".$c, $d->module, $field, "*:1", "filter_value_".$c);
        else //non id field
        {
         //see if field matches a reference
         foreach(array_keys($d->reference) as $ref)
          if($d->reference[$ref]["from"]==$field) //generate pick list
           $this->data->addReference("filter_value_".$c, $d->reference[$ref]["module"], $d->reference[$ref]["to"], $d->reference[$ref]["type"], "filter_value_".$c);
        }
       }
       else if($this->data->getValue("filter_type_".$c)=="Datetime")
       {
        $this->data->field["filter_value_".$c]["type"]="text";
        $this->data->field["filter_value_".$c]["min"]=1;
        $this->data->field["filter_value_".$c]["max"]=25;

        if($d->field[$field]["type"]=="date")
         $this->data->field["filter_value_".$c]["defaultvalue"]=$d->field[$field]["format"][$d->dataformat]["dateformat"];
        else if($d->field[$field]["type"]=="datetime")
         $this->data->field["filter_value_".$c]["defaultvalue"]=$d->field[$field]["format"][$d->dataformat]["datetimeformat"];
        else if($d->field[$field]["type"]=="time")
         $this->data->field["filter_value_".$c]["defaultvalue"]=$d->field[$field]["format"][$d->dataformat]["timeformat"];

        //unset value if it's the default value of filter type
        if($this->data->getValue("filter_value_".$c)=="variable"||$this->data->getValue("filter_value_".$c)=="value"||(isset($d->field[$field]["format"][$d->dataformat])&&($this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["dateformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["datetimeformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["timeformat"])))
         $this->data->setValue("filter_value_".$c, "");
       }
       else if($this->data->getValue("filter_type_".$c)=="Request")
       {
        $this->data->field["filter_value_".$c]["type"]="title";
        $this->data->field["filter_value_".$c]["min"]=1;
        $this->data->field["filter_value_".$c]["max"]=50;
        $this->data->field["filter_value_".$c]["defaultvalue"]="variable";
        $this->data->field["filter_value_".$c]["format"]=$d->field[$field]["format"];

        //unset value if it's the default value of filter type
        if($this->data->getValue("filter_value_".$c)=="variable"||$this->data->getValue("filter_value_".$c)=="value"||(isset($d->field[$field]["format"][$d->dataformat])&&($this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["dateformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["datetimeformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["timeformat"])))
         $this->data->setValue("filter_value_".$c, "");
       }
       else if($this->data->getValue("filter_type_".$c)=="Custom")
       {
        $this->data->field["filter_value_".$c]["type"]="text";
        $this->data->field["filter_value_".$c]["min"]=0;
        $this->data->field["filter_value_".$c]["max"]=255;
        $this->data->field["filter_value_".$c]["defaultvalue"]="value";
        $this->data->field["filter_value_".$c]["format"]=$d->field[$field]["format"];

        //unset value if it's the default value of filter type
        if($this->data->getValue("filter_value_".$c)=="variable"||$this->data->getValue("filter_value_".$c)=="value"||(isset($d->field[$field]["format"][$d->dataformat])&&($this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["dateformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["datetimeformat"]||$this->data->getValue("filter_value_".$c)==$d->field[$field]["format"][$d->dataformat]["timeformat"])))
         $this->data->setValue("filter_value_".$c, "");
       }

       //set default value if nothing is set
       if(!$this->data->getValue("filter_value_".$c))
        $this->data->setDefaultValue("filter_value_".$c);
      }
      else //no field selected, don't show other fields
      {
       $this->data->field["filter_operator_".$c]["properties"].=";hidden";
       $this->data->field["filter_value_".$c]["properties"].=";hidden";
       $this->data->field["filter_type_".$c]["properties"].=";hidden";

       $this->data->listOptions("filter_operator_".$c, $this->data->fieldOperators["default"]);
      }
     }
    }


    //Aggregation
    $this->data->field["aggregation_field"]["properties"]=preg_replace("/hidden;?/", "", $this->data->field["aggregation_field"]["properties"]); //don't use resetField, aggregation_field is hidden by default
    $this->data->listOptions("aggregation_field", array_merge(Array("" => ""), $fields));

    if(($aggregation_field=$this->data->getValue("aggregation_field"))&&isset($fields[$aggregation_field]))
    {
     $this->data->field["aggregation_type"]["properties"]=preg_replace("/hidden;?/", "", $this->data->field["aggregation_type"]["properties"]); //don't use resetField, aggregation_type is hidden by default
     $this->data->field["aggregation_method"]["properties"]=preg_replace("/hidden;?/", "", $this->data->field["aggregation_method"]["properties"]); //don't use resetField, aggregation_method is hidden by default
     $this->data->field["aggregation_interval"]["properties"]=preg_replace("/hidden;?/", "", $this->data->field["aggregation_interval"]["properties"]); //don't use resetField, aggregation_interval is hidden by default

     if($d->field[$aggregation_field]["type"]=="date"||$d->field[$aggregation_field]["type"]=="time"||$d->field[$aggregation_field]["type"]=="datetime")
      $this->data->listOptions("aggregation_interval", Array("" => "", 60 => "Minute", 1800 => "Half hour", 3600 => "Hour", "43200" => "Half day", "86400" => "Day", "604800" => "Week", "2592000" => "Month", "31536000" => "Year"));

     if(!$this->data->getValue("orderby1")) //set sort aggregation_field
      $this->data->setValue("orderby1", $aggregation_field);
    }
    else
    {
     $this->data->field["aggregation_type"]["properties"].=";hidden";
     $this->data->field["aggregation_method"]["properties"].=";hidden";
     $this->data->field["aggregation_interval"]["properties"].=";hidden";
    }


    //Sorting
    $this->data->resetField("orderby1");
    $this->data->listOptions("orderby1", $fields);
    $this->data->listOptions("orderby2", $fields);
    $this->data->listOptions("orderby3", $fields);

    if(!$this->data->getValue("orderby1"))
    {
     $this->data->field["orderbysort1"]["properties"].=";hidden";

     $this->data->field["orderby2"]["properties"].=";hidden";
     $this->data->field["orderbysort2"]["properties"].=";hidden";
     $this->data->field["orderby3"]["properties"].=";hidden";
     $this->data->field["orderbysort3"]["properties"].=";hidden";
    }
    else
    {
     $this->data->resetField("orderbysort1");

     if(!$this->data->getValue("orderby2"))
     {
      $this->data->field["orderbysort2"]["properties"].=";hidden";

      $this->data->field["orderby3"]["properties"].=";hidden";
      $this->data->field["orderbysort3"]["properties"].=";hidden";
     }
     else
     {
      $this->data->resetField("orderbysort2");

      if(!$this->data->getValue("orderby3"))
       $this->data->field["orderbysort3"]["properties"].=";hidden";
      else
       $this->data->resetField("orderbysort3");
     }
    }

    //get count value
    if($this->data->action=="add"||$this->data->action=="edit")
     $this->setDataSet(true);
   }
   else
   {
    $this->data->field["count"]["properties"].=";hidden";
    $this->data->field["extra"]["properties"].=";hidden";

    for($c=1;$c<=$this->filters;$c++)
    {
     $this->data->field["filter_field_".$c]["properties"].=";hidden";
     $this->data->field["filter_operator_".$c]["properties"].=";hidden";
     $this->data->field["filter_value_".$c]["properties"].=";hidden";
     $this->data->field["filter_type_".$c]["properties"].=";hidden";
    }

    $this->data->field["orderby1"]["properties"].=";hidden";
    $this->data->field["orderbysort1"]["properties"].=";hidden";
    $this->data->field["orderby2"]["properties"].=";hidden";
    $this->data->field["orderbysort2"]["properties"].=";hidden";
    $this->data->field["orderby3"]["properties"].=";hidden";
    $this->data->field["orderbysort3"]["properties"].=";hidden";

    $this->data->field["aggregation_field"]["properties"].=";hidden";
    $this->data->field["aggregation_type"]["properties"].=";hidden";
    $this->data->field["aggregation_method"]["properties"].=";hidden";
    $this->data->field["aggregation_interval"]["properties"].=";hidden";
   }

   $_datasetInitting=false;
   return true;
  }


  function viewPreGet(){return $this->prepareShowFields();}
  function copyPreGet(){return $this->prepareShowFields();}
  function editPretGet(){return $this->prepareShowFields();}
  function movePreGet(){return $this->prepareShowFields();}
  function deletePreGet(){return $this->prepareShowFields();}

  function prepareShowFields()
  {
   if(!$this->data->check("use", "show", "data"))
    return true;

   if($this->data->check("valuetype", "set", "showFields"))
    $this->showFields=$this->data->getValue("showFields");

   if(!$this->data->check("use", "show", "module"))
    $this->data->field["module"]["use"].=";show";

   if(!$this->data->check("use", "show", "extra"))
    $this->data->field["extra"]["use"].=";show";

   for($c=1;$c<=$this->filters;$c++)
   {
    if(!$this->data->check("use", "show", "filter_field_".$c))
     $this->data->field["filter_field_".$c]["use"].=";show";
    if(!$this->data->check("use", "show", "filter_operator_".$c))
     $this->data->field["filter_operator_".$c]["use"].=";show";
    if(!$this->data->check("use", "show", "filter_value_".$c))
     $this->data->field["filter_value_".$c]["use"].=";show";
    if(!$this->data->check("use", "show", "filter_type".$c))
     $this->data->field["filter_type_".$c]["use"].=";show";
   }

   if(!$this->data->check("use", "show", "orderby1"))
    $this->data->field["orderby1"]["use"].=";show";
   if(!$this->data->check("use", "show", "orderby2"))
    $this->data->field["orderby2"]["use"].=";show";
   if(!$this->data->check("use", "show", "orderby3"))
    $this->data->field["orderby3"]["use"].=";show";

   if(!$this->data->check("use", "show", "orderbysort1"))
    $this->data->field["orderbysort1"]["use"].=";show";
   if(!$this->data->check("use", "show", "orderbysort2"))
    $this->data->field["orderbysort2"]["use"].=";show";
   if(!$this->data->check("use", "show", "orderbysort3"))
    $this->data->field["orderbysort3"]["use"].=";show";

   if(!$this->data->check("use", "show", "aggregation_field"))
    $this->data->field["aggregation_field"]["use"].=";show";
   if(!$this->data->check("use", "show", "aggregation_type"))
    $this->data->field["aggregation_type"]["use"].=";show";
   if(!$this->data->check("use", "show", "aggregation_method"))
    $this->data->field["aggregation_method"]["use"].=";show";
   if(!$this->data->check("use", "show", "aggregation_interval"))
    $this->data->field["aggregation_interval"]["use"].=";show";

   return true;
  }

  function &getDataObject($vc=0)
  {
   if($vc<0||$vc>=$this->data->vmax)
   {
    $this->data->e->add("No data object for entry: \"".$vc."\"", "error");
    return false;
   }

   $d=Data::cache($this->data->getValue("module", $vc), "view", strtolower($this->data->getValue("extra", $vc)));

   if($this->showFields)
    $d->setShowFields($this->showFields);

   if($this->data->check("valuetype", "set", "dataset_show_from"))
    $d->setValue("show_from", $this->data->getValue("dataset_show_from"));

   $d->setValue("show_total", $this->data->getValue("dataset_show_total"));

   //Filters
   $c=1;
   $entry=0;
   while($this->data->getValue("filter_field_".$c, $vc))
   {
    if(!$this->data->getValue("filter_field_".$c, $vc))
     break;

    if($this->data->getValue("filter_type_".$c, $vc)=="Value")
    {
     $d->setValue($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_value_".$c, $vc), $entry);
     $d->setOperator($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_operator_".$c, $vc), $entry);
     $entry++;
    }
    else if($this->data->getValue("filter_type_".$c, $vc)=="Datetime")
    {
     //convert value with date (http://php.net/date) to proper date
     $d->setValue($this->data->getValue("filter_field_".$c, $vc), date($this->data->getValue("filter_value_".$c, $vc)), $entry);
     $d->setOperator($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_operator_".$c, $vc), $entry);
     $entry++;
    }
    else if($this->data->getValue("filter_type_".$c, $vc)=="Request")
    {
     //get request variable value
     $var=$this->data->getValue("filter_value_".$c, $vc);

     if(preg_match("/^\d+$/", $var)) //numeric, page var (p[0], p[1], etc)
     {
      if($var=request("p", $var)) //only set if request value is set
      {
       $d->setValue($this->data->getValue("filter_field_".$c, $vc), $var, $entry);
       $d->setOperator($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_operator_".$c, $vc), $entry);
       $entry++;
      }
     }
     else //normal request var
     {
      if($var=request($var)) //only set if request value is set
      {
       $d->setValue($this->data->getValue("filter_field_".$c, $vc), $var, $entry);
       $d->setOperator($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_operator_".$c, $vc), $entry);
       $entry++;
      }
     }
    }
    else if($this->data->getValue("filter_type_".$c, $vc)=="Custom")
    {
     //set custom value
     $d->setValue($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_value_".$c, $vc), $entry);
     $d->setOperator($this->data->getValue("filter_field_".$c, $vc), $this->data->getValue("filter_operator_".$c, $vc), $entry);
     $entry++;
    }

    $c++;
   }


   //Aggregation
   if($this->data->getValue("aggregation_field", $vc))
   {
    $d->setValue("aggregation_field", $this->data->getValue("aggregation_field", $vc));
    $d->setValue("aggregation_type", $this->data->getValue("aggregation_type", $vc));
    $d->setValue("aggregation_method", $this->data->getValue("aggregation_method", $vc));
    $d->setValue("aggregation_interval", $this->data->getValue("aggregation_interval", $vc));
   }


   //Sorting
   if($this->data->getValue("orderby1", $vc))
   {
    $orderby=Array($this->data->getValue("orderby1", $vc).($this->data->getValue("orderbysort1", $vc)=="Asc"?"+":"-"));

    if($this->data->getValue("orderby2", $vc))
    {
     $orderby[]=$this->data->getValue("orderby2", $vc).($this->data->getValue("orderbysort2", $vc)=="Asc"?"+":"-");

     if($this->data->getValue("orderby3", $vc))
      $orderby[]=$this->data->getValue("orderby3", $vc).($this->data->getValue("orderbysort3", $vc)=="Asc"?"+":"-");
    }

    $d->sortOrder($orderby);
   }

   return $d;
  }

  function setDataSet($count)
  {
   if(!$count&&!$this->data->check("use", "show", "data"))
    return true;

   for($vc=0;$vc<$this->data->vmax;$vc++)
   {
    $d=&$this->getDataObject($vc);

    if($count)
    {
     if(!$d->get(false, true))
     {
      $this->data->e->add($d->e);
      return false;
     }

     $this->data->field["count"]["value"][$vc]=$d->getValue("show_count");
    }
    else
    {
     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     $this->data->field["data"]["value"][$vc]=&$d;
    }
   }

   return true;
  }


  function addPreSubmitErrorCheck()
  {
   for($this->data->vc=0;$this->data->vc<$this->data->vmax;$this->data->vc++)
   {
    if(!$this->data->getValue("name", $this->data->vc))
    {
     $name=$this->data->getValue("module", $this->data->vc);

     for($c=1;$c<=$this->filters;$c++)
     {
      if($this->data->getValue("filter_field_", $this->data->vc))
       $name.="_".$this->data->getValue("filter_field_", $this->data->vc);
      else
       break;
     }

     $this->data->setValue("name", $name, $this->data->vc);
    }
   }

   return true;
  }

 }
?>
