<?

 /*****************************************************************************
 * Filename: include/mp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."socket.php");

 declare(ticks=1); //ensure signals are handled immediately

 class MP
 {
  function __construct()
  {
   $this->e=new Errors();
   $this->e->l->read("fs");

   $this->messageBusDir="/tmp/";
   $this->messageBusPrefix="jas_bus-"; //just add PID

   $this->clear();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  function clear()
  {
   $this->pid=posix_getpid();

   $this->parent=false;
   $this->childs=Array();

   if(isset($this->sockets)) //disconnected all message busses
    $this->close();

   $this->sockets=Array();


   if(isset($this->signalHandlers)) //unset all signal handlers
    $this->unsetSignals();

   $this->signalHandlers=Array();
  }


  function convertSignal($signal)
  {
   if(!defined("SIGHUP"))
   {
    $this->e->add("Signals not available", "error");
    return false;
   }

   if(is_string($signal))
   {
    switch(strtolower($signal))
    {
     case "hup";
      return SIGHUP;
     case "int";
     case "interrupt";
      return SIGINT;
     case "quit";
      return SIGQUIT;
     case "ill";
     case "illegal";
      return SIGILL;
     case "abrt";
     case "abort";
      return SIGABRT;
     case "fpe";
      return SIGFPE;
     case "kill";
      return SIGKILL;
     case "segv";
     case "segfault";
      return SIGSEGV;
     case "pipe";
      return SIGPIPE;
     case "alrm";
     case "alarm";
      return SIGALRM;
     case "term";
     case "terminate";
      return SIGTERM;
     case "usr";
     case "usr1";
     case "user";
     case "user1";
      return SIGUSR1;
     case "usr2";
     case "user2";
      return SIGUSR2;
     case "chld";
     case "child";
      return SIGCHLD;
     case "cont";
     case "continue";
      return SIGCONT;
     case "stop";
      return SIGSTOP;
    }
   }
   else
   {
    switch($signal)
    {
     case SIGHUP;
      return "hup";
     case SIGINT;
      return "int";
     case SIGQUIT;
      return "quit";
     case SIGILL;
      return "ill";
     case SIGABRT;
      return "abrt";
     case SIGFPE;
      return "fpe";
     case SIGKILL;
      return "kill";
     case SIGSEGV;
      return "segv";
     case SIGPIPE;
      return "pipe";
     case SIGALRM;
      return "alrm";
     case SIGTERM;
      return "term";
     case SIGUSR1;
      return "usr1";
     case SIGUSR2;
      return "usr2";
     case SIGCHLD;
      return "chld";
     case SIGCONT;
      return "cont";
     case SIGSTOP;
      return "stop";
    }
   }

   $this->e->add("Unknown signal: ".$signal, "system");
   return false;
  }

  function setSignal($signal, &$object=NULL, $method=NULL, $arguments=NULL)
  {
   if(!function_exists("pcntl_signal"))
   {
    $this->e->add("Signals not available", "error");
    return false;
   }

   if($object!==NULL)
   {
    if(!method_exists($object, $method))
    {
     $this->e->add("Signal handler: ".get_class($object)."::".$method." doesn't exist", "system");
     return false;
    }
   }

   if(is_string($signal))
    if(($signal=$this->convertSignal($signal))===false)
     return false;

   //set signal to call this->handleSignal
   if(!@pcntl_signal($signal, Array(&$this, "handleSignal")))
   {
    $error=pcntl_get_last_error();
    $error="[".$error."] ".pcntl_strerror($error);
    $this->e->add("Unable to set signal", "system", $error);
    return false;
   }

   //add signal handlers
   if(!isset($this->signalHandlers[$signal]))
    $this->signalHandlers[$signal]=Array();

   if($arguments!==NULL)
   {
    if(!is_array($arguments))
    {
     $arg=&$arguments;
     unset($arguments);
     $arguments=Array(&$arg);
    }
   }

   if($object!==NULL) //else catch signal, but don't handle it
   {
    $this->signalHandlers[$signal][]=Array(
     "object" => &$object,
     "method" => $method,
     "arguments" => &$arguments,
    );
   }

   return true;
  }

  function unsetSignal($signal, $ignore=false)
  {
   if(!function_exists("pcntl_signal"))
   {
    $this->e->add("Signals not available", "error");
    return false;
   }

   if(is_string($signal))
    if(($signal=$this->convertSignal($signal))===false)
     return false;

   //unset/ignore signal
   if($ignore)
    $handler=SIG_IGN;
   else
    $handler=SIG_DFL;

   if(!@pcntl_signal($signal, $handler))
   {
    $error=pcntl_get_last_error();
    $error="[".$error."] ".pcntl_strerror($error);
    $this->e->add("Unable to unset signal", "system", $error);
    return false;
   }

   //remove signal handlers from list
   unset($this->signalHandlers[$signal]);
   return true;
  }

  function unsetSignals()
  {
   //unset all signal handlers
   $success=true;
   foreach(array_keys($this->signalHandlers) as $signal)
    if(!$this->unsetSignal($signal))
     $success=false;

   return $success;
  }

  function handleSignal($signal)
  {
   if(!isset($this->signalHandlers[$signal]))
    return;

   //call all handlers for this signal
   foreach($this->signalHandlers[$signal] as $handler)
   {
    if($handler["arguments"])
     call_user_func_array(Array(&$handler["object"], $handler["method"]), $handler["arguments"]);
    else
     call_user_func(Array(&$handler["object"], $handler["method"]));
   }
  }

  function sendSignal($signal, $target=false) //self, all, childs, parent, child #
  {
   if(!function_exists("posix_kill"))
   {
    $this->e->add("Send signals not available", "error");
    return false;
   }

   if(is_string($signal))
    if(($signal=$this->convertSignal($signal))===false)
     return false;

   if($target===false)
    $target="self";

   if(is_int($target)) //child #
   {
    if(!isset($this->childs[$target]))
    {
     $this->e->add("Send signal failed, unknown process # ".$target, "error");
     return false;
    }

    $pid=$this->childs[$target];
   }
   else if($target=="all"||$target=="childs")
   {
    $success=true;

    if($target=="all") //include self and parent
    {
     if(!$this->sendSignal($signal, "self"))
      $success=false;

     if($this->parent)
      if(!$this->sendSignal($signal, "parent"))
       $success=false;
    }

    foreach(array_keys($this->childs) as $child)
     if(!$this->sendSignal($signal, $child))
      $success=false;

    return $success;
   }
   else if($target=="parent")
   {
    if(!$this->parent)
    {
     $this->e->add("Send signal failed, no parent", "error");
     return false;
    }

    $pid=$this->parent;
   }
   else if($target=="self")
   {
    $pid=$this->pid;
   }
   else
   {
    $this->e->add("Send signal failed, unknown target: ".$target, "error");
    return false;
   }


   //send signal
   if(!@posix_kill($pid, $signal))
   {
    $error=posix_get_last_error();
    $error="[".$error."] ".posix_strerror($error);
    $this->e->add("Unable to send signal", "system", $error);
    return false;
   }

   return true;
  }


  function fork($messageBus=false)
  {
   if(!function_exists("pcntl_fork"))
   {
    $this->e->add("Fork not available", "error");
    return false;
   }

   if($messageBus)
   {
    $socket=new Socket();
    $messageBus=$this->messageBusDir.$this->messageBusPrefix;
   }

   $pid=pcntl_fork();
   if($pid==-1) //fail
   {
    $this->e->add("Unable to fork", "error");
    return false;
   }
   else if($pid) //parent
   {
    $process="parent";
    $this->childs[]=$pid;

    if($messageBus) //setup message bus
    {
     $messageBus.=$pid; //use child's message bus

     for($c=0;$c<10;$c++) //10 retries to connect to childs
     {
      if(!file_exists($messageBus)||!$socket->connect("file://".$messageBus))
       usleep(10); //wait 10 microseconds for parent process to set up socket
      else
       break;
     }

     if(!$socket->socket)
     {
      $this->e->add("Unable to connect to child's message bus: \"".$messageBus."\"", "error");
      return false;
     }

     $this->sockets[$pid]=&$socket; //store connected socket with child's pid
     $socket->e->clear(); //clear any errors that may have occurred
    }
   }
   else //child
   {
    $process="child";
    $this->parent=$this->pid; //store parent's pid
    $this->pid=posix_getpid(); //set new pid

    //clear previously created childs & sockets
    $this->childs=Array();
    $this->sockets=Array();

    if($messageBus) //setup message bus
    {
     $messageBus.=$this->pid; //child pid

     if(!$socket->listen("file://".$messageBus))
     {
      $this->e->add("Unable to set up message bus: \"".$messageBus."\"", "error");
      $this->e->add($socket->e);
      return false;
     }

     if(!($socket2=$socket->accept(false))) //single client
     {
      $this->e->add("Unable to set up message bus: \"".$messageBus."\"", "error");
      $this->e->add($socket->e);
      return false;
     }

     $socket->close(); //close listen bus

     $this->sockets[$this->pid]=&$socket2; //store connected socket with child's pid
     $socket2->e->clear(); //clear any errors that may have occurred
    }
   }

   return $process;
  }

  function read($source=false) //all, childs, parent, child #
  {
   if(!$this->childs&&!$this->parent)
   {
    $this->e->add("Read failed, no childs or parents", "error");
    return false;
   }

   if($source===false)
    $source="all";

   if(is_int($source)) //child #
   {
    if(!isset($this->childs[$source]))
    {
     $this->e->add("Read failed, unknown process # ".$source, "error");
     return false;
    }

    $pid=$this->childs[$source];
   }
   else if($source=="all"||$source=="childs")
   {
    $data=Array();

    if($source=="all"&&$this->parent) //include parent
     $data["parent"]=$this->read("parent");

    foreach($this->childs as $child => $pid)
     if(isset($this->sockets[$pid]))
      $data[$child]=$this->read($child);

    return $data;
   }
   else if($source=="parent")
   {
    if(!$this->parent)
    {
     $this->e->add("Read failed, no parent", "error");
     return false;
    }

    $pid=$this->pid; //socket with parent has child's PID
   }
   else
   {
    $this->e->add("Read failed, unknown source: ".$source, "error");
    return false;
   }


   if(!isset($this->sockets[$pid]))
   {
    $this->e->add("Write failed, no message bus for: ".$source, "error");
    return false;
   }

   if(($data=$this->sockets[$pid]->read())===false)
   {
    $this->e->add($this->sockets[$pid]->e);
    return false;
   }

   return $data;
  }

  function write($data, $target=false) //all, childs, parent, child #
  {
   if(!$this->childs&&!$this->parent)
   {
    $this->e->add("Write failed, no childs or parents", "error");
    return false;
   }

   if($target===false)
    $target="all";

   if(is_int($target)) //child #
   {
    if(!isset($this->childs[$target]))
    {
     $this->e->add("Write failed, unknown process # ".$target, "error");
     return false;
    }

    $pid=$this->childs[$target];
   }
   else if($target=="all"||$target=="childs")
   {
    $success=true;

    if($target=="all"&&$this->parent) //include parent
     if(!$this->write($data, "parent"))
      $success=false;

    foreach(array_keys($this->childs) as $child)
     if(!$this->write($data, $child))
      $success=false;

    return $success;
   }
   else if($target=="parent")
   {
    if(!$this->parent)
    {
     $this->e->add("Write failed, no parent", "error");
     return false;
    }

    $pid=$this->pid; //socket with parent has child's PID
   }
   else
   {
    $this->e->add("Write failed, unknown target: ".$target, "error");
    return false;
   }


   if(!isset($this->sockets[$pid]))
   {
    $this->e->add("Write failed, no data bus for: ".$target, "error");
    return false;
   }

   if($this->sockets[$pid]->write($data)===false)
   {
    $this->e->add($this->sockets[$pid]->e);
    return false;
   }

   return true;
  }

  function close($socket=false) //all, childs, parent, child #
  {
   if(!$this->childs&&!$this->parent)
   {
    $this->e->add("Close failed, no childs or parents", "error");
    return false;
   }

   if($socket===false)
    $socket="all";

   if(is_int($socket))//child #
   {
    if(!isset($this->childs[$socket]))
    {
     $this->e->add("Close failed, unknown process # ".$socket, "error");
     return false;
    }

    $pid=$this->childs[$socket];
   }
   else if($socket=="all"||$socket=="childs")
   {
    $success=true;

    if($socket=="all"&&$this->parent) //include parent
     if(!$this->close("parent"))
      $success=false;

    foreach(array_keys($this->childs) as $child)
    {
     if(!$this->close($child))
      $success=false;
    }

    return $success;
   }
   else if($socket=="parent")
   {
    if(!$this->parent)
    {
     $this->e->add("Close failed, no parent", "error");
     return false;
    }

    $pid=$this->pid; //socket with parent has child's PID
   }
   else
   {
    $this->e->add("Close failed, unknown socket: ".$socket, "error");
    return false;
   }


   if(!isset($this->sockets[$pid]))
   {
    $this->e->add("Close failed, no message bus for: ".$socket, "error");
    return false;
   }

   if($this->sockets[$pid]->close()===false)
   {
    $this->e->add($this->sockets[$pid]->e);
    unset($this->sockets[$pid]);
    return false;
   }

   unset($this->sockets[$pid]);
   return true;
  }


  function wait($process=false) //childs, child #
  {
   if(!$this->childs)
   {
    $this->e->add("Wait failed, no childs", "error");
    return false;
   }

   if($process===false)
    $process="childs";

   if(is_int($process)) //child #
   {
    if(!isset($this->childs[$process]))
    {
     $this->e->add("Wait failed, unknown process # ".$process, "error");
     return false;
    }

    $pid=$this->childs[$process];
   }
   else if($process=="childs")
   {
    $data=Array();

    foreach(array_keys($this->childs) as $child)
     $data[$child]=$this->wait($child);

    return $data;
   }
   else
   {
    $this->e->add("Wait failed, unknown process: ".$process, "error");
    return false;
   }


   $status=false;
   if(@pcntl_waitpid($pid, $status)<0)
   {
    $error=pcntl_get_last_error();
    $error="[".$error."] ".pcntl_strerror($error);
    $this->e->add("Waiting for process: ".$process." failed", "system", $error);
    return false;
   }

   //close socket
   if(isset($this->sockets[$pid]))
   {
    $this->sockets[$pid]->close();
    unset($this->sockets[$pid]);
   }

   unset($this->childs[$process]);
   return pcntl_wexitstatus($status);
  }


  function __destruct()
  {
   $this->close();
   $this->unsetSignals();
  }

 }

?>
