<?

 /*****************************************************************************
 * Filename: include/mutilate.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."language.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["format"]."xml.php");

 class Mutilate extends DT
 {
  function __construct()
  {
   parent::__construct();
  }

  static function __callStatic($method, $arguments)
  {
   $object=new self();
   $object->e->logging(false); //disable logging for static calls

   if(substr($method, -1)=="S") //method must have a S suffix
   {
    $method=substr($method, 0, -1); //remove S suffix
    if(method_exists($object, $method)) //check if dynamic method exists
     return call_user_func_array(Array($object, $method), $arguments);
   }

   $bt=debug_backtrace();
   print "<br/>\n<b>Fatal error</b>:  Call to undefined method ".get_class($object)."::".$method." in <b>".$bt[0]["file"]."</b> on line <b>".$bt[0]["line"]."</b><br />\n";
   exit;
  }


//NUMBERS

  function humanReadableNumber($number, $decimals=false, $decimalseparator=false)
  {
   //array values
   if(is_array($number))
   {
    foreach($number as $k => $v)
     $number[$k]=$this->humanReadableNumber($v, $decimals, $decimalseparator);

    return $number;
   }

   if($decimals===false)
    $decimals=0;

   if(-1000000000000000>$number||$number>1000000000000000)
   {
    $number=round($number/1000000000000000, $decimals);
    $suffix="P";
   }
   else if(-1000000000000>$number||$number>1000000000000)
   {
    $number=round($number/1000000000000, $decimals);
    $suffix="T";
   }
   else if(-1000000000>$number||$number>1000000000)
   {
    $number=round($number/1000000000, $decimals);
    $suffix="G";
   }
   else if(-1000000>$number||$number>1000000)
   {
    $number=round($number/1000000, $decimals);
    $suffix="M";
   }
   else if(-1000>$number||$number>1000)
   {
    $number=round($number/1000, $decimals);
    $suffix="K";
   }
   else
   {
    $number=round($number, $decimals);
    $suffix="";
   }

   if($decimalseparator)
    $number=number_format($number, $decimals, $decimalseparator, " ");

   return $number.$suffix;
  }

  function humanReadableFileSize($size, $decimals=false, $decimalseparator=false)
  {
   //array values
   if(is_array($size))
   {
    foreach($size as $k => $v)
    {
     if(is_array($v))
      $size[$k]=$this->humanReadableFileSize($v, $decimals, $decimalseparator);
     else
      $size[$k]=$this->humanReadableNumber($v, $decimals, $decimalseparator)."B";
    }

    return $size;
   }

   return $this->humanReadableNumber($size, $decimals, $decimalseparator)."B";
  }

  function leadingZeros($number, $length)
  {
   //array values
   if(is_array($number))
   {
    foreach($number as $k => $v)
     $number[$k]=$this->leadingZeros($v, $length);

    return $number;
   }

   $zeros=$length-strlen($number);

   if($zeros<=0)
    return $number;

   return str_repeat("0", $zeros).$number;
  }


//STRINGS

  function strip($text, $length=false, $suffix=false)
  {
   if($length===false)
    $length=32;

   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->strip($v, $length, $suffix);

    return $text;
   }

   if(strlen($text)>$length)
   {
    if($suffix===false)
     $suffix="...";

    $length-=strlen($suffix);
   }
   else
    $suffix="";

   return substr($text, 0, $length).$suffix;
  }


  function replace($text, $pattern, $replacement=false)
  {
   if(!$replacement)
    $replacement="";

   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->replace($v, $pattern, $replacement);

    return $text;
   }

   return preg_replace($pattern, $replacement, $text);
  }


  function changeCase($text, $type)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->changeCase($v, $type);

    return $text;
   }

   switch(strtolower($type))
   {
    case "lower":
     return strtolower($text);

    case "upper":
     return strtoupper($text);

    case "first":
     return ucfirst(strtolower($text)); //only first character!

    case "words":
     return ucwords(strtolower($text));

    default: break;
   }

   return $text;
  }


  function trim($text, $chars=false, $type=false)
  {
   if(!$chars)
    $chars=" \t\n\r\0\x0B";

   if(!$type)
    $type="both";

   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->trim($v, $chars, $type);

    return $text;
   }

   switch($type)
   {
    case "left":
    case "begin":
    case "ltrim":
    {
     $text=ltrim($text, $chars);
     break;
    }
    case "right":
    case "end":
    case "rtrim":
    {
     $text=rtrim($text, $chars);
     break;
    }
    default:
    {
     $text=trim($text, $chars);
     break;
    }
   }

   return $text;
  }


  function detectLineBreaks($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->detectLineBreaks($v);

    return $text;
   }

   if(($pos=strrpos($text, "\n"))!==false||($pos=strrpos($text, "\r"))!==false) //find last linebreak
   {
    if(isset($text[$pos+1])&&$text[$pos+1]=="\r") //\n\r is found on \n
     $pos++; //set position to \r

    if($pos==0) //first character, no linebreak before this
     return $text[$pos];
    else if($text[$pos-1]=="\n"&&$text[$pos]=="\r") //previous character is different linebreak
     return "\n\r";
    else if($text[$pos-1]=="\r"&&$text[$pos]=="\n") //previous character is different linebreak
     return "\r\n";
    else //previous character is same linebreak
     return $text[$pos];
   }

   return false;
  }


  function fixLineBreaks($text, $newLineBreaks=false)
  {
   if($newLineBreaks===false)
    $newLineBreaks="\n";

   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->fixLineBreaks($v, $newLineBreaks);

    return $text;
   }

   if(!($oldLineBreaks=$this->detectLineBreaks($text))) //no linebreaks found
    return $text; //done

   if($oldLineBreaks!=$newLineBreaks)
   {
    switch($oldLineBreaks)
    {
     case "\r\n":
      $text=preg_replace("/\r\n/", $newLineBreaks, $text);
      break;

     case "\n\r":
      $text=preg_replace("/\n\r/", $newLineBreaks, $text);
      break;

     case "\n":
      $text=preg_replace("/\n([^\n\r]?)/", $newLineBreaks."\\1", $text);
     break;

     case "\r":
      $text=preg_replace("/\r([^\n\r]?)/", $newLineBreaks."\\1", $text);
     break;

     default: break;
    }
   }

   return $text;
  }


//HTML

  function stripTags($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->stripTags($v);

    return $text;
   }

   return strip_tags($text);
  }

  function htmlEncode($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->htmlEncode($v);

    return $text;
   }

   return htmlentities($text, ENT_COMPAT|ENT_XHTML);
  }

  function htmlDecode($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->htmlDecode($v);

    return $text;
   }

   return html_entity_decode($text, ENT_QUOTES|ENT_XHTML);
  }


//MARKUP

  function _markupMarkers($markup)
  {
   $allowed=Array(
    "image" => Array("module", "field", "width", "height", "rotate", "nocache", "redirect", "title"), //main=nr
    "file" => Array("module", "field", "redirect", "external", "title"), //main=nr
    "link" => Array("external", "title"), //main=href
    "quote" => Array("author", "time"), //main=title
    "code" => Array("language"), //main=title

    //HTML tags
    "h1" => Array(),
    "h2" => Array(),
    "h3" => Array(),
    "h4" => Array(),
    "p" => Array(),
    "b" => Array(),
    "i" => Array(),
    "u" => Array(),
   );

   //get all markers
   $open=Array();
   $markers=Array();
   if(preg_match_all("/\[([a-z][^\]]*)\]/", $markup, $m, PREG_OFFSET_CAPTURE)) //get markers
   {
    $lm=sizeof($m[0]);
    for($c=0;$c<$lm;$c++)
    {
     $start=$m[0][$c][1];
     $stop=$m[0][$c][1]+strlen($m[0][$c][0]);

     if(preg_match_all("/((\w+)=(\"([^\"]*)\"|'([^']*)'|([^\s]+))|(\w+))/", $m[1][$c][0], $a)) //get attribute
     {
      //get marker
      if($a[7][0]) //no main attribute value
       $marker=strtolower($a[7][0]);
      else //marker=value
       $marker=strtolower($a[2][0]);

      if(strlen($marker)==1&&$a[7][0]!="i") //short marker except [i]
      {
       foreach(array_keys($allowed) as $alias)
       {
        if($marker==$alias[0])
        {
         $marker=$alias;
         break;
        }
       }
      }

      if(!isset($allowed[$marker])) //unknown marker
       continue; //skip

      if($open) //open tags
      {
       $close=end($open);

       if($markers[$close]["marker"]==$marker) //closing marker matches current
       {
        //close previous marker
        $markers[$close]["text"]=substr($markup, $markers[$close]["stop"], $start-$markers[$close]["stop"]); //get inner text
        $markers[$close]["stop"]=$stop; //now stops at current stop

        array_pop($open); //remove from open list
        continue; //don't parse attributes
       }
      }


      $la=sizeof($a[0]);
      $attributes=Array();
      for($x=0;$x<$la;$x++)
      {
       if($a[7][$x]) //no attribute value
       {
        $key=strtolower($a[7][$x]);
        $value=true;
       }
       else //marker=value
       {
        $key=strtolower($a[2][$x]);
        if($a[4][$x]) //\"
         $value=$a[4][$x];
        else if($a[5][$x]) //'
         $value=$a[5][$x];
        else if($a[6][$x]) //no quotes
         $value=$a[6][$x];
        else //no value
         $value=false;
       }

       if(strlen($key)==1) //short attribute
       {
        if($key==$marker[0]&&!isset($attributes[$marker])) //main attribute
         $key=$marker;
        else
        {
         foreach($allowed[$marker] as $alias)
         {
          if($key==$alias[0]&&!isset($attributes[$alias]))
          {
           $key=$alias;
           break;
          }
         }
        }
       }

       if($key!=$marker) //main attribute
        if($key!="class"&&$key!="id") //global attributes
         if(!in_array($key, $allowed[$marker])) //unknown attribute for marker
          continue; //skip

       $attributes[$key]=$value; //store attribute
      }

      //save marker
      $markers[]=Array(
       "marker" => $marker,
       "text" => false, //no text... yet
       "attributes" => $attributes,
       "start" => $start,
       "stop" => $stop,
      );

      if($marker!="image")
      $open[]=sizeof($markers)-1; //store marker id as open
     }
    }
   }

   //replace start/stop with actual match
   $l=sizeof($markers);
   for($c=0;$c<$l;$c++)
   {
    $markers[$c]["match"]=substr($markup, $markers[$c]["start"], $markers[$c]["stop"]-$markers[$c]["start"]);
    unset($markers[$c]["start"]);
    unset($markers[$c]["stop"]);
   }

   return $markers;
  }

  function markupDecode($markup)
  {
   global $_config, $_path;

   //array values
   if(is_array($markup))
   {
    $html=Array();
    foreach($markup as $k => $v)
     $html[$k]=$this->markupDecode($v);

    return $html;
   }

   //convert specials characters
   $html=htmlentities($markup, ENT_NOQUOTES|ENT_XHTML);

   //get markers
   if($markers=$this->_markupMarkers($html))
   {
    //convert from markup to HTML
    foreach($markers as $marker)
    {
     switch($marker["marker"])
     {
      case "image":
      {
       //tag
       $tag="<img";
       $style="";

       //attributes
       if(!preg_match("/^\d+$/", $marker["attributes"][$marker["marker"]])) //custom URL
       {
        $url=$marker["attributes"][$marker["marker"]];

        if(isset($marker["attributes"]["width"])&&$marker["attributes"]["width"]) //width
         $style=($style?$style." ":"")."width: ".$marker["attributes"]["width"].(preg_match("/^\d+$/", $marker["attributes"]["width"])?"px":"").";";
        if(isset($marker["attributes"]["height"])&&$marker["attributes"]["height"]) //height
         $style=($style?$style." ":"")."height: ".$marker["attributes"]["height"].(preg_match("/^\d+$/", $marker["attributes"]["height"])?"px":"").";";
       }
       else if($_config->get("useModRewrite"))
       {
        $url=$_path["HTML"]["root"]."images/";

        if(isset($marker["attributes"]["module"])&&$marker["attributes"]["module"]) //module
         $url.=$marker["attributes"]["module"]."/";
        if(isset($marker["attributes"]["field"])&&$marker["attributes"]["field"]) //field
         $url.=$marker["attributes"]["field"]."/";

        $url.=$marker["attributes"][$marker["marker"]]."/"; //nr

        if(isset($marker["attributes"]["width"])&&$marker["attributes"]["width"]) //width
         $url.="width/".$marker["attributes"]["width"]."/";
        if(isset($marker["attributes"]["height"])&&$marker["attributes"]["height"]) //height
         $url.="height/".$marker["attributes"]["height"]."/";
        if(isset($marker["attributes"]["rotate"])&&$marker["attributes"]["rotate"]) //rotate
         $url.="rotate/".$marker["attributes"]["rotate"]."/";

        if(isset($marker["attributes"]["nocache"])&&$marker["attributes"]["nocache"]) //nocache
         $url.="nocache/";
        if(isset($marker["attributes"]["redirect"])&&$marker["attributes"]["redirect"]) //redirect
         $url.="redirect/";
       }
       else //normal URL
       {
        $url=$_path["HTML"]["root"]."images/";
        $url.="?nr=".$marker["attributes"][$marker["marker"]]; //nr

        if(isset($marker["attributes"]["module"])&&$marker["attributes"]["module"]) //module
         $url.="&m=".$marker["attributes"]["module"];
        if(isset($marker["attributes"]["field"])&&$marker["attributes"]["field"]) //field
         $url.="&field=".$marker["attributes"]["field"];

        if(isset($marker["attributes"]["width"])&&$marker["attributes"]["width"]) //width
         $url.="&width=".$marker["attributes"]["width"];
        if(isset($marker["attributes"]["height"])&&$marker["attributes"]["height"]) //height
         $url.="&height=".$marker["attributes"]["height"];
        if(isset($marker["attributes"]["rotate"])&&$marker["attributes"]["rotate"]) //rotate
         $url.="&rotate=".$marker["attributes"]["rotate"];

        if(isset($marker["attributes"]["nocache"])&&$marker["attributes"]["nocache"]) //nocache
         $url.="&nocache";
        if(isset($marker["attributes"]["redirect"])&&$marker["attributes"]["redirect"]) //redirect
         $url.="&redirect";
       }

       $tag.=" src=\"".$url."\"";

       if(isset($marker["attributes"]["title"])&&$marker["attributes"]["title"]) //title
        $tag.=" alt=\"".$marker["attributes"]["title"]."\"";
       else
        $tag.=" alt=\"\"";

       if(isset($marker["attributes"]["class"])&&$marker["attributes"]["class"]) //class
        $tag.=" class=\"".$marker["attributes"]["class"]."\"";
       if($style) //style
        $tag.=" style=\"".$style."\"";
       if(isset($marker["attributes"]["id"])&&$marker["attributes"]["id"]) //id
        $tag.=" id=\"".$marker["attributes"]["id"]."\"";

       $tag.="/>";

       break;
      }
      case "file":
      {
       //tag
       $tag="<a";

       //attributes
       $url=$_path["HTML"]["root"]."files/";
       if($_config->get("useModRewrite"))
       {
        if(isset($marker["attributes"]["module"])&&$marker["attributes"]["module"]) //module
         $url.=$marker["attributes"]["module"]."/";
        if(isset($marker["attributes"]["field"])&&$marker["attributes"]["field"]) //field
         $url.=$marker["attributes"]["field"]."/";

        $url.=$marker["attributes"][$marker["marker"]]."/"; //nr

        if(isset($marker["attributes"]["redirect"])&&$marker["attributes"]["redirect"]) //redirect
         $url.="redirect/";
       }
       else //normal URL
       {
        $url.="?nr=".$marker["attributes"][$marker["marker"]]; //nr

        if(isset($marker["attributes"]["module"])&&$marker["attributes"]["module"]) //module
         $url.="&m=".$marker["attributes"]["module"];
        if(isset($marker["attributes"]["field"])&&$marker["attributes"]["field"]) //field
         $url.="&field=".$marker["attributes"]["field"];

        if(isset($marker["attributes"]["redirect"])&&$marker["attributes"]["redirect"]) //redirect
         $url.="&redirect";
       }

       $tag.=" href=\"".$url."\"";

       if(isset($marker["attributes"]["external"])) //open in other window
       {
        if($marker["attributes"]["external"]&&$marker["attributes"]["external"]!==true) //window specified
         $tag.=" rel=\"".$marker["attributes"]["external"]."\"";
        else //no window specified
         $tag.=" rel=\"external\"";
       }

       if(!isset($marker["attributes"]["title"])) //no title, use text
       {
        if($title=trim(preg_replace("/\[[^\]]*\]/", "", strip_tags($marker["text"])))) //remove [markers] and <tags>
         $tag.=" title=\"".$title."\"";
       }
       else if($marker["attributes"]["title"]) //title
        $tag.=" title=\"".$marker["attributes"]["title"]."\"";
       else //empty title
        $tag.=" title=\"\"";

       if(isset($marker["attributes"]["class"])&&$marker["attributes"]["class"]) //class
        $tag.=" class=\"".$marker["attributes"]["class"]."\"";
       if(isset($marker["attributes"]["id"])&&$marker["attributes"]["id"]) //id
        $tag.=" id=\"".$marker["attributes"]["id"]."\"";

       $tag.=">";

       //inner text
       $tag.=$marker["text"];

       //close tag
       $tag.="</a>";

       break;
      }
      case "link":
      {
       //tag
       $tag="<a href=\"".$marker["attributes"][$marker["marker"]]."\"";

       //attributes
       if(isset($marker["attributes"]["external"])) //open in other window
       {
        if($marker["attributes"]["external"]&&$marker["attributes"]["external"]!==true) //window specified
         $tag.=" rel=\"".$marker["attributes"]["external"]."\"";
        else //no window specified
         $tag.=" rel=\"external\"";
       }

       if(!isset($marker["attributes"]["title"])) //no title, use text
       {
        if($title=trim(preg_replace("/\[[^\]]*\]/", "", strip_tags($marker["text"])))) //remove [markers] and <tags>
         $tag.=" title=\"".$title."\"";
       }
       else if($marker["attributes"]["title"]) //title
        $tag.=" title=\"".$marker["attributes"]["title"]."\"";
       else //empty title
        $tag.=" title=\"\"";

       if(isset($marker["attributes"]["class"])&&$marker["attributes"]["class"]) //class
        $tag.=" class=\"".$marker["attributes"]["class"]."\"";
       if(isset($marker["attributes"]["id"])&&$marker["attributes"]["id"]) //id
        $tag.=" id=\"".$marker["attributes"]["id"]."\"";

       $tag.=">";

       //inner text
       $tag.=$marker["text"];

       //close tag
       $tag.="</a>";

       break;
      }
      case "quote":
      {
       //tag
       $tag="<div";

       //attributes
       $tag.=" class=\"quote".(isset($marker["attributes"]["class"])&&$marker["attributes"]["class"]?" ".$marker["attributes"]["class"]:"")."\"";

       if(isset($marker["attributes"]["id"])&&$marker["attributes"]["id"]) //id
        $tag.=" id=\"".$marker["attributes"]["id"]."\"";

       $tag.=">";

       //special attributes
       if(!is_bool($marker["attributes"][$marker["marker"]])) //title
        $tag.="<div class=\"title\">".$marker["attributes"][$marker["marker"]]."</div>";

       if(isset($marker["attributes"]["author"])&&$marker["attributes"]["author"]) //author
        $tag.="<div class=\"author\">".$marker["attributes"]["author"]."</div>";
       else if(isset($marker["attributes"]["a"])&&$marker["attributes"]["a"]) //author
        $tag.="<div class=\"author\">".$marker["attributes"]["a"]."</div>";

       if(isset($marker["attributes"]["time"])&&$marker["attributes"]["time"]) //time
        $tag.="<div class=\"time\">".$marker["attributes"]["time"]."</div>";
       else if(isset($marker["attributes"]["t"])&&$marker["attributes"]["t"]) //time
        $tag.="<div class=\"time\">".$marker["attributes"]["t"]."</div>";

       //inner text
       $tag.=$marker["text"];

       //close tag
       $tag.="</div>";

       break;
      }
      case "code":
      {
       //tag
       $tag="<div";

       //attributes
       $tag.=" class=\"code".(isset($marker["attributes"]["class"])&&$marker["attributes"]["class"]?" ".$marker["attributes"]["class"]:"")."\"";

       if(isset($marker["attributes"]["id"])&&$marker["attributes"]["id"]) //id
        $tag.=" id=\"".$marker["attributes"]["id"]."\"";

       $tag.=">";

       //special attributes
       if(!is_bool($marker["attributes"][$marker["marker"]])) //title
        $tag.="<div class=\"title\">".$marker["attributes"][$marker["marker"]]."</div>";

       if(isset($marker["attributes"]["language"])&&$marker["attributes"]["language"]) //language
        $tag.="<div class=\"language\">".$marker["attributes"]["language"]."</div>";
       else if(isset($marker["attributes"]["l"])&&$marker["attributes"]["l"]) //language
        $tag.="<div class=\"language\">".$marker["attributes"]["l"]."</div>";

       //inner text
       $tag.=$marker["text"];

       //close tag
       $tag.="</div>";

       break;
      }
      default: //convert directly to HTML
      {
       //tag
       $tag="<".$marker["marker"];

       //attributes
       foreach($marker["attributes"] as $attribute => $value)
        if($attribute!=$marker["marker"]) //skip main attribute
         $tag.=" ".$attribute."=\"".$value."\"";

       $tag.=">";

       //inner text
       $tag.=$marker["text"];

       //close tag
       $tag.="</".$marker["marker"].">";

       break;
      }
     }

     if(($pos=strpos($html, $marker["match"]))!==false)
      $html=substr_replace($html, $tag, $pos, strlen($marker["match"]));
    }
   }
   //else //no markers found

   if($lb=$this->detectLineBreaks($html))
   {
    //convert bullet/number lists
    if(preg_match_all("/(^|".$lb.")([\-\.].*)(".$lb.$lb."|".$lb."[A-Z0-9]|$)/siU", $html, $m1, PREG_OFFSET_CAPTURE)) //get all lists
    {
     $listType=$m1[2][0][0][0];

     $offset=0;
     $l1=sizeof($m1[0]);
     for($c=0;$c<$l1;$c++) //get items and sub lists
     {
      if(preg_match_all("/(\s*[\-\.]?)\s*(.+)(".$lb."|$)/", $m1[2][$c][0], $m2))
      {
       $depthCur=0;
       $depthPrev=0;
       $depth=false;

       $list=$m1[1][$c][0];

       $l2=sizeof($m2[0]);
       for($x=0;$x<$l2;$x++)
       {
        $listType=substr($m2[1][$x], -1);

        if($listType=="-"||$listType==".") //bullet/number entry
        {
         $depth=strlen($m2[1][$x]);

         if($depth>$depthPrev) //item deeper than previous
         {
          if($depthCur>0) //sub level
           $list=substr($list, 0, -strlen($lb));

          $list.="<ul class=\"".($listType=="."?"numbers":"bullets")."\">".$lb;
          $depthCur++;
         }
         else if($x>0) //subsequent entries on same level
          $list=substr($list, 0, -strlen($lb))."</li>".$lb;

         if($depth<$depthPrev) //item higher than previous
         {
          $depthCur--;
          $list.=str_repeat(" ", $depthCur)."</ul></li>".$lb;
         }

         $list.=str_repeat(" ", $depthCur)."<li>";

         $depthPrev=$depth;
        }

        $list.=$m2[2][$x].$lb;
       }

       $list=substr($list, 0, -strlen($lb))."</li>".$lb; //close last item

       while($depthCur>1)
       {
        $depthCur--;
        $list.=str_repeat(" ", $depthCur)."</ul></li>".$lb;
       }

       $list.="</ul>";

       $start=$m1[0][$c][1]+$offset; //markup list start + previous list(s) offset
       $length=strlen($m1[1][$c][0])+strlen($m1[2][$c][0]); //replace markup list
       $offset+=strlen($list)-$length; //(re)calc offset: diff between markup and html list length

       $html=substr_replace($html, $list, $start, $length);
      }
     }
    }

    //convert linebreaks
    $html=str_replace($lb, "<br/>".$lb, $html);
    $html=preg_replace("!(</(div|p|h1|h2|h3|h4)>)<br/>".$lb."!", "\\1".$lb, $html); //remove bugs br from block elements
    $html=preg_replace("/(<ul[^>]*>|<\/ul>|<\/li>)<br\/>/", "\\1", $html); //removed bogus br from ul/li
   }

   //convert whitespace
   $html=str_replace("  ", "&nbsp; ", $html);
   if(preg_match_all("/".$lb."(&nbsp;\s\s?)+<\/?(ul|li)/", $html, $m)) //removed bogus nbsp from ul/li
   {
    foreach(array_unique($m[0]) as $search)
    {
     $replace=str_replace("&nbsp; ", "  ", $search);
     $html=str_replace($search, $replace, $html);
    }
   }

   return $html;
  }

  function markupEncode($html, $short=false)
  {
   //array values
   if(is_array($html))
   {
    $markup=Array();
    foreach($html as $k => $v)
     $markup[$k]=$this->markupEncode($v);

    return $markup;
   }

   //convert whitespace
   $html=str_replace("&nbsp; ", "  ", $html);

   //convert linebreaks
   if(!($lb=$this->detectLineBreaks($html)))
    $lb="\n";

   $markup=preg_replace("!<br\s*/?>[\r\n]*!", $lb, $html);

   //get HTML tags
   $xml=new Format_XML();
   if(strpos($markup, "<")!==false&&$xml->parse($markup))
   {
    $xml->reset();
    while($xml->next()!==false)
    {
     $match=false;
     $name=strtolower($xml->v["name"]);
     switch($name)
     {
      //image
      case "img":
      {
       if(!isset($xml->v["attributes"]["src"]))
        break;

       $name="image";

       //marker
       $marker="[".($short?$name[0]:$name);

       if(preg_match("!^\w+\://!", $xml->v["attributes"]["src"])||!preg_match("!^.*images/(.+)$!", $xml->v["attributes"]["src"], $url)) //custom URL
        $marker.="=".$xml->v["attributes"]["src"]; //don't quote
       else
       {
        $url=$url[1];

        //attributes
        if(!strstr($url, "?")) //rewrite
        {
         if(preg_match("!^([a-z][a-z0-9_]+)/([a-z][a-z0-9_]+)/(\d+)!i", $url, $m)) //module/field/nr
         {
          $module=$m[1];
          $field=$m[2];
          $nr=$m[3];
         }
         else if(preg_match("!^([a-z][a-z0-9_]+)/(\d+)!i", $url, $m)) //module/nr
         {
          $module=$m[1];
          $field=false;
          $nr=$m[2];
         }
         else if(preg_match("!^(\d+)!", $url, $m)) //module/nr
         {
          $module=false;
          $field=false;
          $nr=$m[1];
         }
         else
          break;

         $marker.="=".$nr; //don't quote

         if($module)
          $marker.=" ".($short?"m":"module")."=".$module; //don't quote
         if($field)
          $marker.=" ".($short?"f":"field")."=".$field; //don't quote

         if(preg_match("!width/(\d+)!", $url, $m))
          $marker.=" ".($short?"w":"width")."=".$m[1]; //don't quote
         if(preg_match("!height/(\d+)!", $url, $m))
          $marker.=" ".($short?"h":"height")."=".$m[1]; //don't quote
         if(preg_match("!rotate/(\w+)!", $url, $m))
          $marker.=" ".($short?"r":"rotate")."=".strtolower($m[1]); //don't quote
        }
        else //normal url vars
        {
         if(preg_match("![?&]nr=(\d+)!", $url, $m))
          $marker.="=".$m[1]; //don't quote
         else
          break;

         if(preg_match("![?&]m=(\w+)!", $url, $m))
          $marker.=" ".($short?"m":"module")."=".$m[1]; //don't quote
         if(preg_match("![?&]field=(\w+)!", $url, $m))
          $marker.=" ".($short?"f":"field")."=".$m[1]; //don't quote

         if(preg_match("![?&]width=(\d+)!", $url, $m))
          $marker.=" ".($short?"w":"width")."=".$m[1]; //don't quote
         if(preg_match("![?&]height=(\d+)!", $url, $m))
          $marker.=" ".($short?"h":"height")."=".$m[1]; //don't quote
         if(preg_match("![?&]rotate=(\w+)!", $url, $m))
          $marker.=" ".($short?"r":"rotate")."=".strtolower($m[1]); //don't quote
        }

        if(stristr($url, "nocache"))
         $marker.=" nocache";
        if(stristr($url, "redirect"))
         $marker.=" redirect";
       }

       if(isset($xml->v["attributes"]["alt"])&&$xml->v["attributes"]["alt"])
       {
        if(!strstr($xml->v["attributes"]["alt"], " ")) //no spaces
         $marker.=" ".($short?"t":"title")."=".$xml->v["attributes"]["alt"]; //don't quote
        else //spaces
         $marker.=" ".($short?"t":"title")."=\"".$xml->v["attributes"]["alt"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["class"]))
       {
        if(!strstr($xml->v["attributes"]["class"], " ")) //no spaces
         $marker.=" class=".$xml->v["attributes"]["class"]; //don't quote
        else //spaces
         $marker.=" class=\"".$xml->v["attributes"]["class"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["style"])&&$xml->v["attributes"]["style"])
       {
        if(preg_match("/width:\s*(\d+)([^;]*);/", $xml->v["attributes"]["style"], $m))
         $marker.=" ".($short?"w":"width")."=".$m[1].($m[2]!="px"?$m[2]:""); //don't quote
        if(preg_match("/height:\s*(\d+)([^;]*);/", $xml->v["attributes"]["style"], $m))
         $marker.=" ".($short?"h":"height")."=".$m[1].($m[2]!="px"?$m[2]:""); //don't quote
       }

       if(isset($xml->v["attributes"]["id"]))
       {
        if(!strstr($xml->v["attributes"]["id"], " ")) //no spaces
         $marker.=" id=".$xml->v["attributes"]["id"]; //don't quote
        else //spaces
         $marker.=" id=\"".$xml->v["attributes"]["id"]."\""; //add quotes
       }

       $marker.="]";

       $match=$xml->getOuter($xml->vc-1);
       break;
      }

      //file/link
      case "a":
      {
       if(!isset($xml->v["attributes"]["href"]))
        break;

       if(preg_match("!^\w+\://!", $xml->v["attributes"]["href"])||!preg_match("!^.*files/(.+)$!", $xml->v["attributes"]["href"], $url)) //custom URL
       {
        $url=$xml->v["attributes"]["href"];
        $name="link";
       }
       else
       {
        $url=$url[1];
        $name="file";
       }

       //marker
       $marker="[".($short?$name[0]:$name);

       //attributes
       if($name=="link")
        $marker.="=".$url; //don't quote
       else //file
       {
        if(!strstr($url, "?")) //rewrite
        {
         if(preg_match("!^([a-z][a-z0-9_]+)/([a-z][a-z0-9_]+)/(\d+)!i", $url, $m)) //module/field/nr
         {
          $module=$m[1];
          $field=$m[2];
          $nr=$m[3];
         }
         else if(preg_match("!^([a-z][a-z0-9_]+)/(\d+)!i", $url, $m)) //module/nr
         {
          $module=$m[1];
          $field=false;
          $nr=$m[2];
         }
         else if(preg_match("!^(\d+)!", $url, $m)) //module/nr
         {
          $module=false;
          $field=false;
          $nr=$m[1];
         }
         else
          break;

         $marker.="=".$nr; //don't quote

         if($module)
          $marker.=" ".($short?"m":"module")."=".$module; //don't quote
         if($field)
          $marker.=" ".($short?"f":"field")."=".$field; //don't quote
        }
        else //normal url vars
        {
         if(preg_match("![?&]nr=(\d+)!", $url, $m))
          $marker.="=".$m[1]; //don't quote
         else
          break;

         if(preg_match("![?&]m=(\w+)!", $url, $m))
          $marker.=" ".($short?"m":"module")."=".$m[1]; //don't quote
         if(preg_match("![?&]field=(\w+)!", $url, $m))
          $marker.=" ".($short?"f":"field")."=".$m[1]; //don't quote
        }

        if(stristr($url, "redirect"))
         $marker.=" redirect";
       }

       if(isset($xml->v["attributes"]["rel"])&&$xml->v["attributes"]["rel"])
       {
        if($xml->v["attributes"]["rel"]=="external") //external
         $marker.=" ".($short?"e":"external"); //no attribute value
        else if(!strstr($xml->v["attributes"]["rel"], " ")) //no spaces
         $marker.=" ".($short?"e":"external")."=".$xml->v["attributes"]["rel"]; //don't quote
        else //spaces
         $marker.=" ".($short?"e":"external")."=\"".$xml->v["attributes"]["rel"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["title"])&&$xml->v["attributes"]["title"]!=trim(preg_replace("/\[[^\]]*\]/", "", strip_tags($xml->getInner($xml->vc-1))))) //no title or same as text
       {
        if(!$xml->v["attributes"]["title"]) //empty title
         $marker.=" ".($short?"t":"title")."=\"\"";
        else if(!strstr($xml->v["attributes"]["title"], " ")) //no spaces
         $marker.=" ".($short?"t":"title")."=".$xml->v["attributes"]["title"]; //don't quote
        else //spaces
         $marker.=" ".($short?"t":"title")."=\"".$xml->v["attributes"]["title"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["class"]))
       {
        if(!strstr($xml->v["attributes"]["class"], " ")) //no spaces
         $marker.=" class=".$xml->v["attributes"]["class"]; //don't quote
        else //spaces
         $marker.=" class=\"".$xml->v["attributes"]["class"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["id"]))
       {
        if(!strstr($xml->v["attributes"]["id"], " ")) //no spaces
         $marker.=" id=".$xml->v["attributes"]["id"]; //don't quote
        else //spaces
         $marker.=" id=\"".$xml->v["attributes"]["id"]."\""; //add quotes
       }

       $marker.="]";

       //inner text
       $marker.=$xml->getInner($xml->vc-1);

       //close marker
       $marker.="[".($short?$name[0]:$name)."]";

       $match=$xml->getOuter($xml->vc-1);
       break;
      }

      //quote+code
      case "div":
      {
       if(!isset($xml->v["attributes"]["class"])) //no class
        break;
       else if(stristr($xml->v["attributes"]["class"], "quote"))
        $name="quote";
       else if(stristr($xml->v["attributes"]["class"], "code"))
        $name="code";
       else
        break;

       //marker
       $marker="[".($short?$name[0]:$name);

       //attributes

       $inner=$xml->getInner($xml->vc-1);
       if(preg_match("!<div class=\"title\">([^<]+)</div>!", $inner, $m))
       {
        if(!strstr($m[1], " ")) //no spaces
         $marker.="=".$m[1]; //don't quote
        else //spaces
         $marker.="=\"".$m[1]."\""; //add quotes

        $inner=str_replace($m[0], "", $inner); //remove tag from inner text
       }

       if($name=="quote")
       {
        if(preg_match("!<div class=\"author\">([^<]+)</div>!", $inner, $m))
        {
         if(!strstr($m[1], " ")) //no spaces
          $marker.=" ".($short?"a":"author")."=".$m[1]; //don't quote
         else //spaces
          $marker.=" ".($short?"a":"author")."=\"".$m[1]."\""; //add quotes

         $inner=str_replace($m[0], "", $inner); //remove tag from inner text
        }

        if(preg_match("!<div class=\"time\">([^<]+)</div>!", $inner, $m))
        {
         if(!strstr($m[1], " ")) //no spaces
          $marker.=" ".($short?"t":"time")."=".$m[1]; //don't quote
         else //spaces
          $marker.=" ".($short?"t":"time")."=\"".$m[1]."\""; //add quotes

         $inner=str_replace($m[0], "", $inner); //remove tag from inner text
        }
       }
       else if($name=="code")
       {
        if(preg_match("!<div class=\"language\">([^<]+)</div>!", $inner, $m))
        {
         if(!strstr($m[1], " ")) //no spaces
          $marker.=" ".($short?"l":"language")."=".$m[1]; //don't quote
         else //spaces
          $marker.=" ".($short?"l":"language")."=\"".$m[1]."\""; //add quotes

         $inner=str_replace($m[0], "", $inner); //remove tag from inner text
        }
       }

       if($class=preg_replace("/".$name."\s?/i", "", $xml->v["attributes"]["class"]))
       {
        if(!strstr($class, " ")) //no spaces
         $marker.=" class=".$class; //don't quote
        else //spaces
         $marker.=" class=\"".$class."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["id"]))
       {
        if(!strstr($xml->v["attributes"]["id"], " ")) //no spaces
         $marker.=" id=".$xml->v["attributes"]["id"]; //don't quote
        else //spaces
         $marker.=" id=\"".$xml->v["attributes"]["id"]."\""; //add quotes
       }

       $marker.="]";

       //inner text
       $marker.=$inner;

       //close marker
       $marker.="[".($short?$name[0]:$name)."]";

       $match=$xml->getOuter($xml->vc-1);
       break;
      }

      //HTML tags
      case "h1":
      case "h2":
      case "h3":
      case "h4":
      case "p":
      case "b":
      case "i":
      case "u":
      {
       //marker
       $marker="[".$name;

       //attributes
       if(isset($xml->v["attributes"]["class"]))
       {
        if(!strstr($xml->v["attributes"]["class"], " ")) //no spaces
         $marker.=" class=".$xml->v["attributes"]["class"]; //don't quote
        else //spaces
         $marker.=" class=\"".$xml->v["attributes"]["class"]."\""; //add quotes
       }

       if(isset($xml->v["attributes"]["id"]))
       {
        if(!strstr($xml->v["attributes"]["id"], " ")) //no spaces
         $marker.=" id=".$xml->v["attributes"]["id"]; //don't quote
        else //spaces
         $marker.=" id=\"".$xml->v["attributes"]["id"]."\""; //add quotes
       }

       $marker.="]";

       //inner text
       $marker.=$xml->getInner($xml->vc-1);

       //close marker
       $marker.="[".$name."]";

       $match=$xml->getOuter($xml->vc-1);
       break;
      }

      default: break;
     }

     if($match!==false)
      if(($pos=strpos($markup, $match))!==false)
       $markup=substr_replace($markup, $marker, $pos, strlen($match));
    }

    //bullet/number list
    if(preg_match_all("/(<ul.*<\/ul>)(".$lb.$lb."|".$lb."[A-Z0-9]|$)/siU", $markup, $m1, PREG_OFFSET_CAPTURE)) //get all lists
    {
     $offset=0;
     $l1=sizeof($m1[0]);
     for($c=0;$c<$l1;$c++) //get items and sub lists
     {
      if(preg_match_all("/<(\/?)(ul|li)( class=\"([^\"]+)\")?>([^<]*)/s", $m1[1][$c][0], $m2))
      {
       $list="";
       $original="";
       $depthCur=-1;
       $listType=Array();

       $l=sizeof($m2[0]);
       for($x=0;$x<$l;$x++)
       {
        $original.=$m2[0][$x];
        if($m2[1][$x]) //close
        {
         if($m2[2][$x]=="ul") //list
         {
          array_pop($listType);
          $depthCur--;

          if($depthCur<0)
           break;
         }
        }
        else //open
        {
         if($m2[2][$x]=="ul") //list
         {
          $depthCur++;
          $listType[]=($m2[4][$x]=="numbers"?".":"-");
         }
         else //item
         {
          $list.=str_repeat(" ", $depthCur*2).end($listType)." ";
          $list.=str_replace($lb, $lb.str_repeat(" ", 2+($depthCur*2)), $m2[5][$x]); //indent multiline items
          $list.=$lb;
         }
        }
       }

       $list=substr($list, 0, -strlen($lb)); //remove linebreak from last item

       $start=$m1[0][$c][1]+$offset; //html list start + previous list(s) offset
       $length=strlen($m1[1][$c][0]); //replace html list
       $offset+=strlen($list)-$length; //(re)calc offset: diff between markup and html list length

       $markup=substr_replace($markup, $list, $start, $length);
      }
     }
    }
   }

   //else //parse failed, probably no tags found

   //convert specials characters
   $markup=html_entity_decode($markup, ENT_QUOTES|ENT_XHTML);

   return $markup;
  }


//URL

  function urlEncode($url)
  {
   //array values
   if(is_array($url))
   {
    foreach($url as $k => $v)
     $url[$k]=$this->urlEncode($v);

    return $url;
   }

   if(preg_match("!(^[a-z0-9]+://)([^/\?]+)([^\?]*)(.*)$!i", $url, $m))  //url and args (args can contain ?)
   {
    $prefix=$m[1]; //protocol

    if(preg_match("/^([^:]*)(:?)(.*)@([^@]+)$/", $m[2], $m2)) //get user:pass from domain
    {
     if(!$m2[2]) //no : just a user
      $prefix.=rawurlencode($m2[1])."@";
     else //user and pass (could be empty)
      $prefix.=rawurlencode($m2[1]).":".rawurlencode($m2[3])."@";

     $prefix.=$m2[4];
    }
    else //just a domain
     $prefix.=$m[2];

    if($m[3]) //directories/files
     $prefix.=str_replace("%2F", "/", rawurlencode($m[3])); //replace all special chars except /

    if($m[4]) //args, ? included
    {
     $prefix.="?";
     $args=substr($m[4], 1);
    }
    else //no args
     $args="";
   }
   else if($url&&$url[0]=="?") //arg ? included
   {
    $prefix="?";
    $args=substr($url, 1);
   }
   else
   {
    $prefix="";
    $args=$url;
   }

   if($args)
   {
    $encargs=Array();
    foreach(explode("&", $args) as $arg)
    {
     $arg=explode("=", $arg);

     if(sizeof($arg)==1) //only key name
      $encargs[]=urlencode($arg[0]);
     else //key and value
      $encargs[]=urlencode($arg[0])."=".urlencode($arg[1]);
    }

    $args=implode("&", $encargs);
   }

   return $prefix.$args;
  }

  function urlDecode($url)
  {
   //array values
   if(is_array($url))
   {
    foreach($url as $k => $v)
     $url[$k]=$this->urlDecode($v);

    return $url;
   }

   return urldecode($url);
  }


//BASE64

  function base64Encode($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->base64Encode($v);

    return $text;
   }

   return base64_encode($text);
  }

  function base64Decode($text)
  {
   //array values
   if(is_array($text))
   {
    foreach($text as $k => $v)
     $text[$k]=$this->base64Decode($v);

    return $text;
   }

   return base64_decode($text);
  }


//EMAIL

  function scrambleEmail($email)
  {
   //array values
   if(is_array($email))
   {
    foreach($email as $k => $v)
     $email[$k]=$this->scrambleEmail($v);

    return $email;
   }

   return str_replace("@", "(at)", str_replace(".", "[dot]", $email));
  }
  function unscrambleEmail($email)
  {
   //array values
   if(is_array($email))
   {
    foreach($email as $k => $v)
     $email[$k]=$this->unscrambleEmail($v);

    return $email;
   }

   return str_replace("(at)", "@", str_replace("[dot]", ".", $email));
  }


 }

?>
