<?

 /*****************************************************************************
 * Filename: include/plugin.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."fs.php");

 class Plugin
 {
  function __construct($name, $tmp=false)
  {
   $this->fs=new FS();
   $this->e=new Errors();

   $this->l=&$this->e->l;
   $this->l->read("plugin");

   $this->class=Array(
    "frontend" => Array(
     "filename" => $name.".php",
     "name" => "Plugin_".$name,
     "base" => "Base_Plugin",
     "defined" => false,
     "object" => false,
    ),
    "backend" => Array(
     "filename" => "backend.php",
     "name" => "Plugin_Backend_".$name,
     "base" => "Base_Plugin_Backend",
     "defined" => false,
     "object" => false,
    ),
    "setup" => Array(
     "filename" => "setup.php",
     "name" => "Plugin_Setup_".$name,
     "base" => "Base_Plugin_Setup",
     "defined" => false,
     "object" => false,
    ),
   );

   $this->info=false;
   $this->deps=false;
   $this->config=Array();

   $this->baseURL="";
   $this->vars=Array();

   $this->enabled=false;

   $this->name=$name;

   if(!$this->initPath($tmp))
    return;

   $this->l->read("plugin_".$this->name);

   $this->getStatus();

   foreach(array_keys($this->class) as $class)
    $this->checkClass($class);
  }

  function __clone()
  {
   $tmpE=&$this->e;
   $tmpL=&$this->e->l;

   //restore original language (dereference), so it's cloned correctly
   unset($this->e->l);
   $this->e->l=$tmpL;

   //clone object
   unset($this->e);
   $this->e=clone $tmpE;

   //restore original
   unset($tmpE->l);
   $tmpE->l=&$tmpL;

   //update references
   unset($this->l);
   $this->l=&$this->e->l;

   //clone objects
   $this->fs=clone $this->fs;
   if($this->info)
    $this->info=clone $this->info;
   if($this->deps)
    $this->deps=clone $this->deps;

   if($this->class["frontend"]["object"]) //frontend class initialized
   {
    $tmp=$this->config;
    unset($this->config);
    $this->config=$tmp;
    unset($tmp);
   }

   //clone class objects
   foreach(array_keys($this->class) as $class)
   {
    if(!$this->class[$class]["object"])
     continue;

    $this->class[$class]["object"]=clone $this->class[$class]["object"];

    //update reference
    switch($class)
    {
     case "frontend":
     {
      //unset references
      unset($this->class[$class]["object"]->plugin);
      unset($this->class[$class]["object"]->config);
      unset($this->class[$class]["object"]->e);

      //set new references
      $this->class[$class]["object"]->plugin=&$this;
      $this->class[$class]["object"]->config=&$this->config; //also supports alternative config...
      $this->class[$class]["object"]->e=&$this->e;
      break;
     }
     case "backend":
     {
      //unset references
      unset($this->class[$class]["object"]->plugin);
      unset($this->class[$class]["object"]->e);

      //set new references
      $this->class[$class]["object"]->plugin=&$this;
      $this->class[$class]["object"]->e=&$this->e;
      break;
     }
     case "setup":
     {
      //unset references
      unset($this->class[$class]["object"]->plugin);
      unset($this->class[$class]["object"]->e);

      //set new references
      $this->class[$class]["object"]->plugin=&$this;
      $this->class[$class]["object"]->e=&$this->e;

      //clone objects
      $this->class[$class]["object"]->fs=clone $this->class[$class]["object"]->fs;
      break;
     }
     default: break;
    }
   }
  }


  function initPath($tmp=false)
  {
   global $_path;

   $this->path=Array();
   $this->path["status"]=$_path["project"]["plugins"]."status.txt";

   if(!$this->fs->exists($_path["project"]["plugins"], "directory"))
    if(!$this->fs->mkdir($_path["project"]["plugins"])||!$this->fs->setMode($_path["project"]["plugins"], 7, 5, 5))
     $this->e->add("Unable to create plugins directory: \"".$_path["project"]["plugins"]."\", please make sure the webservice user has the appropriate permissions", "system");


   if(!preg_match(REGEXP_DATA_PLUGIN, $this->name))
   {
    $this->e->add("Illegal plugin name: \"".$this->name."\"", "hack");
    return false;
   }

   if($tmp)
    $directory=".tmp";
   else
    $directory=$this->name;

   if($this->fs->exists($_path["project"]["plugins"].$directory, "directory"))
   {
    $this->path["root"]=$_path["project"]["plugins"].$directory."/";
    $this->type="project";
   }
   else if($this->fs->exists($_path["plugins"].$directory, "directory"))
   {
    $this->path["root"]=$_path["plugins"].$directory."/";
    $this->type="system";
   }
   else
   {
    $this->e->add("Plugin: \"".$this->name."\" doesn't exist", "hack", "Plugin directory: \"".$directory."\" doesn't exist");
    return false;
   }

   $this->path["modules"]=$this->path["root"]."modules/";
   $this->path["templates"]=$this->path["root"]."templates/";
   $this->path["pluginInfo"]=$this->path["root"]."pluginInfo.xml";
   $this->path["pluginDependencies"]=$this->path["root"]."pluginDependencies.xml";

   if($this->fs->exists($this->path["root"]."changelog.txt", "file"))
    $this->path["pluginChangelog"]=$this->path["root"]."changelog.txt";
   else
    $this->path["pluginChangelog"]=false;

   if($this->fs->exists($this->path["root"]."LICENSE.txt", "file"))
    $this->path["pluginLicense"]=$this->path["root"]."LICENSE.txt";
   else
    $this->path["pluginLicense"]=false;

   $this->path["pluginLicense2"]=false; //set when info is read

   return true;
  }


  function export($method)
  {
   $archive=new Archive();
   if(!$archive->addDirectory($this->path["root"], preg_replace("!/[^/]+/$!", "/", $this->path["root"])))
   {
    $this->e->add($archive->e);
    return false;
   }

   if($this->type=="system") //replace _path["plugins"] with _path["project"]["plugins"] for system plugins (import a system plugin will make it a project plugin)
    foreach(array_keys($archive->files) as $file)
     if($archive->files[$file]["type"]=="file"&&$archive->files[$file]["contents"])
      $archive->files[$file]["contents"]=str_replace('$_path["plugins"]."'.$this->plugin, '$_path["project"]["plugins"]."'.$this->plugin, $archive->files[$file]["contents"]);

   if(!($output=$archive->output($method, true, $this->name."-".$this->version.".".$method)))
   {
    $this->e->add($archive->e);
    return false;
   }

   print $output;
   exit;
  }

  function import($file)
  {
   global $_config, $_path;

   if(!$_config->userRoot())
   {
    $this->e->add("Plugins can only be imported by the root user", "error");
    return false;
   }

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if($this->fs->exists($tmppath))
    if(!$this->_removeTmp())
     return false;

   if(!$this->fs->mkdir($tmppath))
   {
    $this->e->add("Unable to create plugin temporary import directory: \"".$tmppath."\"", "error");
    return false;
   }

   if(!preg_match("!/?([a-z0-9\_]{1,25})-(\d+\.\d+(\.\d+)?)[^/]+$!i", $file, $name))
   {
    $this->e->add("No plugin name detected from file name: \"".$file."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->name=$name[1];
   $this->version=$name[2];

   $archive=new Archive();

   if(!$archive->open($file))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   $info=false;
   foreach($archive->getFiles() as $entry)
   {
    if(preg_match("!pluginInfo\.xml$!", $entry)&&!preg_match("!language/[a-z]+/modules/pluginInfo.xml$!i", $entry))
    {
     $info=$entry;
     break;
    }
   }

   if(!$info)
   {
    $this->e->add("Unabled to find \"pluginInfo.xml\" in import file: \"".$file."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $pluginroot=preg_replace("!pluginInfo\.xml$!", "", $info);

   if(!$archive->rerootFiles($pluginroot))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   unset($pluginroot, $info);

   if(!$archive->saveFiles($tmppath))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   unset($archive);

   $plugin=new Plugin($this->name, true);
   $plugin->info();

   if($plugin->e->any("fatal"))
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   if($plugin->version!=$this->version)
   {
    $this->e->add("Plugin \"".$this->name."\" info version: \"".$plugin->version."\" doesn't match archive version: \"".$this->version."\"", "error");
    $this->_removeTmp();
    return false;
   }

   if(in_array($this->name, JAS::plugins("project")))
   {
    $plugin2=new Plugin($this->name);
    $plugin2->info();

    if($plugin->version==$plugin2->version)
    {
     $this->e->add("Plugin: \"".$this->name." ".$plugin2->version."\" is already installed, do you want to reinstall it?", "warning");
     return false;
    }
    else
    {
     $this->e->add("Plugin: \"".$this->name." ".$plugin2->version."\" is already installed, do you want to install version ".$plugin->version."?", "warning");
     return false;
    }
   }

   $pluginpath=$_path["project"]["plugins"].$this->name;
   if(!$this->fs->rename($tmppath, $pluginpath))
   {
    $this->e->add("Unable to rename \"".$tmppath."\" to \"".$pluginpath."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->e->add("Plugin: \"".$this->name." ".$plugin->version."\" installed", "note");

   return true;
  }

  function importOverwrite($name)
  {
   global $_config, $_path;

   if(!$_config->userRoot())
   {
    $this->e->add("Plugins can only be imported by the root user", "error");
    return false;
   }

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if(!$this->fs->exists($tmppath))
   {
    $this->e->add("Uploaded plugin not found in: \"".$tmppath."\"", "error");
    return false;
   }

   if(!preg_match("/^([a-z0-9\_]{1,25})$/i", $name))
   {
    $this->e->add("Illegal plugin name: \"".$name."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->name=$name;

   $plugin=new Plugin($this->name, true);
   $plugin->info();

   if($plugin->e->any("fatal"))
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   $pluginpath=$_path["project"]["plugins"].$this->name;

   $enabled=false;
   if($this->fs->exists($pluginpath))
   {
    $plugin2=new Plugin($name);

    if($plugin2->isEnabled())
    {
     $plugin2->setStatus(false);
     $enabled=true;
    }

    if(!$plugin2->remove())
    {
     $this->e->add($plugin2->e);
     $this->_removeTmp();
     return false;
    }
   }

   if(!$this->fs->rename($tmppath, $pluginpath))
   {
    $this->e->add("Unable to rename \"".$tmppath."\" to \"".$pluginpath."\"", "error");
    $this->_removeTmp();
    return false;
   }

   if($enabled)
    $plugin2->setStatus(true);

   $this->e->add("Plugin: \"".$this->name." ".$plugin->version."\" installed", "note");

   return true;
  }

  function _removeTmp()
  {
   global $_path;

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if(!$this->fs->delete($tmppath, true))
   {
    $this->e->add("Unable to remove plugin temporary import directory: \"".$tmppath."\"", "error");
    return false;
   }

   return true;
  }

  function remove()
  {
   global $_path, $_config;

   if(!$_config->userRoot())
   {
    $this->e->add("Plugins can only be removed by the root user", "error");
    return false;
   }

   if($this->type=="system")
   {
    $this->e->add("Plugin: \"".$this->name."\" is a system plugin, refusing to remove", "error");
    return false;
   }

   if($this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is enabled, refusing to remove", "error");
    return false;
   }

   if(!$this->fs->delete($this->path["root"], true))
   {
    $this->e->add("Unable to remove plugin directory: \"".$this->path["root"]."\"", "error");
    return false;
   }

   $this->e->add("Plugin: \"".$this->name."\" removed", "note");

   return true;
  }


  function isEnabled()
  {
   return $this->enabled; //already set by getStatus call in constructor
  }

  function getStatus()
  {
   if(!$this->fs->exists($this->path["status"]))
   {
    if(!$this->fs->write($this->path["status"], ""))
    {
     $this->e->add("Unable to create plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
     return false;
    }
   }

   if(($stats=$this->fs->readLines($this->path["status"], true))===false)
   {
    $this->e->add("Unable to open plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   $l=sizeof($stats);
   for($c=0;$c<$l;$c++)
   {
    $tmp=explode("=", $stats[$c]);
    if($tmp[0]==$this->name)
    {
     if($tmp[1])
     {
      $this->enabled=true;
      return true;
     }

     break;
    }
   }

   $this->enabled=false;
   return false;
  }

  function setStatus($enabled)
  {
   global $_config, $_modules, $_session;

   if(!$_config->userRoot())
   {
    $this->e->add("Plugins can only be enabled/disabled by the root user", "error");
    return false;
   }

   if($enabled)
    $enabled=1;
   else
    $enabled=0;

   $found=false;
   if(($stats=$this->fs->readLines($this->path["status"], true))===false)
   {
    $this->e->add("Unable to open plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   $l=sizeof($stats);

   for($c=0;$c<$l;$c++)
   {
    $tmp=explode("=", $stats[$c]);
    if($tmp[0]==$this->name)
    {
     if($tmp[1]==$enabled) //already on correct state
      return true;

     $found=true;
     $stats[$c]=$this->name."=".$enabled;
     break;
    }
   }

   if(!$found)
    $stats[]=$this->name."=".$enabled;

   if(!$this->fs->write($this->path["status"], implode("\n", $stats), "overwrite"))
   {
    $this->e->add("Unable to write plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   if($enabled)
    $this->enabled=true;
   else
    $this->enabled=false;

   //clear module cache (used by JAS::modules() call)
   unset($_modules[$_session->get("project")]["_modules"]);

   //clear plugin modules cache (used by JAS::modules() call)
   foreach(array_keys($_modules[$_session->get("project")]) as $module)
    if(strpos($module, $this->name."_")===0)
     unset($_modules[$_session->get("project")][$module]);

   //ensure plugin modules are recached
   if($_config->get("cache.memory")&&$_config->get("cache.memoryCacheData"))
   {
    $cache=new Cache("memory");
    foreach($cache->search("DATA::CACHE(".$_session->get("project").",".$this->name."_") as $key)
     $cache->remove($key);
   }

   return true;
  }

  function changelog($html=false)
  {
   if(!$this->path["pluginChangelog"]||!($changelog=FS::readS($this->path["pluginChangelog"])))
   {
    $this->e->add("Can't read ".$this->name." plugin changelog", "error", "Changelog file: \"".$this->path["pluginChangelog"]."\"");
    return false;
   }

   if($html) //html mode (otherwise raw text will be used
    $changelog=JAS::changelogFormatS($changelog);

   return $changelog;
  }

  function license($html=false, $secondary=false)
  {
   if(!$secondary)
    $suf="";
   else
    $suf="2";

   if(!$this->path["pluginLicense".$suf]||!($license=FS::readS($this->path["pluginLicense".$suf])))
   {
    $this->e->add("Can't read ".$this->name." plugin license", "error", "License file: \"".$this->path["pluginLicense".$suf]."\"");
    return false;
   }


   if($html) //html mode (otherwise raw text will be used)
    $license=JAS::licenseFormatS($license);

   return $license;
  }


  function info()
  {
   $succes=true;
   if($this->info===false)
   {
    if(in_array($this->name."_pluginInfo", JAS::modules("plugins", $this->name)))
    {
     $this->info=Data::cache($this->name."_pluginInfo", "view", "all", Array("db" => "xml://".$this->path["pluginInfo"], "format" => "text"));
     $this->info->clearValues();
    }
    else
    {
     $this->info=Data::cache("pluginInfo", "view", "all", Array("db" => "xml://".$this->path["pluginInfo"], "format" => "text"));
     $this->info->clearValues();
     $this->info->plugin=new Plugin($this->name); //can't use this as that will cause an endless loop with data/plugin cloning
    }

    $this->l->read("plugin_".$this->name);

    $this->info->field["plugin"]["defaultvalue"]=$this->name;
    $this->info->field["title"]["defaultvalue"]=$this->name;

    $this->info->get();
   }

   $this->info->setShowFields(array_merge($this->info->check("use", "show"), Array("version_major", "version_minor", "version_maintenance")));
   foreach($this->info->check("use", "show") as $field)
   {
    if($field!="plugin")
     $this->info->checkField($field);

    if(isset($this->$field))
    {
     $this->e->add("Plugin info field: \"".$field."\" already in use in class!", "system");
     $succes=false;
    }
    else if(($value=$this->info->getValue($field))!==false)
     $this->$field=$value;
    else
     $this->$field=$this->info->getDefaultValue($field);
   }

   if(!$succes)
   {
    $this->info=false;
    return false;
   }

   if($this->info->e->any())
    $this->e->add($this->info->e);

   if($this->license)
    if($this->fs->exists($this->path["root"].$this->license.".txt", "file"))
     $this->path["pluginLicense2"]=$this->path["root"].$this->license.".txt";

   return true;
  }

  function dependencies()
  {
   if(!$this->fs->exists($this->path["pluginDependencies"])) //no dependencies
    return true;

   $this->deps=Data::cache("pluginDependencies", "view", "all", Array("db" => "xml://".$this->path["pluginDependencies"], "format" => "text"));
   $this->deps->clearValues();
   $this->deps->plugin=new Plugin($this->name); //can't use this as that will cause an endless loop with data/plugin cloning

   $succes=true;
   if(!$this->deps->e->any())
   {
    $this->deps->get();
    $this->deps->setShowFields(array_merge($this->deps->check("use", "show"), Array("version_major", "version_minor", "version_maintenance")));

    while($this->deps->next())
    {
     if($this->deps->v["required"])
     {
      $error="error";
      $required="required";
     }
     else
     {
      $error="warning";
      $required="preferred";
     }

     if(!$this->deps->v["plugin"]) //JAS dependency
     {
      if(!($version=explode(".", JAS::versionS())))
      {
       $this->e->add("Dependency ".$error.", JAS version can't be determined", $error);
       $succes=false;
      }
      else
      {
       if(!isset($version[1]))
        $version[1]=0;
       if(!isset($version[2]))
        $version[2]=0;

       if(
        $version[0]<$this->deps->v["version_major"]||
        ($version[0]==$this->deps->v["version_major"]&&$version[1]<$this->deps->v["version_minor"])||
        ($version[0]==$this->deps->v["version_major"]&&$version[1]==$this->deps->v["version_minor"]&&$version[2]<$this->deps->v["version_maintenance"])
       )
       {
        $this->e->add("Dependency ".$error.", JAS version \"".$this->deps->v["version"]."\" is ".$required.", version \"".JAS::versionS()."\" found", $error);
        $succes=false;
       }
      }
     }
     else
     {
      if(!in_array($this->deps->v["plugin"], JAS::plugins()))
      {
       $this->e->add("Dependency ".$error.", \"".$this->deps->v["plugin"]."\" plugin not found", $error);
       $succes=false;
      }
      else
      {
       $tmp=new Plugin($this->deps->v["plugin"]);
       $tmp->info();

       if(
        $tmp->version_major<$this->deps->v["version_major"]||
        ($tmp->version_major==$this->deps->v["version_major"]&&$tmp->version_minor<$this->deps->v["version_minor"])||
        ($tmp->version_major==$this->deps->v["version_major"]&&$tmp->version_minor==$this->deps->v["version_minor"]&&$tmp->version_maintenance<$this->deps->v["version_maintenance"])
       )
       {
        $this->e->add("Dependency ".$error.", \"".$this->deps->v["plugin"]."\" plugin version \"".$this->deps->v["version"]."\" is ".$required.", version \"".$tmp->version."\" found", $error);
        $succes=false;
       }
       else if(!$tmp->isEnabled())
       {
        $this->e->add("Dependency ".$error.", \"".$this->deps->v["plugin"]."\" plugin is disabled", $error);
        $succes=false;
       }
      }
     }
    }
   }

   if(!$succes)
    return false;

   return true;
  }

  function config($key, $id=false)
  {
   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to read config", "error");
    return false;
   }

   if(in_array($this->name."_pluginConfigs", JAS::modules("plugins", $this->name)))
    $config=Data::cache($this->name."_pluginConfigs", "view", "all", Array("format" => "text"));
   else
    $config=Data::cache("pluginConfigs", "view", "all", Array("format" => "text"));

   $config->clearValues();
   $config->setValue("plugin", $this->name);

   if(!$id)
    $config->setValue("key", $key);
   else
    $config->setValue("nr", $key);

   if(!$config->get())
   {
    $this->e->add($config->e);
    return false;
   }

   if(!$config->next())
   {
    $this->e->add("Plugin configuration: \"".$key."\" not found", "error");
    return false;
   }

   $this->config=$config->v;
   return true;
  }


  function checkClass($class)
  {
   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::checkClass failed");
    return false;
   }

   if(!$this->fs->exists($this->path["root"].$this->class[$class]["filename"]))
   {
    $this->e->add("Plugin ".$class." file: \"".$this->class[$class]["filename"]."\" not found for: \"".$this->name."\" plugin", "debug");

    if($class=="setup") //no setup found, use default setup
    {
     $this->class[$class]["filename"]=false;
     $this->class[$class]["name"]=$this->class[$class]["base"];
     $this->class[$class]["base"]=false;
    }
    else
     return false;
   }

   if($this->class[$class]["filename"]&&strpos(file_get_contents($this->path["root"].$this->class[$class]["filename"]), "class ".$this->class[$class]["name"])===false)
   {
    $this->e->add("Plugin ".$class." class: \"".$this->class[$class]["name"]."\" not found for: \"".$this->name."\" plugin", "plugin");
    return false;
   }

   $this->class[$class]["defined"]=true;
   return true;
  }

  function classExists($class)
  {
   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::classExists failed");
    return false;
   }

   return $this->class[$class]["defined"];
  }

  function initClass($class)
  {
   global $_path, $_jasPath;

   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::initClass failed");
    return false;
   }

   if(!$this->classExists($class))
   {
    $this->e->add("Plugin ".$class." class: \"".$this->class[$class]["name"]."\" not defined for: \"".$this->name."\" plugin", "plugin");
    return false;
   }

   if($this->class[$class]["filename"])
    require_once($this->path["root"].$this->class[$class]["filename"]);

   if(!class_exists($this->class[$class]["name"]))
   {
    $this->e->add("Plugin ".$class." class: \"".$this->class[$class]["name"]."\" not found for: \"".$this->name."\" plugin", "plugin");
    return false;
   }

   switch($class)
   {
    case "frontend":
    {
     $this->class[$class]["object"]=new $this->class[$class]["name"]($this, $this->config);
     break;
    }
    case "backend":
    {
     $this->class[$class]["object"]=new $this->class[$class]["name"]($this);
     break;
    }
    case "setup":
    {
     $args=func_get_args();
     array_shift($args);

     $this->class[$class]["object"]=new $this->class[$class]["name"]($this, (isset($args[0])?$args[0]:false), (isset($args[1])?$args[1]:false));
     break;
    }
    default:
     return false;
   }

   if($this->class[$class]["base"]&&!in_array($this->class[$class]["base"], class_parents($this->class[$class]["object"])))
   {
    $this->e->add("Plugin ".$class." class: \"".$this->class[$class]["name"]."\" isn't a ".$this->class[$class]["base"]." class for: \"".$this->name."\" plugin", "plugin");
    return false;
   }

   return true;
  }

  function getClass($class)
  {
   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::callExists failed");
    return false;
   }

   if(!$this->class[$class]["object"])
    if(!$this->initClass($class))
     return false;

   return $this->class[$class]["object"];
  }


  function callExists($class, $method)
  {
   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::callExists failed");
    return false;
   }

   if(!$this->class[$class]["object"])
    if(!$this->initClass($class))
     return false;

   if(!method_exists($this->class[$class]["name"], $method))
    return false;

   return true;
  }

  function call($class, $method)
  {
   global $_config;

   if(!isset($this->class[$class]))
   {
    $this->e->add("Unknown plugin class type: \"".$class."\"", "system", "Plugin::call failed");
    return false;
   }

   if(!$this->callExists($class, $method))
   {
    $this->e->add("Plugin ".$class." class method: \"".$this->class[$class]["name"]."->".$method."\" not defined for: \"".$this->name."\" plugin", "error");
    return false;
   }

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to call ".$class." class method: \"".$this->class[$class]["name"]."->".$method."\"", "error");
    return false;
   }

   if($class=="setup"&&!$_config->userRoot())
   {
    $this->e->add("Plugins can only be setup by the root user", "error");
    return false;
   }

   $args=func_get_args();
   array_shift($args);
   array_shift($args);

   return call_user_func_array(Array($this->class[$class]["object"], $method), $args);
  }


  function baseURL($url)
  {
   $this->baseURL=$url;
   return true;
  }

  function addVar($name, $value)
  {
   if(isset($this->vars[$name]))
    $this->e->add("Overwriting variable: \"".$name."\"", "warning");

   $this->vars[$name]=$value;
   return true;
  }

  function url()
  {
   $url=$this->baseURL;

   $url.="?";
   foreach($this->vars as $k => $v)
    $url.=$k."=".$v."&amp;";

   return $url;
  }
 }


 /*
  *
  * Base_Plugin
  */

 class Base_Plugin
 {
  function __construct(&$plugin, &$config)
  {
   $this->plugin=&$plugin;
   $this->config=&$config;
   $this->e=&$plugin->e;
  }

  function main()
  {
   return true;
  }

  function output()
  {
   if($this->e->any("fatal"))
    return false;

   return "";
  }
 }


 /*
  *
  * Base_Plugin_Backend
  */

 class Base_Plugin_Backend
 {
  function __construct(&$plugin)
  {
   $this->plugin=&$plugin;
   $this->e=&$plugin->e;
  }

  function main()
  {
   return true;
  }

  function output()
  {
   if($this->e->any("fatal"))
    return false;

   return "";
  }
 }


 /*
  *
  * Base_Plugin_Setup
  */

 class Base_Plugin_Setup
 {
  function __construct(&$plugin, $force=false, $deSetup=false)
  {
   $this->plugin=&$plugin;
   $this->e=&$plugin->e;
   $this->fs=new FS();

   $this->force=$force;
   $this->deSetup=$deSetup;

   $this->clear();
  }

  function clear()
  {
   $this->directories=Array();
   $this->files=Array();
   $this->modules=Array();

   $this->done=true;
  }

  function done()
  {
   return $this->done;
  }

  function get($type)
  {
   switch($type)
   {
    case "directories":
     return $this->directories;
    case "files":
     return $this->files;
    case "modules":
     return $this->modules;
    default:
     break;
   }

   $this->e->add("Can't get unknown Plugin Setup information: \"".$type."\"", "setup");
   return false;
  }


  function mkdir($directory)
  {
   if(!$this->deSetup)
   {
    if($this->fs->exists($directory))
    {
     if(!$this->fs->exists($directory, "directory"))
     {
      $status="Directory already exists as file";
      $this->done=false;
     }
     else
      $status="Setup correctly";
    }
    else
    {
     $status="Setup";
     $this->done=false;
    }
   }
   else //deSetup
   {
    if($this->fs->exists($directory))
    {
     if(!$this->fs->exists($directory, "directory"))
      $status="Directory is a file";
     else
      $status="DeSetup";
    }
    else
     $status="Not setup";

    $this->done=false;
   }

   $this->directories[]=Array(
    "directory" => $directory,
    "status" => $status,
   );

   return true;
  }

  function file($source, $destination, $symlink=false)
  {
   if(substr($destination, -1)=="/")
    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $source);

   if(!$this->fs->exists($this->plugin->path["root"].$source))
   {
    $status="File not found";
    $this->done=false;
   }
   else if(!$this->deSetup)
   {
    if($this->fs->exists($destination))
    {
     if($this->force)
     {
      $status="Forced setup";
      $this->done=false;
     }
     else
     {
      if($this->fs->size($this->plugin->path["root"].$source)==$this->fs->size($destination))
       $status="Setup correctly";
      else
      {
       $status="File already exists";
       $this->done=false;
      }
     }
    }
    else
    {
     $status="Setup";
     $this->done=false;
    }
   }
   else //deSetup
   {
    if($this->fs->exists($destination))
    {
     if($this->fs->size($this->plugin->path["root"].$source)==$this->fs->size($destination))
      $status="DeSetup";
     else
      $status="File changed";
    }
    else
     $status="Not setup";

    $this->done=false;
   }

   $this->files[]=Array(
    "source" => $source,
    "destination" => $destination,
    "symlink" => ($symlink),
    "status" => $status,
   );

   return true;
  }

  function modules()
  {
   $modules=func_get_args();
   $pluginModules=JAS::modules("plugins", $this->plugin->name);

   if(!$modules)
   {
    if($pluginModules)
     return $this->modules($pluginModules);
    else
     return true;
   }
   else
   {
    if(is_array($modules[0]))
     $modules=$modules[0];

    foreach($modules as $module)
    {
     if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $module))
      $module=$this->plugin->name."_".$module;

     if($this->deSetup)
     {
      $this->modules[]=Array("module" => $module, "status" => "DeSetup not available");
      continue;
     }

     if($module==$this->plugin->name."_pluginInfo"||$module==$this->plugin->name."_pluginDependencies")
      $this->modules[]=Array("module" => $module, "status" => "Skipped");
     else if(!in_array($module, $pluginModules))
     {
      $this->modules[]=Array("module" => $module, "status" => "Non plugin module");
      $this->done=false;
     }
     else
     {
      $d=Data::cache($module, "database", "create", Array("skipCheckTables" => true));
      if($d->tableExists())
      {
       if($this->force)
        $this->modules[]=Array("module" => $module, "status" => "Forced setup");
       else
        $this->modules[]=Array("module" => $module, "status" => "Setup correctly");
      }
      else
      {
       $this->modules[]=Array("module" => $module, "status" => "Setup");
       $this->done=false;
      }

      foreach(array_keys($d->reference) as $ref)
      {
       if($d->reference[$ref]["type"]=="*:*")
       {
        $d2=&$d->reference[$ref]["data"];
        if(!$d2->tableExists())
        {
         $this->modules[sizeof($this->modules)-1]["status"]="Setup";
         $this->done=false;
        }
        unset($d2);
       }
      }

      foreach(array_keys($d->extends) as $parentModule)
      {
       $d2=Data::cache($parentModule, "database", false, Array("skipCheckTables" => true));
       if(!$d2->tableExists())
       {
        $this->modules[]=Array("module" => $d2->module, "status" => "Setup");
        $this->done=false;
       }
       unset($d2);
      }

      unset($d);
     }
    }
   }

   return true;
  }


  function init()
  {
   if(!$this->modules())
    return false;

   return true;
  }

  function exec()
  {
   if(!$this->deSetup)
   {
    if(method_exists($this, "preSetup"))
    {
     $return=$this->preSetup();

     if($return===NULL)
      $this->e->add("Pre Setup call doesn't have a return value", "debug");
     else if($return)
      $this->e->add("Pre Setup call successful", "note");
     else if(!$return)
     {
      $this->e->add("Pre Setup call failed", "error");
      return false;
     }
    }

    foreach($this->directories as $dir)
    {
     if($dir["status"]=="Setup"||$dir["status"]=="Forced setup")
     {
      if(!$this->fs->mkdir($dir["directory"]))
       $this->e->add("Unable to create directory: \"".$dir["directory"]."\"", "system");
     }
    }

    foreach($this->files as $file)
    {
     if($file["status"]=="Setup"||$file["status"]=="Forced setup")
     {
      if($file["symlink"])
      {
       if(!$this->fs->symlink($this->plugin->path["root"].$file["source"], $file["destination"]))
        $this->e->add("Unable to symlink file: \"".$file["source"]."\" to: \"".$file["destination"]."\"", "system");
      }
      else //copy
      {
       if(!$this->fs->copy($this->plugin->path["root"].$file["source"], $file["destination"]))
        $this->e->add("Unable to copy file: \"".$file["source"]."\" to: \"".$file["destination"]."\"", "system");
      }
     }
    }

    foreach($this->modules as $module)
    {
     if($module["status"]=="Setup"||$module["status"]=="Forced setup")
     {
      $d=Data::cache($module["module"], "database", "create", Array("skipCheckTables" => true));

      if($d->tableExists())
       $d->extra="alter";

      if(!$d->submit())
       $this->e->add($d->e);

      foreach(array_keys($d->reference) as $ref)
      {
       if($d->reference[$ref]["type"]=="*:*")
       {
        $d2=$d->getReferenceData($ref);
        $d2->setAction("database");
        $d2->setExtra("create");

        if($d2->tableExists())
         $d2->extra="alter";

        if(!$d2->submit())
         $this->e->add($d2->e);
       }
      }
     }
    }

    if(method_exists($this, "postSetup"))
    {
     $return=$this->postSetup();

     if($return===NULL)
      $this->e->add("Post Setup call doesn't have a return value", "debug");
     else if($return)
      $this->e->add("Post Setup call successful", "note");
     else if(!$return)
     {
      $this->e->add("Post Setup call failed", "error");
      return false;
     }
    }
   }
   else //deSetup
   {
    if(method_exists($this, "preDeSetup"))
    {
     $return=$this->preDeSetup();

     if($return===NULL)
      $this->e->add("Pre DeSetup call doesn't have a return value", "debug");
     else if($return)
      $this->e->add("Pre DeSetup call successful", "note");
     else if(!$return)
     {
      $this->e->add("Pre DeSetup call failed", "error");
      return false;
     }
    }

    foreach($this->files as $file)
    {
     if($file["status"]=="DeSetup")
     {
      if(!$this->fs->delete($file["destination"]))
       $this->e->add("Unable to remove file: \"".$file["destination"]."\"", "system");
     }
    }

    foreach($this->directories as $dir)
    {
     if($dir["status"]=="DeSetup")
     {
      if($this->fs->files($dir["directory"], true))
       $this->e->add("Not removing directory: \"".$dir["directory"]."\", not empty", "warning");
      else if(!$this->fs->delete($dir["directory"]))
       $this->e->add("Unable to remove directory: \"".$dir["directory"]."\"", "system");
     }
    }

    if(method_exists($this, "postDeSetup"))
    {
     $return=$this->postDeSetup();

     if($return===NULL)
      $this->e->add("Post DeSetup call doesn't have a return value", "debug");
     else if($return)
      $this->e->add("Post DeSetup call successful", "note");
     else if(!$return)
     {
      $this->e->add("Post DeSetup call failed", "error");
      return false;
     }
    }
   }

   if($this->e->any("fatal"))
    return false;

   if($this->deSetup)
    $this->e->add("DeSetup successful", "note");
   else if($this->force)
    $this->e->add("Setup forced successfully", "note");
   else
    $this->e->add("Setup successful", "note");

   return true;
  }
 }

?>
