<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/headers.sro.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/headers.php");
 require_once($_path["include"]."headers.php");

 class UT_Headers_Scope_Resolution_Operator extends UT_Headers
 {
  function CallStatic()
  {
   $test=false;
   foreach(headers_list() as $header)
   {
    if(preg_match("/^([a-z\-]+): .+$/i", $header, $m))
    {
     $test=$m[1];
     break;
    }
   }

   if(!$test)
   {
    $this->e->add("No headers found", "warning");
    return UT_SKIPPED;
   }

   if(!Headers::getS($test))
   {
    $this->e->add("GetS ".$test." header shouldn't fail", "error");
    return UT_FAILED;
   }

   /*
   if(Headers::nonexistingexecutableS())
   {
    $this->e->add("nonexistingexecutableS should fail", "error");
    return UT_FAILED;
   }
   */

   return UT_OK;
  }


  function Request()
  {
   if(function_exists("getallexpected"))
    $expected=getallexpected();
   else
   {
    $expected=Array();
    foreach($_SERVER as $name => $value)
     if(substr($name, 0, 5) == "HTTP_")
      $expected[str_replace(" ", "-", ucwords(strtolower(str_replace("_", " ", substr($name, 5)))))]=$value;
   }

   if(!($headers=Headers::requestS()))
   {
    $this->e->add("No request headers found", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $headers))
   {
    $this->e->add("Request header don't match expected output", "error");
    return UT_FAILED;
   }

   //use headers in _SERVER
   if(!($headers=Headers::requestS(true)))
   {
    $this->e->add("No request headers found", "error");
    return UT_FAILED;
   }

   if(!$this->_compare($expected, $headers))
   {
    $this->e->add("Request header don't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Set()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::setS("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("key: value", headers_list()))
   {
    $this->e->add("Adding new header failed", "error", "Header not found");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Exists()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::setS("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::existsS("key"))
   {
    $this->e->add("Exists new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::existsS("KEY"))
   {
    $this->e->add("Exists new header upper case failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::existsS("ASDASDASDASD"))
   {
    $this->e->add("Exists new header should have failed failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Get()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::setS("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("key")===false)
   {
    $this->e->add("Get new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("key")!="value")
   {
    $this->e->add("Get new header failed", "error", "Value doesn't match");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //array
   if(($headers=Headers::getS())===false)
   {
    $this->e->add("Get headers failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   foreach(headers_list() as $header)
   {
    list($key, $value)=explode(": ", $header, 2);

    if(!isset($headers[$key]))
    {
     $this->e->add("Get headers failed", "error", "Key not found");

     $this->_restoreHeaders();
     return UT_FAILED;
    }

    if($headers[$key]!=$value)
    {
     $this->e->add("Get headers failed", "error", "Value doesn't match");

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Delete()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   if(!function_exists("header_remove"))
   {
    $this->e->add("Header_remove function doesn't exist", "warning");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::setS("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::deleteS("key"))
   {
    $this->e->add("Delete header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("key")!==false)
   {
    $this->e->add("Delete header failed", "error", "Key still exists");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //all headers
   if(!Headers::setS("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::deleteS())
   {
    $this->e->add("Delete headers failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS())
   {
    $this->e->add("Delete headers failed", "error", "Keys still exist");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Basic()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //no-cache
   if(!Headers::basicS())
   {
    $this->e->add("Adding basic headers failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Cache key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", Headers::getS("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", Headers::getS("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::getS("Pragma")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Pragma key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(Headers::getS("Pragma")!="private")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(Headers::getS("Pragma")!="no-cache")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::getS("Expires")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Expires key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding basic headers failed", "error", "Expires value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("X-Frame-Options")!=="DENY")
   {
    $this->e->add("Adding basic headers failed", "error", "X-Frame-Options value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Frames()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //DENY
   if(!Headers::framesS(false))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("X-Frame-Options")===false)
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("X-Frame-Options")!=="DENY")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //SAMEORIGIN
   if(!Headers::framesS(true))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("X-Frame-Options")!=="SAMEORIGIN")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //ALL
   if(!Headers::framesS("all"))
   {
    $this->e->add("Adding frames header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("X-Frame-Options")!=="")
   {
    $this->e->add("Adding frames header failed", "error", "X-Frame-Options value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Cache()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //no-cache
   if(!Headers::cacheS())
   {
    $this->e->add("Adding cache header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", Headers::getS("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", Headers::getS("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::getS("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(Headers::getS("Pragma")!="private")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(Headers::getS("Pragma")!="no-cache")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
      $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::getS("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //private+store+now
   if(!Headers::cacheS("private", true, time(), false))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")!="private, max-age=0")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Pragma")!="private")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")!=gmdate("D, d, M Y H:i:s T"))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //public+no-store+now
   if(!Headers::cacheS("public", false, 10, true))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Cache-Control")!="public, no-store, max-age=10")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Pragma")!="public")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Expires")!=gmdate("D, d, M Y H:i:s T", time()+10))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Filename()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //filename
   if(!Headers::filenameS("file.tst"))
   {
    $this->e->add("Adding filename attachment header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding filename attachment header failed", "error", "File key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding filename attachment header failed", "error", "File value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //filename inline
   if(!Headers::filenameS("file.tst", "inline"))
   {
    $this->e->add("Adding filename inline header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding filename inline header failed", "error", "File key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="inline; filename=\"file.tst\"")
   {
    $this->e->add("Adding filename inline header failed", "error", "File value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //filename+no attachment
   if(!Headers::filenameS("file.tst", false))
   {
    $this->e->add("Adding filename header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding filename header failed", "error", "File key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="filename=\"file.tst\"")
   {
    $this->e->add("Adding filename header failed", "error", "File value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //no filename
   if(Headers::filenameS(""))
   {
    $this->e->add("Adding filename header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Type()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //type
   if(!Headers::typeS("text/html"))
   {
    $this->e->add("Adding type header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Type")===false)
   {
    $this->e->add("Adding type header failed", "error", "Size key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr(Headers::getS("Content-Type"), "text/html"))
   {
    $this->e->add("Adding type header failed", "error", "Type value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid type
   if(Headers::typeS("text"))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no type
   if(Headers::typeS(""))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Size()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //type
   if(!Headers::sizeS(2346))
   {
    $this->e->add("Adding size header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Length")===false)
   {
    $this->e->add("Adding size header failed", "error", "Size key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Length")!=2346)
   {
    $this->e->add("Adding size header failed", "error", "Size value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid size
   if(Headers::typeS(-2346))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::typeS("text"))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no siz3
   if(Headers::typeS(""))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function File()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //filename
   if(!Headers::fileS("file.tst"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "File key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "File value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //filename inline
   if(!Headers::fileS("file.tst", false, false, "inline"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "File key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="inline; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "File value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type
   $this->_restoreHeaders();

   if(!Headers::fileS("file.tst", "text/html"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr(Headers::getS("Content-Type"), "text/html"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+auto type
   $this->_restoreHeaders();

   if(!Headers::fileS("file.css", true))
   {
    $this->e->add("Adding file header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="attachment; filename=\"file.css\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr(Headers::getS("Content-Type"), "text/css"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type+size
   $this->_restoreHeaders();

   if(!Headers::fileS("file.tst", "text/html", 245345))
   {
    $this->e->add("Adding file header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!strstr(Headers::getS("Content-Type"), "text/html"))
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Length")===false)
   {
    $this->e->add("Adding file header failed", "error", "Size key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Content-Length")!=245345)
   {
    $this->e->add("Adding file header failed", "error", "Size value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Server()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //empty server
   if(!Headers::serverS())
   {
    $this->e->add("Adding server header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Server")!="")
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //jas+apache+php+ssl
   if(!Headers::serverS(true, true, true, true))
   {
    $this->e->add("Adding server header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ Apache/[0-9\.]+ \([^\)]+\) OpenSSL/[0-9a-z\.]+ PHP/[0-9\.]+$!", Headers::getS("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //custom
   if(!Headers::serverS(false, false, false, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^Test/[0-9\.]+$!", Headers::getS("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //custom jas+php+custom
   if(!Headers::serverS("JAS/1.0", false, true, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::getS("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ PHP/[0-9\.]+ Test/[0-9\.]+$!", Headers::getS("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }

 }

?>
