<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/history.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."history.php");

 class UT_History extends UT_Classes
 {
  function _init()
  {
   $this->db="session://unit_testing_history/global";
  }

  function Constructor()
  {
   $this->_init();

   $object=new History();

   if(!is_object($object)||get_class($object)!="History")
   {
    $this->e->add("Constructor doesn't return a History object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->limit!=10)
   {
    $this->e->add("History limit \"10\"");
    return UT_FAILED;
   }

   if($object->db!="session://history/global")
   {
    $this->e->add("History limit \"session://history/global\"");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new History();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("History object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->limit=-1;
   if($orig->limit==$copy->limit)
   {
    $this->e->add("History object copy also modified original \"limit\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("History object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }



  function SetLimit()
  {
   $hs=new History();

   if(!$hs->setLimit(123))
   {
    $this->e->add("History set limit failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->limit!=123)
   {
    $this->e->add("History limit not set");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetDB()
  {
   $hs=new History();

   if(!$hs->setDB($this->db))
   {
    $this->e->add("History set DB failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->db!=$this->db)
   {
    $this->e->add("History DB not set");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function SetProject()
  {
   //get custom project
   $project="jas";
   foreach(JAS::projects() as $tmp)
   {
    if($tmp!="jas")
    {
     $project=$tmp;
     break;
    }
   }


   $hs=new History();

   //set project
   if(!$hs->setProject($project))
   {
    $this->e->add("History set project failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->project!=$project)
   {
    $this->e->add("History project not set");
    return UT_FAILED;
   }

   //illegal project
   if($hs->setProject("ASDASDASDASD"))
   {
    $this->e->add("History set project should fail");
    return UT_FAILED;
   }

   if($hs->project=="ASDASDASDASD")
   {
    $this->e->add("History project shouldn't be set");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function LockProject()
  {
   global $_historyLockProject, $_session;

   $hs=new History();

   //unlock project
   if(!$hs->lockProject())
   {
    $this->e->add("History lock project failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->project!=$_session->get("project"))
   {
    $this->e->add("History project not set");
    return UT_FAILED;
   }

   if($_historyLockProject!=$_session->get("project"))
   {
    $this->e->add("History project lock not set");
    return UT_FAILED;
   }


   //check if project is still set
   $hs=new History();

   if($hs->project!=$_session->get("project"))
   {
    $this->e->add("History project not set to locked project");
    return UT_FAILED;
   }


   //unlock project
   if(!$hs->lockProject(false))
   {
    $this->e->add("History unlock project failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->project!=$_session->get("project"))
   {
    $this->e->add("History project not set");
    return UT_FAILED;
   }

   if($_historyLockProject)
   {
    $this->e->add("History project lock shouldn't be set");
    return UT_FAILED;
   }


   //get custom project
   $project=false;
   foreach(JAS::projects() as $tmp)
   {
    if($tmp!="jas")
    {
     $project=$tmp;
     break;
    }
   }

   if(!$project)
   {
    $this->e->add("Can't find non \"jas\" project, required for lock tests");
    return UT_SKIPPED;
   }

   //lock specific project
   if(!$hs->lockProject($project))
   {
    $this->e->add("History lock project failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->project!=$project)
   {
    $this->e->add("History project not set");
    return UT_FAILED;
   }

   if($_historyLockProject!=$project)
   {
    $this->e->add("History project lock not set");
    return UT_FAILED;
   }


   //check if project is still set
   $hs=new History();

   if($hs->project!=$project)
   {
    $this->e->add("History project not set to locked project");
    return UT_FAILED;
   }


   //illegal project
   if($hs->lockProject("ASDASDASDASD"))
   {
    $this->e->add("History lock project should fail");
    return UT_FAILED;
   }

   if($hs->project=="ASDASDASDASD")
   {
    $this->e->add("History project shouldn't be set");
    return UT_FAILED;
   }

   if($_historyLockProject=="ASDASDASDASD")
   {
    $this->e->add("History project shouldn't be locked");
    return UT_FAILED;
   }

   if($hs->project!=$project)
   {
    $this->e->add("History project not set to locked project");
    return UT_FAILED;
   }


   //unlock project
   if(!$hs->lockProject(false))
   {
    $this->e->add("History unlock project failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->project!=$project)
   {
    $this->e->add("History project not set");
    return UT_FAILED;
   }

   if($_historyLockProject)
   {
    $this->e->add("History project lock shouldn't be set");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function Clear() 
  {
   $hs=new History();
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add("Clear history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if values have been submitted
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($d->vmax!=0)
   {
    $this->e->add("Clear history failed, still ".$d->vmax." values left");
    return UT_FAILED;
   }


   //store history
   $d->setAction("add");
   $d->setExtra("item");
   $d->clearValues();

   $c=0;
   $d->setDefaultValue("uri", $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   $c++;
   $d->setDefaultValue("uri", $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear())
   {
    $this->e->add("Clear history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if values have been submitted
   $d->setAction("view");
   $d->setExtra("all");
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($d->vmax!=0)
   {
    $this->e->add("Clear history failed, still ".$d->vmax." values left");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Store() 
  {
   $hs=new History();
   $hs->setLimit(2);
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //store history
   if(!$hs->store())
   {
    $this->e->add("Store history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if values have been submitted correctly
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($d->vmax==0)
   {
    $this->e->add("Store history failed, values not submitted");
    return UT_FAILED;
   }

   if($d->vmax>1)
   {
    $this->e->add("Store history failed, too many values submitted");
    return UT_FAILED;
   }

   $d->next();

   if(html_entity_decode($d->v["uri"])!=$_SERVER["REQUEST_URI"])
   {
    $this->e->add("Store history failed, request URI doesn't match: \"".$_SERVER["REQUEST_URI"]."\"");
    return UT_FAILED;
   }

   if($d->v["time"]!=gmdate($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"]))
   {
    $this->e->add("Store history failed, request time doesn't match: \"".date($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"])."\"");
    return UT_FAILED;
   }

   if($d->v["method"]!=$_SERVER["REQUEST_METHOD"])
   {
    $this->e->add("Store history failed, request method doesn't match: \"".$_SERVER["REQUEST_METHOD"]."\"");
    return UT_FAILED;
   }

   if($d->v["cms"])
   {
    $this->e->add("Store history failed, request shouldn't have cms flag set");
    return UT_FAILED;
   }


   //store history
   if(!$hs->store())
   {
    $this->e->add("Store history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   if(!$hs->store())
   {
    $this->e->add("Store history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //check if values have been submitted correctly
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($d->vmax!=2)
   {
    $this->e->add("Store history failed, auto clean should leave 2 values");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //store history, with cms flag
   if(!$hs->store(true))
   {
    $this->e->add("Store history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if values have been submitted correctly
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($d->vmax==0)
   {
    $this->e->add("Store history failed, values not submitted");
    return UT_FAILED;
   }

   if($d->vmax>1)
   {
    $this->e->add("Store history failed, too many values submitted");
    return UT_FAILED;
   }

   $d->next();

   if(html_entity_decode($d->v["uri"])!=$_SERVER["REQUEST_URI"])
   {
    $this->e->add("Store history failed, request URI doesn't match: \"".$_SERVER["REQUEST_URI"]."\"");
    return UT_FAILED;
   }

   if($d->v["time"]!=gmdate($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"]))
   {
    $this->e->add("Store history failed, request time doesn't match: \"".date($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"])."\"");
    return UT_FAILED;
   }

   if($d->v["method"]!=$_SERVER["REQUEST_METHOD"])
   {
    $this->e->add("Store history failed, request method doesn't match: \"".$_SERVER["REQUEST_METHOD"]."\"");
    return UT_FAILED;
   }

   if(!$d->v["cms"])
   {
    $this->e->add("Store history failed, request should have cms flag set");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function CMS() 
  {
   $hs=new History();
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //store history
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if values have been submitted correctly
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $d->next();

   if($d->v["cms"])
   {
    $this->e->add("Store history failed, request shouldn't have cms flag set");
    return UT_FAILED;
   }

   //set CMS flag
   if(!$hs->cms())
   {
    $this->e->add("CMS history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $d->next();

   if(html_entity_decode($d->v["uri"])!=$_SERVER["REQUEST_URI"])
   {
    $this->e->add("Store history failed, request URI doesn't match: \"".$_SERVER["REQUEST_URI"]."\"");
    return UT_FAILED;
   }

   if($d->v["time"]!=gmdate($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"]))
   {
    $this->e->add("Store history failed, request time doesn't match: \"".date($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"])."\"");
    return UT_FAILED;
   }

   if($d->v["method"]!=$_SERVER["REQUEST_METHOD"])
   {
    $this->e->add("Store history failed, request method doesn't match: \"".$_SERVER["REQUEST_METHOD"]."\"");
    return UT_FAILED;
   }

   if(!$d->v["cms"])
   {
    $this->e->add("Store history failed, request should have cms flag set");
    return UT_FAILED;
   }

   //unset CMS flag
   if(!$hs->cms(false))
   {
    $this->e->add("CMS history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $d->next();

   if(html_entity_decode($d->v["uri"])!=$_SERVER["REQUEST_URI"])
   {
    $this->e->add("Store history failed, request URI doesn't match: \"".$_SERVER["REQUEST_URI"]."\"");
    return UT_FAILED;
   }

   if($d->v["time"]!=gmdate($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"]))
   {
    $this->e->add("Store history failed, request time doesn't match: \"".date($d->field["time"]["format"][$d->dataformat]["datetimeformat"], $_SERVER["REQUEST_TIME"])."\"");
    return UT_FAILED;
   }

   if($d->v["method"]!=$_SERVER["REQUEST_METHOD"])
   {
    $this->e->add("Store history failed, request method doesn't match: \"".$_SERVER["REQUEST_METHOD"]."\"");
    return UT_FAILED;
   }

   if($d->v["cms"])
   {
    $this->e->add("Store history failed, request shouldn't have cms flag set");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function IsAlreadyStored() 
  {
   $hs=new History();
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //check if history is already stored
   if($hs->isAlreadyStored())
   {
    $this->e->add("Current history shouldn't already be stored");
    return UT_FAILED;
   }

   //store history
   if(!$hs->store())
   {
    $this->e->add("Store history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   //check if history is already stored
   if(!$hs->isAlreadyStored())
   {
    $this->e->add("Current history should already be stored");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetTotal() 
  {
   $hs=new History();
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $ret=$hs->getTotal();
   if($ret!==0)
   {
    $this->e->add("Get total should return \"0\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   //store history
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   if($hs->getTotal()!=3)
   {
    $this->e->add("Get total should return \"3\"");
    return UT_FAILED;
   }

   //store history
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   $ret=$hs->getTotal();
   if($ret!==6) 
   {
    $this->e->add("Get total should return \"6\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function CleanUp() 
  {
   $hs=new History();
   $hs->setDB($this->db);

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //store custom history
   $d=Data::cache("history", "add", "item", Array("db" => $this->db));
   $d->clearValues();

   $c=0;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //check if all values have been submitted
   $ret=$hs->getTotal();
   if($ret!==4)
   {
    $this->e->add("Get total should return \"4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   //clean up history
   if(!$hs->cleanUp())  //no limit set yet
   {
    $this->e->add("Clean up history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if all values have been submitted
   $ret=$hs->getTotal();
   if($ret!==4)
   {
    $this->e->add("Get total should return \"4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $hs=new History();
   $hs->setLimit(3);
   $hs->setDB($this->db); 

   //clean up history
   if(!$hs->cleanUp()) 
   {
    $this->e->add("Clean up history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if first value has been clean up
   $ret=$hs->getTotal();
   if($ret!==3)
   {
    $this->e->add("Get total should return \"3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   while($d->next())
   {
    if($d->v["uri"]=="?uri=1")
    {
     $this->e->add("Clean up failed, first entry still availble");
     return UT_FAILED;
    }
   }


   //store custom history
   $d=Data::cache("history", "add", "item", Array("db" => $this->db));
   $d->clearValues();

   $c=0;
   $d->setValue("uri", "?uri=".($c+5), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+5), $c);
   $d->setDefaultValue("time", $c);
   $d->setDefaultValue("method", $c);
   $d->setDefaultValue("project", $c);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //check if all values have been submitted
   $ret=$hs->getTotal();
   if($ret!==5)
   {
    $this->e->add("Get total should return \"5\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }


   //clean up history
   if(!$hs->cleanUp()) 
   {
    $this->e->add("Clean up history failed");
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //check if first value has been clean up
   $ret=$hs->getTotal();
   if($ret!==3)
   {
    $this->e->add("Get total should return \"3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   $d->clearValues();

   if(!$d->get())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   while($d->next())
   {
    if($d->v["uri"]=="?uri=1"||$d->v["uri"]=="?uri=2"||$d->v["uri"]=="?uri=3")
    {
     $this->e->add("Clean up failed, first 3 entries still availble");
     return UT_FAILED;
    }
   }


   //clear history
   if(!$hs->clear())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Previous()
  {
   $hs=new History();
   $hs->setDB($this->db); 

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //get custom project
   $project=false;
   foreach(JAS::projects() as $tmp)
   {
    if($tmp!="jas")
    {
     $project=$tmp;
     break;
    }
   }


   //store custom history
   $d=Data::cache("history", "add", "item", Array("db" => $this->db));
   $d->clearValues();

   $c=0;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "POST", $c);
   $d->setDefaultValue("project", $c);
   $d->setDefaultValue("cms", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);

   if($project) //custom project
    $d->setValue("project", $project, $c);
   else
    $d->setDefaultValue("project", $c);

   $d->setDefaultValue("cms", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);
   $d->setDefaultValue("project", $c);
   $d->setValue("cms", true, $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);
   $d->setDefaultValue("project", $c);
   $d->setDefaultValue("cms", $c);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }


   //check previous value
   if($project) //project specific entry (2)
   {
    //second and third skipped due to project+cms filtering
    $ret=$hs->previous();
    if($ret!=="?uri=4") 
    {
     $this->e->add("Previous in history should be \"?uri=4\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(4);
    if($ret!=="?uri=1") 
    {
     $this->e->add("Previous in history should be \"?uri=1\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(5);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }
   else
   {
     //third skipped due to cms filtering
    $ret=$hs->previous();
    if($ret!=="?uri=4") 
    {
     $this->e->add("Previous in history should be \"?uri=4\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(2);
    if($ret!=="?uri=2") 
    {
     $this->e->add("Previous in history should be \"?uri=2\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(3);
    if($ret!=="?uri=1") 
    {
     $this->e->add("Previous in history should be \"?uri=1\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(4);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }


   //GET methods, all projects, no cms filtering: should skip first
   $ret=$hs->previous(1, "GET", false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, "GET", false, false);
   if($ret!=="?uri=3")
   {
    $this->e->add("Previous in history should be \"?uri=3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, "GET", false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, "GET", false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //POST methods, all projects, no cms filtering: should skip all but first
   $ret=$hs->previous(1, "POST", false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, "POST", false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }



   //all methods, custom project: should skip second entry (and third due to cms filtering)
   if($project) 
   {
    //check previous value
    $ret=$hs->previous(1, false, true);
    if($ret!=="?uri=4")
    {
     $this->e->add("Previous in history should be \"?uri=4\"");

     if($ret===false) //failure
      $this->e->add($hs->e);

     return UT_FAILED;
    }

    $ret=$hs->previous(4, false, true);
    if($ret!=="?uri=1")
    {
     $this->e->add("Previous in history should be \"?uri=1\"");

     if($ret===false) //failure
      $this->e->add($hs->e);

     return UT_FAILED;
    }

    $ret=$hs->previous(5, false, true);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }


   //all methods, all projects (should skip third due to cms filtering)
   $ret=$hs->previous(1, false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //all methods, all projects, cms filtering: should skip third 
   $ret=$hs->previous(1, false, false, true);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false, true);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false, true);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false, true);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //all methods, all projects, no cms filtering
   $ret=$hs->previous(1, false, false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, false, false, false);
   if($ret!=="?uri=3")
   {
    $this->e->add("Previous in history should be \"?uri=3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PreviousWithCurrentPage()
  {
   $hs=new History();
   $hs->setDB($this->db); 

   //clear history, probably cleared already
   if(!$hs->clear()) 
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }


   //get custom project
   $project=false;
   foreach(JAS::projects() as $tmp)
   {
    if($tmp!="jas")
    {
     $project=$tmp;
     break;
    }
   }


   //store custom history
   $d=Data::cache("history", "add", "item", Array("db" => $this->db));
   $d->clearValues();

   $c=0;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "POST", $c);
   $d->setDefaultValue("project", $c);
   $d->setDefaultValue("cms", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);

   if($project) //custom project
    $d->setValue("project", $project, $c);
   else
    $d->setDefaultValue("project", $c);

   $d->setDefaultValue("cms", $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);
   $d->setDefaultValue("project", $c);
   $d->setValue("cms", true, $c);

   $c++;
   $d->setValue("uri", "?uri=".($c+1), $c);
   $d->setDefaultValue("time", $c);
   $d->setValue("method", "GET", $c);
   $d->setDefaultValue("project", $c);
   $d->setDefaultValue("cms", $c);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }


   //XXX: difference with Previous function
   if(!$hs->store())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }
   //XXX: difference with Previous function


   //check previous value
   if($project) //project specific entry (2)
   {
    //second and third skipped due to project+cms filtering
    $ret=$hs->previous();
    if($ret!=="?uri=4") 
    {
     $this->e->add("Previous in history should be \"?uri=4\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(4);
    if($ret!=="?uri=1") 
    {
     $this->e->add("Previous in history should be \"?uri=1\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(5);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }
   else
   {
     //third skipped due to cms filtering
    $ret=$hs->previous();
    if($ret!=="?uri=4") 
    {
     $this->e->add("Previous in history should be \"?uri=4\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(2);
    if($ret!=="?uri=2") 
    {
     $this->e->add("Previous in history should be \"?uri=2\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(4);
    if($ret!=="?uri=1") 
    {
     $this->e->add("Previous in history should be \"?uri=1\"");
 
     if($ret===false) //failure
      $this->e->add($hs->e);
 
     return UT_FAILED;
    }

    $ret=$hs->previous(5);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }


   //GET methods, all projects, no cms filtering: should skip first
   $ret=$hs->previous(1, "GET", false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, "GET", false, false);
   if($ret!=="?uri=3")
   {
    $this->e->add("Previous in history should be \"?uri=3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, "GET", false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, "GET", false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //POST methods, all projects, no cms filtering: should skip all but first
   $ret=$hs->previous(1, "POST", false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, "POST", false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //all methods, custom project: should skip second entry (and third due to cms filtering)
   if($project) 
   {
    //check previous value
    $ret=$hs->previous(1, false, true);
    if($ret!=="?uri=4")
    {
     $this->e->add("Previous in history should be \"?uri=4\"");

     if($ret===false) //failure
      $this->e->add($hs->e);

     return UT_FAILED;
    }

    $ret=$hs->previous(4, false, true);
    if($ret!=="?uri=1")
    {
     $this->e->add("Previous in history should be \"?uri=1\"");

     if($ret===false) //failure
      $this->e->add($hs->e);

     return UT_FAILED;
    }

    $ret=$hs->previous(5, false, true);
    if($ret!==false)
    {
     $this->e->add("Previous in history should not exist");
     return UT_FAILED;
    }
   }


   //all methods, all projects (should skip third due to cms filtering)
   $ret=$hs->previous(1, false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //all methods, all projects, cms filtering: should skip third 
   $ret=$hs->previous(1, false, false, true);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false, true);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false, true);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false, true);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //all methods, all projects, no cms filtering
   $ret=$hs->previous(1, false, false, false);
   if($ret!=="?uri=4")
   {
    $this->e->add("Previous in history should be \"?uri=4\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(2, false, false, false);
   if($ret!=="?uri=3")
   {
    $this->e->add("Previous in history should be \"?uri=3\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(3, false, false, false);
   if($ret!=="?uri=2")
   {
    $this->e->add("Previous in history should be \"?uri=2\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(4, false, false, false);
   if($ret!=="?uri=1")
   {
    $this->e->add("Previous in history should be \"?uri=1\"");

    if($ret===false) //failure
     $this->e->add($hs->e);

    return UT_FAILED;
   }

   $ret=$hs->previous(5, false, false, false);
   if($ret!==false)
   {
    $this->e->add("Previous in history should not exist");
    return UT_FAILED;
   }


   //clear history
   if(!$hs->clear())
   {
    $this->e->add($hs->e);
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
