<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.session.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/storage.php");
 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."storage/session.php");
 require_once($_path["include"]."data.php");

 class UT_Storage_Session extends UT_Storage
 {
  var $_db="session";

  function Constructor()
  {
   $d=new Data($this->_module, false, false, Array("db" => $this->_db));
   $object=new System_Storage_session($d);

   if(!is_object($object)||get_class($object)!="System_Storage_session")
   {
    $this->e->add("Constructor doesn't return a System_Storage_session object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $d=Data::cache(false, false, false, Array("db" => "session"));
   $orig=new System_Storage_session($d);
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("System_Storage_session object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->type="different";
   if($orig->type==$copy->type)
   {
    $this->e->add("System_Storage_session object copy also modified original \"type\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("System_Storage_session object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("System_Storage_session object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("System_Storage_session object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("System_Storage_session object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->data->format=Array();
   if($orig->data->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_session object copy didn't modify original \"Data\" object", "error");
    return UT_FAILED;
   }

   if($d->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_session object copy didn't modify referenced \"Data\" object", "error");
    return UT_FAILED;
   }

   $copy->storage=Array();
   if($orig->storage!=$copy->storage)
   {
    $this->e->add("System_Storage_session object copy didn't modify original \"storage\" reference", "error");
    return UT_FAILED;
   }

   return UT_Storage::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function _init()
  {
   global $_path;

   if(!isset($this->_var))
   {
    $d=new Data($this->_module, false, false, Array("db" => $this->_db));
    $obj=new Storage($d, false);
    $db=$obj->getConfig();

    if(isset($db["var"]))
     $this->_var=$db["var"];
    else if(isset($db["host"]))
     $this->_var=$db["host"];
    else
    {
     $this->e->add("Database configuration: \"".$this->_db."\" incorrect: \"var\" parameter not set", "system");
     return false;
    }
   }

   if(!isset($this->_dataSet))
   {
    $this->_dataSet=Array(
     "categories" => Array(
      Array("nr" => 1, "catnr" => 0, "updated" => "2010-04-09 09:35:32", "name" => "cat1", "module" => ""),
      Array("nr" => 2, "catnr" => 1, "updated" => "2010-04-10 10:46:33", "name" => "cat2", "module" => $this->_module),
      Array("nr" => 3, "catnr" => 1, "updated" => "2010-04-11 11:57:34", "name" => "cat3", "module" => $this->_module),
     ),
     $this->_module => Array(
      Array("nr" => 1, "catnr" => 2, "updated" => "2010-04-09 09:35:32", "name" => "test1"),
      Array("nr" => 2, "catnr" => 2, "updated" => "2010-04-10 10:46:33", "name" => "test2"),
      Array("nr" => 3, "catnr" => 3, "updated" => "2010-04-11 11:57:34", "name" => "test3"),
     ),
    );
   }

   return true;
  }

  function GetItem()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $fields=Array("nr", "name", "updated");

   $d=new Data($this->_module, "view", "item", Array("db" => $this->_db));
   $d->setShowFields($fields);
   $d->setValue("nr", "2");
   $d->sortOrder("nr");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->get())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($s->results)!=1)
   {
    $this->e->add("Result set should contain 1 entry", "error");
    return UT_FAILED;
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[1][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $storage=Array(); //clean up

   return UT_OK;
  }

  function GetList()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $fields=Array("nr", "name", "updated");

   $d=new Data($this->_module, "view", "list", Array("db" => $this->_db));
   $d->setShowFields($fields);
   $d->setValue("catnr", "2");
   $d->sortOrder("nr");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->get())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($s->results)!=2)
   {
    $this->e->add("Result set should contain 2 entries", "error");
    return UT_FAILED;
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[0][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[1][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $storage=Array(); //clean up

   return UT_OK;
  }

  function GetAll()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $fields=Array("nr", "name", "updated");

   $d=new Data($this->_module, "view", "all", Array("db" => $this->_db));
   $d->setShowFields($fields);
   $d->sortOrder("nr");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->get())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($s->results)!=3)
   {
    $this->e->add("Result set should contain 3 entries", "error");
    return UT_FAILED;
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[0][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[1][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $s->next();
   foreach($fields as $field)
   {
    if($s->res[$field]!=$storage[2][$field])
    {
     $this->e->add("Field \"".$field."\" in result set doesn't match original data", "error");
     return UT_FAILED;
    }
   }

   $storage=Array(); //clean up
   return UT_OK;
  }

  function GetAggregation()
  {
   $d=new Data($this->_module, "view", "all", Array("db" => $this->_db));
   $d->setValue("aggregation_field", "catnr");
   $d->setValue("aggregation_type", "average");
   $d->setValue("aggregation_method", "around");
   $d->setValue("aggregation_interval", 1);

   $s=new System_Storage_session($d);

   if($s->get())
   {
    $this->e->add("Getting aggregated data should fail, not support in class", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Add()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $d=new Data($this->_module, "add", "item", Array("db" => $this->_db));
   $d->setValue("nr", "1234");
   $d->setValue("catnr", "2");
   $d->setValue("name", "test4");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->add())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($storage)!=4)
   {
    $this->e->add("Storage should contain 4 entries", "error");
    return UT_FAILED;
   }

   if($storage[3]["nr"]!="4")
   {
    $this->e->add("Field \"nr\" in new entry should be \"4\"", "error");
    return UT_FAILED;
   }
   if($storage[3]["name"]!="test4")
   {
    $this->e->add("Field \"name\" in new entry should be \"test4\"", "error");
    return UT_FAILED;
   }
   if($storage[3]["catnr"]!="2")
   {
    $this->e->add("Field \"catnr\" in new entry should be \"2\"", "error");
    return UT_FAILED;
   }

   $storage=Array(); //clean up
   return UT_OK;
  }

  function AddInsertID()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $d=new Data($this->_module, "add", "item", Array("db" => $this->_db, "format" => "sql"));
   $d->setValue("nr", "1234");
   $d->setValue("catnr", "2");
   $d->setValue("name", "test4");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->add())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($storage)!=4)
   {
    $this->e->add("Storage should contain 4 entries", "error");
    return UT_FAILED;
   }

   if($storage[3]["nr"]!="1234")
   {
    $this->e->add("Field \"nr\" in new entry should be \"4\"", "error");
    return UT_FAILED;
   }
   if($storage[3]["name"]!="test4")
   {
    $this->e->add("Field \"name\" in new entry should be \"test4\"", "error");
    return UT_FAILED;
   }
   if($storage[3]["catnr"]!="2")
   {
    $this->e->add("Field \"catnr\" in new entry should be \"2\"", "error");
    return UT_FAILED;
   }

   $storage=Array(); //clean up
   return UT_OK;
  }

  function Edit()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $d=new Data($this->_module, "edit", "item", Array("db" => $this->_db));
   $d->setValue("nr", "2");
   $d->setValue("catnr", "4");
   $d->setValue("name", "test4");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->edit())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($storage)!=3)
   {
    $this->e->add("Storage should contain 3 entries", "error");
    return UT_FAILED;
   }

   if($storage[1]["nr"]!="2")
   {
    $this->e->add("Field \"nr\" in changed entry should be \"2\"", "error");
    return UT_FAILED;
   }
   if($storage[1]["name"]!="test4")
   {
    $this->e->add("Field \"name\" in changed entry should be \"test4\"", "error");
    return UT_FAILED;
   }
   if($storage[1]["catnr"]!="4")
   {
    $this->e->add("Field \"catnr\" in changed entry should be \"4\"", "error");
    return UT_FAILED;
   }

   $storage=Array(); //clean up
   return UT_OK;
  }

  function Delete()
  {
   global $_session;

   if(!$this->_init())
    return UT_FAILED;

   $d=new Data($this->_module, "", "item", Array("db" => $this->_db));
   $d->setValue("nr", "2");

   $s=new System_Storage_session($d);

   $storage=&$_session->getReference($s->session.".".$this->_var.".".$this->_module);
   $storage=$this->_dataSet[$this->_module];

   if(!$s->delete())
   {
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(sizeof($storage)!=2)
   {
    $this->e->add("Storage should contain 2 entries", "error");
    return UT_FAILED;
   }

   for($c=0;$c<sizeof($storage);$c++)
   {
    if($storage[$c]["nr"]=="2")
    {
     $this->e->add("Entry 2 should be deleted", "error");
     return UT_FAILED;
    }
   }

   $storage=Array(); //clean up
   return UT_OK;
  }

 }

?>
