<?

 /*****************************************************************************
 * Filename: include/storage/api.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["protocol"]."http.php");

 require_once($_path["api"]."data.php");
 require_once($_path["api"]."csv.php");
 require_once($_path["api"]."xml.php");

 class System_Storage_api extends Storage
 {
  function __construct(&$d)
  {
   global $_session;

   parent::__construct($d, "api");

   if(!$_session->get("apiStorage.".$_session->get("project")))
   {
    $_session->set("apiStorage.".$_session->get("project"), Array(
     "session" => false,
     "project" => false,
     "modules" => Array(),
    ));
   }

   $types=Array("data", "csv", "xml"); //supported API types, first is default

   if(!($db=$this->getConfig()))
    return;

   if(!$db["host"])
   {
    $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
    return;
   }

   if(isset($db["vars"]["type"])&&in_array(strtolower($db["vars"]["type"]), $types))
    $type=strtolower($db["vars"]["type"]);
   else
    $type=reset($types);

   $this->url=(isset($db["vars"]["secure"])?"https":"http")."://".$db["host"].($db["path"]?$db["path"]:"/")."api/".$type."/";
   $this->session=&$_session->getReference("apiStorage.".$_session->get("project").".session"); //referenced: ensures it's stored on change

   if($type=="xml")
    $this->api=new API_XML("client");
   else if($type=="csv")
    $this->api=new API_CSV("client");
   else //data
    $this->api=new API_Data("client");

   if(!($this->http=$this->_getConnection($db["hash"]))) //get existing connection
   {
    $this->http=new Protocol_HTTP();
    $this->_setConnection($db["hash"], $this->http); //store connection for later use

    //switch project
    if(isset($db["project"])&&!$_session->get("apiStorage.".$_session->get("project").".project")) //project specified and project not yet set
    {
     if(!$this->_request("project", $db["project"])) //switch project
      return;

     $_session->set("apiStorage".$_session->get("project").".project", true);
    }
   }

   $this->clear();
  }

  function __clone()
  {
   parent::__clone();

   //clone objects
   $this->api=clone $this->api;
   $this->http=clone $this->http;
  }


  function _request($type, &$request, $count=false)
  {
   //prepare request
   $this->api->clear();

   if($type=="data") //data object
   {
    $request->db="default"; //overwrite DB
    $request->dataformat="api"; //overwrite dataformat
    if(!$this->api->addData($request, $count))
    {
     $this->e->add($this->api->e);
     return false;
    }
   }
   else if($type=="module") //module info
   {
    if(!$this->api->addModules(Array($request)))
    {
     $this->e->add($this->api->e);
     return false;
    }
   }
   else if($type=="project") //set project
   {
    if(!$this->api->addProjects($request))
    {
     $this->e->add($this->api->e);
     return false;
    }
   }
   else
   {
    $this->e->add("Unknown request type".$type, "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!$this->api->format()) //format request
   {
    for($c=0;$c<$this->api->e->getErrors();$c++) //save errors as system errors
     $this->e->add($this->api->e->getError($c), $this->api->e->getLevel($c), $this->api->e->getExtra($c), false, ($this->data->module!="log")); //don't log

    $this->api->e->clear();
    return false;
   }

   if(!$this->http->setSession($this->session)) //set session id (unset if no active session)
   {
    for($c=0;$c<$this->http->e->getErrors();$c++) //save errors as system errors
     $this->e->add($this->http->e->getError($c), $this->http->e->getLevel($c), $this->http->e->getExtra($c), false, ($this->data->module!="log")); //don't log

    $this->http->e->clear();
    return false;
   }

   if(!$this->http->setVar("request", $this->api->output()))
   {
    for($c=0;$c<$this->http->e->getErrors();$c++) //save errors as system errors
     $this->e->add($this->http->e->getError($c), $this->http->e->getLevel($c), $this->http->e->getExtra($c), false, ($this->data->module!="log")); //don't log

    $this->http->e->clear();
    return false;
   }

   //send request
   if(!$this->http->request($this->url, "POST", true))
   {
    for($c=0;$c<$this->http->e->getErrors();$c++) //save errors as system errors
     $this->e->add($this->http->e->getError($c), $this->http->e->getLevel($c), $this->http->e->getExtra($c), false, ($this->data->module!="log")); //don't log

    $this->http->e->clear();
    return false;
   }

   if($session=$this->http->getSession()) //server provided session (new) id
    $this->session=$session;

   //handle response
   $this->api->clear();
   $this->api->setRequestResponseHeaders($this->http->get("response.headers"));
   if(!$this->api->parse($this->http->get("response.body")))
   {
    for($c=0;$c<$this->api->e->getErrors();$c++) //save errors as system errors
     $this->e->add($this->api->e->getError($c), $this->api->e->getLevel($c), $this->api->e->getExtra($c), false, ($this->data->module!="log")); //don't log

    $this->api->e->clear();
    return false;
   }

   //add response errors
   $this->e->add($this->api->e);
   $this->api->e->clear();

   if($this->e->any("fatal")) //fatal
    return false;

   //reference data object to response
   if($type=="data")
    $request=$this->api->commands[0]["data"];

   return true;
  }


  function insertID($forcedID=false)
  {
  }


  function checkRights($action)
  {
   global $_config;

   if($_config->userRoot())
    return true;

   $this->e->add("No ".$action." rights"."!", "hack", false, ($this->data->module!="log")); //don't log

   return false;
  }


  function table()
  {
   global $_session;

   //module already check this session?
   if($_session->get("apiStorage.".$_session->get("project").".modules.".$this->data->module))
    return true;

   //get module
   $module=($this->data->initializingModule?$this->data->initializingModule.":":"").$this->data->module; //added initializing module as prefix: initModule:module
   if(!$this->_request("module", $module))
    return;

   $_session->set("apiStorage.".$_session->get("project").".modules.".$this->data->module, true); //store module ok
   return true;
  }


  function info()
  {
   $this->clear();
   $d=clone $this->data; //copy data object

   if(!$this->_request("data", $d)) //handle data request: result in $d
    return false;

   $fields=Array();
   foreach($d->check("use", "info") as $field)
    $fields[]=$field;

   $l=sizeof($fields);

   for($c=0;$c<$d->vmax;$c++)
   {
    $this->results[$c]=Array();

    for($x=0;$x<$l;$x++)
     $this->results[$c][$fields[$x]]=$d->field[$fields[$x]]["value"][$c];

    if(isset($this->results[$c]["info_type"]))
     $this->results[$c]["info_type"]="JAS API ".$this->api->version;
   }

   return true;
  }


  function treeUp($type="data"){return $this->get($type);}
  function treeDown($type="data"){return $this->get($type);}
  function get($type="data")
  {
   $this->clear();
   $d=clone $this->data; //copy data object

   if($type=="count") //don't get results, only show_count
   {
    $d->setValue("show_total", 1);
    $d->setShowFields("nr");
   }

   if($d->action=="tree") //tree up/down
    $d->field["treedepth"]["use"]=""; //don't get treedepth field, will be calculated locally if needed
   else if($d->action=="user"&&$d->extra=="login") //user login handled in login function
   {
    //get actual login information
    $d->setAction("view");
    $d->setExtra("all");
   }
   else if($d->action=="add"||$d->action=="copy"||$d->action=="edit"||$d->action=="delete") //manipulation actions
   {
    //get item
    $d->setAction("view");
    $d->setExtra("item");
   }

   if(!$this->_request("data", $d, ($type=="count"||$type=="both"?true:false))) //handle data request: result in $d
    return;

   if($type=="count"||$type=="both")
   {
    $this->count=$d->getValue("show_count");

    if($type=="count")
     return true;
   }

   //get data
   $fields=Array();
   foreach($d->check("use", "sql") as $field)
    if($field!="r")
     if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
      $fields[]=$field;

   $l=sizeof($fields);

   for($c=0;$c<$d->vmax;$c++)
   {
    $this->results[$c]=Array();

    for($x=0;$x<$l;$x++)
    {
     if(isset($d->field[$fields[$x]]["value"][$c]))
      $this->results[$c][$fields[$x]]=$d->field[$fields[$x]]["value"][$c];
     else
      $this->results[$c][$fields[$x]]=NULL;
    }
   }

   return true;
  }


  function add(){return $this->edit();}
  function delete(){return $this->edit();}
  function edit()
  {
   //HACK: don't add userRights+groupRights, done server side
   if(
    $this->data->module=="userRights"||
    $this->data->module=="groupRights"
   )
    return true;

   $this->clear();
   $d=clone $this->data; //copy data object

   if($d->action=="copy")
    $d->setAction("add");

   if(!$this->_request("data", $d)) //handle data request: result in $d
    return;

   return true;
  }


  function logout(){return $this->login();}
  function login()
  {
   $this->clear();
   $d=clone $this->data; //copy data object

   if(!$this->_request("data", $d)) //handle data request: result in $d
    return;

   return true;
  }


 }
?>
