<?

 /*****************************************************************************
 * Filename: include/storage/xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["format"]."xml.php");

 class System_Storage_xml extends Storage
 {
  function __construct(&$d)
  {
   global $_xmlStorage, $_session;

   parent::__construct($d, "xml");

   $this->gotLock=false;
   $this->lockedRetries=100;
   $this->lockedTimeout=.01*1000000;

   $this->path=false;
   $this->lockFile=false;
   $this->dataFile=false;
   $this->indexFile=false;
   $this->filedataDirectory=false;
   $this->filedataPrefix=false;

   $this->cache=false;

   if(!($db=$this->getConfig()))
    return;

   if(!$db["host"])
   {
    $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
    return;
   }

   $this->path=$db["host"].$db["path"]; //merge path

   //setup files
   if(preg_match("/\.xml$/i", $this->path, $m)) //direct filename
   {
    $this->dataFile=$this->path;

    foreach($this->data->check("type", "filedata") as $k)
    {
     if($this->data->check("use", "sql", $k))
     {
      $this->e->add("File fields not supported for single file based database: \"".$m[1]."\"", "system", false, ($this->data->module!="log")); //don't log
      return;
     }
    }
   }
   else
   {
    if(!file_exists($this->path))
    {
     if(!@mkdir($this->path))
     {
      $this->e->add("Unable to create storage directory: \"".$this->path."\"", "system", false, ($this->data->module!="log")); //don't log
      return;
     }
    }
    else if(!is_dir($this->path))
    {
     $this->e->add("Storage directory: \"".$this->path."\" isn't a directory", "system", false, ($this->data->module!="log")); //don't log
     return;
    }

    $this->lockFile=$this->path."/".$this->data->module.".lock";
    $this->dataFile=$this->path."/".$this->data->module.".data.xml";
    $this->indexFile=false; //XXX: currently not maintained
    //$this->indexFile=$this->path."/".$this->data->module.".index.xml";
    $this->filedataDirectory=$this->path."/".$this->data->module."/";
    $this->filedataPrefix="xml_";
   }

   //get cached version if available
   if(!isset($_xmlStorage[$_session->get("project")]))
    $_xmlStorage[$_session->get("project")]=Array();
   if(!isset($_xmlStorage[$_session->get("project")][$this->dataFile]))
    $_xmlStorage[$_session->get("project")][$this->dataFile]=Array("updated" => false, "xml" => false);

   $this->cache=&$_xmlStorage[$_session->get("project")][$this->dataFile];

   $this->clear();
  }


  function insertID($forcedID=false)
  {
   if($this->type=="fake")
    return 0;

   $d=&$this->data; //D for short

   if(!$this->gotLock)
   {
    $this->e->add("Refusing to get insertID, haven't got lock for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!($xml=$this->read("data")))
    return false;

   if(($insertID=$xml->getAttribute("jas/data", "insertID"))===false)
   {
    $this->e->add("No insertID attribute in data tag in XML storage: \"".$this->dataFile."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if($forcedID) //overwrite saved ID
   {
    if($insertID<$forcedID) //new insert id, increase saved ID
    {
     $insertID=$forcedID;
     $saveID=$insertID+1;
    }
    else //forcedID in saved range
    {
     $insertID=$forcedID;
     $saveID=false;
    }
   }
   else //based on saved ID
    $saveID=$insertID+1;

   //check primary key violation
   if(!($id=$d->getID()))
   {
    $this->e->add("Unable to get primary key for table: \"".$d->module."\"", "module", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if($this->indexFile)
   {
    $values=Array($id => $insertID);

    if(($index=$this->getIndex("PRIMARY", $values))===false) //unable to get index
     return false;

    $isUnique=($index?false:true);
   }
   else
   {
    $isUnique=true;
    $xml->reset();
    while(($x=$xml->search("row"))!==false)
    {
     $tmp=$xml->getOuterXML($x);

     while(($x2=$tmp->search("field"))!==false)
     {
      if($tmp->getAttribute($x2, "name")==$id)
      {
       if($tmp->getAttribute($x2, "value")==$insertID)
       {
        $isUnique=false;
        break;
       }
      }
     }
    }
   }

   if(!$isUnique)
   {
    $this->e->add("Primary key violation for table: \"".$d->module."\", value \"".$insertID."\" already exists", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if($saveID)
    if(($xml->setAttribute("jas/data", "insertID", $saveID))===false)
     return false;

   if(!$this->write($xml))
    return false;

   return $insertID;
  }


  function checkRights($action)
  {
   global $_config;

   if($_config->userRoot())
    return true;

   $this->e->add("No ".$action." rights"."!", "hack", false, ($this->data->module!="log")); //don't log

   return false;
  }


  function table()
  {
   if($this->dataFile&&file_exists($this->dataFile))
    return true;

   return false;
  }


  function info()
  {
   $d=&$this->data; //D for short

   if($this->dataFile&&file_exists($this->dataFile))
   {
    $current=date_default_timezone_get();
    date_default_timezone_set($d->format["sql"]["timezone"]);
    $results["info_created"]=date($d->format["sql"]["datetimeformat"], FS::getTimeS($this->dataFile, "created"));
    $results["info_updated"]=date($d->format["sql"]["datetimeformat"], FS::getTimeS($this->dataFile, "modified"));
    date_default_timezone_set($current);

    if(!($xml=$this->read("data")))
     return false;

    $results["info_type"]="JAS XML ".$xml->getAttribute("jas", "version");

    $results["info_rows"]=$xml->getAttribute("jas/data", "rows");
    $results["info_insertID"]=$xml->getAttribute("jas/data", "insertID");

    $this->clear();
    $this->type="fake";
    $this->results=Array($results);

    return true;
   }

   return false;
  }


  function get($gather="data")
  {
   $this->clear();

   $d=&$this->data; //D for short

   if($d->check("valuetype", "set", "aggregation_field"))
   {
    $this->e->add($this->type." storage doesn't support: \"aggregation\"", "system", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!($xml=$this->read("data")))
    return false;

   //XXX: add index based subset selection here

   if(isset($xml->_dataSetCache))
    $set=&$xml->_dataSetCache;
   else
   {
    $setSize=0;
    $set=Array();
    while(($x=$xml->search("row"))!==false)
    {
     $set[$setSize]=Array();

     while(($x2=$xml->next(false, false, false))!==false&&$xml->getName($x2)=="field")
      $set[$setSize][$xml->getAttribute($x2, "name")]=$xml->getAttribute($x2, "value");

     $xml->vc=$x+1; //reset internal iterator for mainloop
     $setSize++;
    }

    $xml->_dataSetCache=&$set;
   }

   //get extended module values
   if($id=$d->getID())
   {
    $extendedModules=Array();
    foreach(array_keys($d->field) as $field)
     if($d->field[$field]["module"]!="empty"&&$d->field[$field]["module"]!=$d->module&&!in_array($d->field[$field]["module"], $extendedModules))
      $extendedModules[]=$d->field[$field]["module"];

    $l=sizeof($set);
    foreach($extendedModules as $module)
    {
     $tmp=Data::cache($module, "view", "all", Array("format" => "sql"));
     $tmp->setShowFields($tmp->check("use", "sql"));
     $tmp->clearValues();
     $tmp->get();

     while($tmp->next())
     {
      for($c=0;$c<$l;$c++)
      {
       if($tmp->v[$id]==$set[$c][$id])
       {
        $set[$c]=array_merge($tmp->v, $set[$c]);
        break;
       }
      }
     }
    }
   }

   if(!$this->_filterDataset($set, $gather))
    return false;

   if(!$this->_gatherFilterRights())
    return false;

   if($gather=="count"||$gather=="both")
   {
    if(!$this->_setCount())
     return false;

    if($gather=="count")
     return true;
   }

   if(!$this->_sort())
    return false;

   if(!$this->_filterShowTotal())
    return false;

   if($l=sizeof($this->results))
   {
    foreach(array_keys($this->results[0]) as $k)
    {
     if($d->field[$k]["type"]=="filedata"&&$this->filedataDirectory!==false)
     {
      for($c=0;$c<$l;$c++)
      {
       if(!file_exists($this->filedataDirectory.$this->results[$c][$k]))
       {
        $this->e->add("Unable to read file data, file: \"".$this->filedataDirectory.$this->results[$c][$k]."\" doesn't exist", "error", false, ($this->data->module!="log")); //don't log
        return false;
       }

       $this->results[$c][$k]=FS::readS($this->filedataDirectory.$this->results[$c][$k]);
      }
     }
    }
   }

   return true;
  }

  function add()
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("add"))) //nothing to add... ?
    return true;

   $row=new Format_XML();
   $row->parse("  <row>\n</row>\n");

   if(!$this->lock())
    return false;

   if($id=$d->getID())
   {
    if($d->check("type", "nr", $id)) //get unique ID for number fields
    {
     if(($d->dataformat=="sql"||$d->extends)) //allow force id insertion
      $insertID=$d->sqlValue($id);
     else
      $insertID=false;

     if(!($insertID=$this->insertID($insertID)))
     {
      $this->unlock();
      return false;
     }

     $d->setValue($id, $insertID, $d->vc);
    }
    else //non number field, just use value
     $insertID=$d->getValue($id, $d->vc);

    $tag="\n   <field name=\"".$id."\" value=\"";
    $tag.=htmlentities($insertID, ENT_COMPAT|ENT_XML1);
    $tag.="\"/>";
    $row->addInsideAtBegin(0, $tag); //add at begin, speeds up ID finding
   }

   foreach($values as $k => $v)
   {
    if($d->check("properties", "identifier", $k)) //skip id, already done
     continue;

    $tag="   <field name=\"".$k."\" value=\"";

    if($d->check("type", "filedata", $k))
     $tag.=htmlentities($this->filedataPrefix.$k."_".$insertID, ENT_COMPAT|ENT_XML1);
    else
     $tag.=htmlentities($v, ENT_COMPAT|ENT_XML1);

    $tag.="\"/>\n";

    $row->addInsideAtEnd(0, $tag);
   }

   $row->addInsideAtEnd(0, "  ");

   if(!($xml=$this->read("data")))
   {
    $this->unlock();
    return false;
   }

   $xml->addInsideAtEnd("jas/data", $row->output(false));

   $rows=$xml->getAttribute("jas/data", "rows");

   if(!$xml->setAttribute("jas/data", "rows", $rows+1))
   {
    $this->unlock();
    return false;
   }

   if(!$this->write($xml))
   {
    $this->unlock();
    return false;
   }

   foreach($d->check("type", "filedata") as $k)
   {
    if($d->check("module", $d->module, $k)) //don't update fields from extended tables
    {
     if(!$this->filedata($this->filedataPrefix.$k."_".$insertID, $d->field[$k]["value"][$d->vc])) //store (non base64) raw data
     {
      $this->unlock();
      return false;
     }
    }
   }

   if($this->indexFile)
   {
    //XXX: doesn't work
    if(!$this->updateIndexes($insertID))
    {
     $this->unlock();
     return false;
    }
   }

   if(!$this->unlock())
    return false;

   $this->e->add($d->title." inserted successfully", "note");
   return true;
  }

  function edit($looseFiltering=false)
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("edit"))) //nothing to update
    return true;

   if(!$this->lock())
    return false;

   if(!($xml=$this->read("data")))
   {
    $this->unlock();
    return false;
   }

   $save=false;
   $filter=Array();

   if(!$looseFiltering) //identifier based
   {
    if(!($id=$d->getID()))
    {
     $this->e->add("Unable to get primary key for table: \"".$d->module."\"", "module");
     $this->unlock();
     return false;
    }

    $filter[$id]=$d->sqlValue($id);
   }
   else //looseFiltering: watch what you're doing!
   {
    $id=$d->getID();

    foreach($d->check("valuetype", "set") as $k)
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
      $filter[$k]=$d->sqlValue($k);
   }

   if(!$filter)
   {
    if(!$looseFiltering) //identifier based
     $this->e->add("No identifiers set for updating!", "sql");
    else //looseFiltering: watch what you're doing!
     $this->e->add("Update all not permitted!", "sql");

    $this->unlock();
    return false;
   }

   $filters=sizeof($filter);
   while(($x=$xml->search("row"))!==false)
   {
    $matches=0;
    while(($x2=$xml->next(false, false, false))!==false&&$xml->getName($x2)=="field")
    {
     $field=$xml->getAttribute($x2, "name");
     if(isset($filter[$field])&&$xml->getAttribute($x2, "value")==$filter[$field])
      $matches++;
    }
    $xml->vc=$x+1; //reset internal iterator for mainloop

    if($matches==$filters) //all filters match
    {
     //update all fields
     while(($x2=$xml->next(false, false, false))!==false&&$xml->getName($x2)=="field")
     {
      $k=$xml->getAttribute($x2, "name");

      if($k==$id) //get idValue for filedata update, also ensure id isn't overwritten
       $idValue=$xml->getAttribute($x2, "value");
      else if(isset($values[$k]))
       if($d->field[$k]["type"]!="filedata")
        $xml->setAttribute($x2, "value", $values[$k]);
     }

     foreach(array_keys($values) as $k) //only update if valuetype!=none (checked above)
     {
      if($d->field[$k]["type"]=="filedata")
      {
       if(!$this->filedata($this->filedataPrefix.$k."_".$idValue, $d->field[$k]["value"][$d->vc])) //store (non base64) raw data
       {
        $this->unlock();
        return false;
       }
      }
     }

     $xml->vc=$x+1; //reset internal iterator for mainloop
     $save=true;

     if(!$looseFiltering) //identifier based
      break;
    }
   }

   if($save)
   {
    if(!$this->write($xml))
    {
     $this->unlock();
     return false;
    }

    if($this->indexFile)
    {
     //XXX: doesn't work
     if(!$this->updateIndexes($idValue))
     {
      $this->unlock();
      return false;
     }
    }
   }

   if(!$this->unlock())
    return false;

   $this->e->add($d->title." updated successfully", "note");
   return true;
  }

  function delete($looseFiltering=false)
  {
   $d=&$this->data; //D for short

   if(!$this->lock())
    return false;

   if(!($xml=$this->read("data")))
   {
    $this->unlock();
    return false;
   }

   $save=false;
   $filter=Array();

   if(!$looseFiltering) //identifier based
   {
    if(!($id=$d->getID()))
    {
     $this->e->add("Unable to get primary key for table: \"".$d->module."\"", "module");
     $this->unlock();
     return false;
    }

    $filter[$id]=$d->sqlValue($id);
   }
   else //looseFiltering: watch what you're doing!
   {
    $id=$d->getID();

    foreach($d->check("valuetype", "set") as $k)
     if($d->check("module", $d->module, $k)&&$d->check("use", "sql", $k))
      $filter[$k]=$d->sqlValue($k);
   }

   if(!$filter)
   {
    if(!$looseFiltering) //identifier based
     $this->e->add("No identifiers set for updating!", "sql");
    else //looseFiltering: watch what you're doing!
     $this->e->add("Update all not permitted!", "sql");

    $this->unlock();
    return false;
   }

   $delete=Array();
   $filters=sizeof($filter);
   while(($x=$xml->search("row"))!==false)
   {
    $matches=0;
    while(($x2=$xml->next(false, false, false))!==false&&$xml->getName($x2)=="field")
    {
     $field=$xml->getAttribute($x2, "name");
     if(isset($filter[$field])&&$xml->getAttribute($x2, "value")==$filter[$field])
      $matches++;
    }
    $xml->vc=$x+1; //reset internal iterator for mainloop

    if($matches==$filters) //all filters match
    {
     //update all fields
     while(($x2=$xml->next(false, false, false))!==false&&$xml->getName($x2)=="field")
     {
      $k=$xml->getAttribute($x2, "name");

      if($k==$id) //get idValue for filedata delete
      {
       $idValue=$xml->getAttribute($x2, "value");
       break;
      }
     }

     $xml->delete($x); //delete entry (internal iterator keeps working)

     foreach($d->check("type", "filedata") as $k)
     {
      if(!$this->filedata($this->filedataPrefix.$k."_".$idValue, false)) //delete
      {
       $this->unlock();
       return false;
      }
     }

     $rows=$xml->getAttribute("jas/data", "rows");

     if(!$xml->setAttribute("jas/data", "rows", $rows-1))
     {
      $this->unlock();
      return false;
     }

     $xml->vc=$x+1; //reset internal iterator for mainloop
     $save=true;

     if(!$looseFiltering) //identifier based
      break;
    }
   }

   if($save)
   {
    if(!$this->write($xml))
    {
     $this->unlock();
     return false;
    }

    if($this->indexFile)
    {
     //XXX: doesn't work
     if(!$this->updateIndexes($idValue))
     {
      $this->unlock();
      return false;
     }
    }
   }

   if(!$this->unlock())
    return false;

   $this->e->add($d->title." updated successfully", "note");
   return true;
  }

/*
  function getIndex($name, $values)
  {
   $d=&$this->data; //D for short

   if(!($xml=$this->read("index")))
    return false;

   $info=Array();

   $found=false;
   while(($x=$xml->search("index"))!==false)
   {
    if($xml->getAttribute($x, "name")==$name)
    {
     $tmp=$xml->getOuterXML($x);

     while(($x2=$tmp->search("entry"))!==false)
     {
      $tmp2=$tmp->getOuterXML($x2);

      $allmatch=true;
      while(($x3=$tmp2->search("value"))!==false)
      {
       if(!isset($values[$tmp2->getAttribute($x3, "field")])||$values[$tmp2->getAttribute($x3, "field")]!=$tmp2->getAttribute($x3, "value"))
       {
        $allmatch=false;
        break;
       }
      }

      if($allmatch)
      {
       $info["start"]=$tmp->getAttribute($x2, "start");
       $info["end"]=$tmp->getAttribute($x2, "end");
       $found=true;
       break;
      }
     }
    }

    if($found==true)
     break;
   }

   return $info;
  }

  function updateIndex($name, $info)
  {
   $d=&$this->data; //D for short

   if(!$this->gotLock)
   {
    $this->e->add("Refusing to update index: \"".$name."\", haven't got lock for table: \"".$d->module."\"", "sql");
    return false;
   }

   if(!($xml=$this->read("index")))
    return false;

   $indexEntry=false;
   while(($x=$xml->search("index"))!==false)
   {
    if($xml->getAttribute($x, "name")==$name)
    {
     $indexEntry=$x;
     break;
    }
   }

   if(!$indexEntry) //index not in table, skip updating
   {
    $this->e->add("Index: \"".$name."\" not found for table: \"".$d->module."\"", "warning");
    return true;
   }

   if($name=="PRIMARY")
    $fields=Array($d->getID());
   else if(isset($d->index[$name]))
    $fields=$d->index[$name];
   else if(isset($d->unique[$name]))
    $fields=$d->unique[$name];
   else
   {
    $this->e->add("Unknown index: \"".$name."\" for table: \"".$d->module."\"", "sql");
    return false;
   }

   if(!$info["start"]||!$info["end"]) //delete
   {
   }
   else if($currentInfo=$this->getIndex($name, $info["values"])) //update
   {

   }
   else //new
   {
    $entry=new Format_XML();
    $entry->parse("   <entry start=\"".$info["start"]."\" end=\"".$info["end"]."\">\n</entry>\n");

    foreach($fields as $k)
    {
     $tag="    <value field=\"".$k."\" value=\"";
     $tag.=htmlentities($info["values"][$k], ENT_COMPAT|ENT_XML1);
     $tag.="\"/>\n";

     $entry->addInsideAtEnd(0, $tag);
    }

    $entry->addInsideAtEnd(0, "   ");

    $xml->addInsideAtEnd($indexEntry, $entry->output(false));

    if(!$this->write($xml))
     return false;
   }

   return true;
  }

  function updateIndexes($primaryKey)
  {
   $d=&$this->data; //D for short

   if(!$this->gotLock)
   {
    $this->e->add("Refusing to update indexes, haven't got lock for table: \"".$d->module."\"", "sql");
    return false;
   }

   $id=$d->getID();

   if(!($xml=$this->read("data")))
    return false;

   $info=Array("values" => Array(), "start" => false, "end" => false);
   while(($x=$xml->search("row"))!==false)
   {
    $tmp=$xml->getOuterXML($x);

    $found=false;
    while(($x2=$tmp->search("field"))!==false)
    {
     if($tmp->getAttribute($x2, "name")==$id&&$tmp->getAttribute($x2, "value")==$primaryKey)
     {
      $found=true;
      break;
     }
    }

    if($found)
    {
     $info["start"]=$xml->tag[$x]["outer"]["start"];
     $info["end"]=$xml->tag[$x]["outer"]["end"];

     $tmp->reset();
     while(($x2=$tmp->search("field"))!==false)
      $info["values"][$tmp->getAttribute($x2, "name")]=$tmp->getAttribute($x2, "value");
    }
   }

   if(!$this->updateIndex("PRIMARY", $info))
    return false;

   foreach(array_keys($d->index) as $name)
    if(!$this->updateIndex($name, $info))
     return false;

   foreach(array_keys($d->unique) as $name)
    if(!$this->updateIndex($name, $info))
     return false;

   return true;
  }
*/

  function filedata($file, $content)
  {
   $d=&$this->data; //D for short

   if(!$this->gotLock)
   {
    $this->e->add("Refusing to write file data, haven't got lock for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!file_exists($this->filedataDirectory))
   {
    if(!FS::mkdirS($this->filedataDirectory))
    {
     $this->e->add("Unable to create file data directory: \"".$this->filedataDirectory."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }

   $file=$this->filedataDirectory.$file;

   if($content) //create/ovewrite
   {
    if(!FS::writeS($file, $content, "overwrite", true))
    {
     $this->e->add("Unable to write file data to: \"".$file."\" for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }
   else if(file_exists($file)) //delete
   {
    if(!FS::deleteS($file))
    {
     $this->e->add("Unable to remove file data: \"".$file."\" for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }

   return true;
  }


  function create($submit=false)
  {
   $d=&$this->data; //D for short

   if($this->table())
   {
    $this->e->add("Table: \"".$d->module."\" already exists", "sql");
    return false;
   }

   if(!$submit)
   {
    if($this->dataFile&&$this->indexFile)
     $this->e->add("Query for table: \"".$d->module."\": Create \"".$this->dataFile."\" and \"".$this->indexFile."\"", "note");
    else if($this->dataFile)
     $this->e->add("Query for table: \"".$d->module."\": Create \"".$this->dataFile."\"", "note");
    else if($this->indexFile)
     $this->e->add("Query for table: \"".$d->module."\": Create \"".$this->indexFile."\"", "note");

    return true;
   }

   if($this->dataFile&&!$this->_create("data"))
    return false;

   if($this->indexFile&&!$this->_create("index"))
    return false;

   $this->e->add("Table: \"".$d->module."\" created successfully", "note");
   return true;
  }

  function _create($type)
  {
   $d=&$this->data; //D for short

   $xml=new Format_XML();
   $xml->parse("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>
<jas version=\"1.0\">
</jas>");

   switch($type)
   {
    case "data":
    {
     $file=$this->dataFile;
     $xml->addInsideAtEnd("jas", " <data module=\"".$d->module."\" insertID=\"1\" rows=\"0\">\n </data>\n");
     break;
    }
    case "index":
    {
     $file=$this->indexFile;
     $xml->addInsideAtEnd("jas", " <indexes module=\"".$d->module."\">\n </indexes>\n");
     break;
    }
    default:
    {
     $this->e->add("Unable to create unknown file type: \"".$type."\"", "sql");
     return false;
    }
   }

   if(!$file)
   {
    $this->e->add("Table: \"".$d->module."\" ".$type." file not set", "sql");
    return false;
   }

   if(file_exists($file))
   {
    $this->e->add("Table: \"".$d->module."\" file: \"".$file."\" already exists", "sql");
    return false;
   }

   if(!$this->lock())
    return false;

   if(!$this->write($xml))
   {
    $this->unlock();
    return false;
   }

   if($type=="index")
   {
    if(!$this->_createIndexes())
    {
     $this->unlock();
     return false;
    }
   }

   if(!$this->unlock())
    return false;

   return true;
  }

  function _createIndexes()
  {
   $d=&$this->data; //D for short

   if(!($xml=$this->read("index")))
    return false;

   $indexes=new Format_XML();
   $indexes->xml="  <index name=\"PRIMARY\" type=\"primary\">\n  </index>\n";

   foreach(array_keys($d->index) as $index)
    $indexes->xml.="  <index name=\"".$index."\" type=\"index\">\n  </index>\n";

   foreach(array_keys($d->unique) as $unique)
    $indexes->xml.="  <index name=\"".$unique."\" type=\"unique\">\n  </index>\n";

   $indexes->xml.=" ";

   $xml->addInsideAtEnd("jas/indexes", $indexes->output(false));

   if(!$this->write($xml))
    return false;

   return true;
  }


  function alter($submit=false)
  {
   $d=&$this->data; //D for short

   if(!$this->table())
   {
    $this->e->add("Table: \"".$d->module."\" doesn't exist", "sql");
    return false;
   }

   if(!$submit)
   {
    if($this->info())
     $this->e->add("Table: \"".$d->module."\" last updated: ".$this->results[0]["info_updated"], "note");

    if($this->dataFile&&$this->indexFile)
     $this->e->add("Query for table: \"".$d->module."\": Update \"".$this->dataFile."\" and \"".$this->indexFile."\"", "note");
    else if($this->dataFile)
     $this->e->add("Query for table: \"".$d->module."\": Update \"".$this->dataFile."\"", "note");
    else if($this->indexFile)
     $this->e->add("Query for table: \"".$d->module."\": Update \"".$this->indexFile."\"", "note");

    return true;
   }

   if($this->dataFile)
   {
    if(!file_exists($this->dataFile))
    {
     if(!$this->_create("data"))
      return false;
    }
    else
    {
     if(!$this->_dataMaintenance())
      return false;
    }
   }

   if($this->indexFile)
   {
    if(!file_exists($this->indexFile))
    {
     if(!$this->_create("index"))
      return false;
    }
    else
    {
     if(!$this->_indexMaintenance())
      return false;
    }
   }

   $this->e->add("Table: \"".$d->module."\" maintenance successful", "note");
   return true;
  }

  function _dataMaintenance()
  {
   $d=clone $this->data; //copy data object

   $rows=0;
   $insertID=0;
   $id=$d->getID();

   if(!$this->lock())
    return false;

   if(!($xml=$this->read("data")))
   {
    $this->unlock();
    return false;
   }

   $fields=Array();
   foreach($d->check("use", "sql") as $field)
   {
    if($field!="r"&&$d->field[$field]["module"]==$d->module) //only use module fields
    {
     $d->setDefaultValue($field);
     $fields[$field]=$d->sqlValue($field, 0); //get default (sql) value
    }
   }

   $write=false;
   while(($x=$xml->search("row"))!==false)
   {
    $rows++; //count row numbers

    $tmp=$xml->getOuterXML($x);

    $sqlFields=Array();
    while(($x2=$tmp->search("field"))!==false)
    {
     //get max insertID
     if($tmp->getAttribute($x2, "name")==$id)
     {
      $rowInsertID=$tmp->getAttribute($x2, "value"); //needed to add missing filedata fields

      if($rowInsertID>$insertID)
       $insertID=$rowInsertID;
     }

     //detect missing fields
     $sqlFields[]=$tmp->getAttribute($x2, "name");
    }

    foreach($fields as $field => $value)
    {
     if(!in_array($field, $sqlFields)) //field missing
     {
      $tag="   <field name=\"".$field."\" value=\"";

      if($d->check("type", "filedata", $field))
       $tag.=htmlentities($this->filedataPrefix.$field."_".$rowInsertID, ENT_COMPAT|ENT_XML1);
      else
       $tag.=htmlentities($value, ENT_COMPAT|ENT_XML1);

      $tag.="\"/>\n";

      $xml->addInsideAtEnd($x, $tag);
      $write=true;
     }
    }
   }

   //update insertID
   if($xml->getAttribute("jas/data", "insertID")<=$insertID)
   {
    if(!$xml->setAttribute("jas/data", "insertID", ($insertID+1)))
    {
     $this->unlock();
     return false;
    }
    $write=true;
   }

   //update rows
   if((int)$xml->getAttribute("jas/data", "rows")!==$rows)
   {
    if(!$xml->setAttribute("jas/data", "rows", $rows))
    {
     $this->unlock();
     return false;
    }
    $write=true;
   }

   if($write)
   {
    if(!$this->write($xml))
    {
     $this->unlock();
     return false;
    }
   }

   if(!$this->unlock())
    return false;

   return true;
  }

  function _indexMaintenance()
  {
//XXX: regenerates entire index file, should check if all index definition are in file and add/update/delete existing
   if(!$this->indexFile)
    return false;

   $d=&$this->data; //D for short

   if(!@unlink($this->indexFile))
   {
    $this->e->add("Unable regenerate indexes, can't to remove index file: \"".$this->indexFile."\" for table: \"".$d->module."\"", "sql");
    return false;
   }

   if(!$this->_create("index"))
    return false;

   $id=$d->getID();

   if(!$this->lock())
    return false;

   if(!($xml=$this->read("data")))
   {
    $this->unlock();
    return false;
   }

   $primaryKeys=Array();
   while(($x=$xml->search("field"))!==false)
   {
    if($xml->getAttribute($x, "name")==$id)
     $primaryKeys[]=$xml->getAttribute($x, "value");
   }

   foreach($primaryKeys as $primaryKey)
   {
    if(!$this->updateIndexes($primaryKey))
    {
     $this->unlock();
     return false;
    }
   }

   if(!$this->unlock())
    return false;

   return true;
  }


  function locked()
  {
   if($this->gotLock)
    return false;

   if(!$this->lockFile)
    return false;

   for($c=0;$c<$this->lockedRetries;$c++)
   {
    clearstatcache();
    if(!file_exists($this->lockFile))
     return false;

    usleep($this->lockedTimeout);
   }

   $d=&$this->data; //D for short

   $this->e->add("Table: \"".$d->module."\" locked for ".(($this->lockedTimeout/1000000)*$this->lockedRetries)." second(s), giving up", "sql", false, ($this->data->module!="log")); //don't log
   return true;
  }

  function unlock()
  {
   return $this->lock(false);
  }
  function lock($lock=true)
  {
   if(!$this->lockFile)
   {
    $this->gotLock=$lock;
    return true;
   }

   $d=&$this->data; //D for short

   if($lock)
   {
    if($this->locked()) //not locked if gotLock
     return false;

    if(!@touch($this->lockFile))
    {
     $this->e->add("Unable to lock table: \"".$d->module."\", can't create lock file: \"".$this->lockFile."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }
   else //unlock
   {
    if(!$this->gotLock)
    {
     $this->e->add("Refusing to unlock, haven't got lock for table: \"".$d->module."\"", "sql");
     return false;
    }
    if(!@unlink($this->lockFile))
    {
     $this->e->add("Unable to unlock table: \"".$d->module."\", can't remove lock file: \"".$this->lockFile."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }

   $this->gotLock=$lock;
   return true;
  }


  function read($type)
  {
   $d=&$this->data; //D for short

   switch($type)
   {
    case "data":
    {
     $file=$this->dataFile;
     $tag="jas/data";
     break;
    }
    case "index":
    {
     $file=$this->indexFile;
     $tag="jas/indexes";
     break;
    }
    default:
    {
     $this->e->add("Unable to read unknown file type: \"".$type."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }

   if($this->locked())
    return false;

   if(!$file)
   {
    $this->e->add("Unable to read table: \"".$d->module."\", ".$type." file not set", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if($this->cache["updated"]&&$this->cache["xml"])
   {
    clearstatcache();
    if($this->cache["updated"]==FS::getTimeS($file, "modified"))
     return $this->cache["xml"];
   }

   $xml=new Format_XML();

   $this->cache["updated"]=FS::getTimeS($file, "modified");
   if(!$xml->read($file))
   {
    $this->e->add($xml->e);
    return false;
   }

   if(
    $xml->getAttribute($tag, "module")!=$d->module&&
    (!preg_match(REGEXP_DATA_PLUGIN_MODULE, $d->module, $tmp)||$xml->getAttribute($tag, "module")!=$tmp[2])
   )
   {
    $this->e->add("Module attribute in XML storage: \"".$file."\" doesn't match module: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   $this->cache["xml"]=&$xml;

   return $xml;
  }


  function write($xml)
  {
   $d=&$this->data; //D for short

   if(!$this->gotLock)
   {
    $this->e->add("Refusing to write, haven't got lock for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   $xml->reset();
   if(($xml->search("jas")===false)||(($x=$xml->next(false, false, false))===false))
   {
    $this->e->add("Unable to determine XML type for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   $type=$xml->getName($x);

   switch($type)
   {
    case "data":
    {
     $file=$this->dataFile;
     break;
    }
    case "indexes":
    {
     $file=$this->indexFile;
     break;
    }
    default:
    {
     $this->e->add("Unable to determine XML type for table: \"".$d->module."\"", "sql", false, ($this->data->module!="log")); //don't log
     return false;
    }
   }

   if(!$file)
   {
    $this->e->add("Unable to write table: \"".$d->module."\", ".$type." file not set", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(!FS::writeS($file, $xml->output(false), "overwrite", true))
   {
    $this->e->add("Unable to write table: \"".$d->module."\", can't write file: \"".$file."\"", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   //force cache update
   $this->cache["updated"]=false;

   return true;
  }

 }
?>
