<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/base.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."form.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."fs.php");

 class Backend_Base
 {
  function __construct(&$backend)
  {
   $this->backend=&$backend;
   $this->e=&$this->backend->e;
   $this->language=&$this->backend->language;

   $this->section=false;

   $this->menu=Array();
   $this->table=Array();

   $this->form=new Form();
  }

  function main($section=false)
  {
   if(preg_match("/^[a-z0-9_]+$/i", $section)) //valid section
    $this->section=$section;
   //else //no/invalidate section

   return true;
  }

  function output()
  {
   if($this->section&&method_exists($this, "output_".$this->section))
    $show="output_".$this->section;
   else
    $show="output_default";

   if(method_exists($this, $show))  //show_section call exists
    return $this->$show();

   return false;
  }


  /*
   * Menu functions
   */

  function menuItem($menu, $content, $action=false, $title=false, $icon=false, $properties=false)
  {
   if(!isset($this->menu[$menu])) //new menu
    $this->menu[$menu]=Array();

   //Properties:
   //- class: item class
   //- style: item specific styling
   //- disabled: disable item (remove link and add disabled class)
   //other properties forwarded to link

   if(!$properties)
    $properties=Array();

   //forward properties to link, except class+style
   $linkProperties=$properties;
   unset($linkProperties["class"], $linkProperties["style"]);

   $this->menu[$menu][]=Array(
    "content" => $this->link($content, $action, $title, $icon, $linkProperties),
    "properties" => $properties,
   );

   return true;
  }

  function menu($menu, $class=false)
  {
   $r="<div class=\"menu".($class?" ".$class:"")."\">\n";

   $previous=false;
   $menus=Array($menu);
   foreach($this->menu as $menu => $items)
   {
    if(!in_array($menu, $menus))
     continue;

    $r.=" <ul class=\"".$menu."\">\n";

    foreach($items as $item)
    {
     //class
     $class="";
     if(isset($item["properties"]["class"]))
      $class.=$item["properties"]["class"];

     if(is_int(array_search("disabled", $item["properties"]))) //disabled flag set
     {
      $disabled=true;
      $class.=($class?" ":"")."disabled";
     }
     else
      $disabled=false;

     if(is_int(array_search("selected", $item["properties"]))) //selected flag set
      $class.=($class?" ":"")."selected";

     //style
     $style="";
     if(isset($item["properties"]["style"]))
      $style=$item["properties"]["style"];

     //sub menu
     $content=$item["content"];
     if(isset($item["properties"]["menu"])&&$item["properties"]["menu"]!=$menu)
      $content.=$this->menu($item["properties"]["menu"]); //add sub menu

     //render list item
     $r.="  <li".($class?" class=\"".$class."\"":"").($style?" style=\"".$style."\"":"").">".$content."</li>\n";
    }

    $r.=" </ul>\n";
   }

   $r.="</div>\n";
   return $r;
  }


  /*
   * Table functions
   */

  function tableRow($table, $header=false, $properties=false)
  {
   if(!isset($this->table[$table])) //new table
    $this->table[$table]=Array();

   //Properties:
   //- class: row class
   //- style: row specific styling
   //- disabled: disable row (remove links and add disabled class)
   //- hilite: hilite row (class)
   //- selected: selected row (class)

   if(!$properties)
    $properties=Array();

   $this->table[$table][]=Array(
    "header" => ($header),
    "properties" => $properties,
    "columns" => Array(),
   );

   return true;
  }

  function tableColumn($table, $content, $action=false, $title=false, $properties=false)
  {
   if(!isset($this->table[$table])) //unknown table
   {
    $this->e->add("Can't add column to unknown table: \"".$table."\"", "error");
    return false;
   }

   //Properties:
   //- class: column class
   //- style: column specific styling
   //- disabled: disable column (remove link and add disabled class)
   //- hilite: hilite column (class)
   //- selected: selected column (class)
   //- colspan: colspan for column
   //- rowspan: rowspan for column
   //other properties forwarded to link

   if(!$properties)
    $properties=Array();

   //forward properties to link, except class+style
   $linkProperties=$properties;
   unset($linkProperties["class"], $linkProperties["style"]);

   if(!is_int(array_search("disabled", $properties))) //column disabled flag not set
    if(is_int(array_search("disabled", $this->table[$table][sizeof($this->table[$table])-1]["properties"]))) //row disabled flag set
     $linkProperties[]="disabled";

   $this->table[$table][sizeof($this->table[$table])-1]["columns"][]=Array( //add to current table row
    "content" => $this->link($content, $action, $title, false, $linkProperties),
    "properties" => $properties,
   );

   return true;
  }

  function table($table, $class=false)
  {
   if(!isset($this->table[$table])) //unknown table
   {
    $this->e->add("Can't render unknown table: \"".$table."\"", "error");
    return false;
   }

   $r="<table class=\"".$table.($class?" ".$class:"")."\">\n";

   $previous=false;
   foreach($this->table[$table] as $row)
   {
    //class
    $class="";
    if(isset($row["properties"]["class"]))
     $class.=$row["properties"]["class"];

    if(is_int(array_search("disabled", $row["properties"]))) //disabled flag set
    {
     $disabled=true;
     $class.=($class?" ":"")."disabled";
    }
    else
     $disabled=false;

    if(is_int(array_search("selected", $row["properties"]))) //selected flag set
     $class.=($class?" ":"")."selected";
    if(is_int(array_search("hilite", $row["properties"]))) //hilite flag set
     $class.=($class?" ":"")."hilite";

    if($row["header"]) //header flag set
    {
     $header=true;
     $class.=($class?" ":"")."header";
    }
    else
     $header=false;

    //render row
    $r.=" <tr".($class?" class=\"".$class."\"":"").">\n";

    foreach($row["columns"] as $column)
    {
     //class
     $class="";
     if(isset($column["properties"]["class"]))
      $class.=$column["properties"]["class"];

     if(is_int(array_search("disabled", $column["properties"]))) //disabled flag set
     {
      $disabledColumn=true;
      $class.=($class?" ":"")."disabled";
     }
     else
      $disabledColumn=false;

     if(is_int(array_search("selected", $column["properties"]))) //selected flag set
      $class.=($class?" ":"")."selected";
     if(is_int(array_search("hilite", $column["properties"]))) //hilite flag set
      $class.=($class?" ":"")."hilite";

     //style
     $style="";
     if(isset($column["properties"]["style"]))
      $style=$column["properties"]["style"];

     //rowspan
     $rowspan="";
     if(isset($column["properties"]["rowspan"]))
      $rowspan=$column["properties"]["rowspan"];

     //colspan
     $colspan="";
     if(isset($column["properties"]["colspan"]))
      $colspan=$column["properties"]["colspan"];

     //render list column
     $r.="  <".($row["header"]?"th":"td").($class?" class=\"".$class."\"":"").($style?" style=\"".$style."\"":"").($style?" style=\"".$style."\"":"").($rowspan>1?" rowspan=\"".$rowspan."\"":"").($colspan>1?" colspan=\"".$colspan."\"":"").">".$column["content"]."</".($row["header"]?"th":"td").">\n";
    }

    $r.=" </tr>\n";
   }

   $r.="</table>\n";
   return $r;
  }


  /*
   * Statistics functions
   */

  function statsEntry($stats, $content, $value, $properties=false)
  {
   if(!isset($this->stats[$stats])) //new stats
    $this->stats[$stats]=Array();

   //Properties:
   //- class: entry class

   if(!$properties)
    $properties=Array();

   $this->stats[$stats][]=Array( //add to current stats entry
    "content" => $content,
    "value" => $value,
    "properties" => $properties,
   );

   return true;
  }

  function stats($stats, $properties=false)
  {
   if(!isset($this->stats[$stats])) //unknown stats
   {
    $this->e->add("Can't render unknown stats: \"".$stats."\"", "error");
    return false;
   }

   //Properties:
   //- class: entry class

   if(!$properties)
    $properties=Array();

   //determine statistics
   $sum=0;
   $min=false;
   $max=false;
   $entries=sizeof($this->stats[$stats]);
   foreach($this->stats[$stats] as $entry)
   {
    $sum+=$entry["value"];

    if($min===false||$entry["value"]<$min)
     $min=$entry["value"];

    if($max===false||$entry["value"]>$max)
     $max=$entry["value"];
   }
   $average=$max/$entries;

   //render entries
   foreach($this->stats[$stats] as $entry)
   {
    $class="p".(($max?round(($entry["value"]/$max)*10):0)*10);
    if(isset($entry["properties"]["class"])&&$entry["properties"]["class"])
     $class.=" ".$entry["properties"]["class"];

    $this->tableRow("stats ".$stats, false, Array("class" => $class));
    $this->tableColumn("stats ".$stats, $entry["content"]);
    $this->tableColumn("stats ".$stats, Mutilate::humanReadableNumberS($entry["value"], 1), false, false, Array("class" => "nr"));

    if($sum&&$entry["value"]&&$percentage=round(($entry["value"]/$sum)*100, 1))
     $this->tableColumn("stats ".$stats, "<div class=\"bar\" style=\"width: ".($max?round(($entry["value"]/$max)*100):0)."%\">".$percentage."%</div>", false, false, Array("class" => "percentage"));
    else
     $this->tableColumn("stats ".$stats, "&nbsp;", false, false, Array("class" => "percentage"));
   }

   //render total
   if(!isset($properties["total"])||$properties["total"])
   {
    $this->tableRow("stats ".$stats, false, Array("class" => "total"));
    $this->tableColumn("stats ".$stats, $this->language->t("Total"));
    $this->tableColumn("stats ".$stats, Mutilate::humanReadableNumberS($sum, 1), false, false, Array("class" => "nr"));
    $this->tableColumn("stats ".$stats, "");
   }

   return $this->table("stats ".$stats, (isset($properties["class"])?$properties["class"]:false));
  }


  /*
   * Data
   */

  function viewList(&$d, $properties=false)
  {
   //Properties:
   //- get: get data (true or depth (0, 1, 2, etc)
   //- fields: additional fields to show
   //- rights: show rights information for categories
   //- user: show rights for specific user
   //- table: alternate table name
   //- render: render table
   //- class: table class (if table is rendered)
   //- checkboxes: add checkboxes to table
   //- pager: use & render pager table
   //- nopagerform: don't add form around pager (viewList already in a form)

   global $_config;

   if(!$properties)
    $properties=Array();

   $vars=Array();

   //determine table name
   if(isset($properties["table"]))
    $table=$properties["table"];
   else
    $table="data ".$d->module;

   //determine rights fields
   if(is_int(array_search("rights", $properties))&&$d->module=="categories") //only for categories module
   {
    $rights=true;

    if(isset($properties["user"]))
    {
     $user=$properties["user"];

     //get user's groups
     $dtmp=Data::cache($_config->get("usermodule"), "view", "item");
     $dtmp->setShowFields("catnr", "usersAdditionalGroups");
     $dtmp->getReferenceData("catnr")->setShowFields("name");
     $dtmp->getReferenceData("usersAdditionalGroups")->getReferenceData("categories")->setShowFields("name");
     $dtmp->setValue($dtmp->getID(), $user);

     if(!$dtmp->get(1)||!$dtmp->next())
     {
      $this->e->add($dtmp->e);
      return false;
     }

     $usergroups=Array();
     if(isset($dtmp->v["catnr"]["nr"]))
     {
      $usergroups[$dtmp->v["catnr"]["nr"]]=$dtmp->v["catnr"]["name"];
      foreach($dtmp->v["usersAdditionalGroups"] as $group)
       if(!isset($usergroups[$group["nr"]]))
        $usergroups[$group["nr"]]=$group["name"];
     }
    }
    else
     $user=false;

    //don't show timestamp
    unset($d->field["added"]);
    unset($d->field["updated"]);

    if(!isset($d->field["usernr"]))
     $d->moduleObject->initRightsFields();

    if(!isset($properties["fields"]))
     $properties["fields"]=Array("usernr", "groupnr");
    else if(!is_array($properties["fields"]))
     $properties["fields"]=Array("usernr", "groupnr", $properties["fields"]);
    else
     $properties["fields"]=array_merge(Array("usernr", "groupnr"), $properties["fields"]);
   }
   else
    $rights=false;

   //fields
   $titles=$d->check("properties", "title");
   $showFields=$titles;

   foreach($titles as $field)
    if($d->field[$field]["type"]=="text"||$d->field[$field]["type"]=="html")
     $d->addMutilation($field, "stripTags");

   if($id=$d->getID())
    $showFields[]=$id;

   //additional fields
   $fields=Array();
   if(isset($properties["fields"]))
   {
    if(!is_array($properties["fields"]))
     $properties["fields"]=Array($properties["fields"]);

    foreach($properties["fields"] as $field)
    {
     $fields[]=$field;
     $showFields[]=$field;

     if(isset($d->reference[$field])) //reference
     {
      $dr=$d->getReferenceData($field);
      $dr->setShowFields($dr->check("properties", "title"));
      unset($dr);
     }
    }
   }

   if(isset($d->field["updated"]))
   {
    $fields[]="updated";
    $showFields[]="updated";

    $d->addMutilation("updated", "timeAgo");
    $d->field["updated"]["title"]="Updated";
   }
   else if(isset($d->field["added"]))
   {
    $fields[]="added";
    $showFields[]="added";

    $d->addMutilation("added", "timeAgo");
    $d->field["added"]["title"]="Added";
   }

   $d->setShowFields($showFields);

   //search
   if($d->action=="search")
   {
    if($d->extra!="deleted")
    {
     if($d->fulltextIndex) //full text index exists
     {
      $d->setExtra("index");

      //default sort on index desc
      $sort=Array();
      foreach(array_keys($d->fulltextIndex) as $index)
       $sort[].=$index."-";

      $d->sortOrder($sort);
     }
     else //normal search
      $d->setExtra("full");
    }

    $d->setRequestValue("search");
    $vars["search"]=$d->getValue("search");
   }
   else if($d->extra=="list")
    $vars["catnr"]=$d->getValue($d->getRightsReference());

   //sort
   if($sort=request("sort"))
    $d->sortOrder(explode(";", $sort));

   if($d->sortOrder)
   {
    //sort references
    foreach($d->sortOrder as $field)
    {
     if(preg_match("/^(.*)(\-|\+)$/", $field, $field))
     {
      if(isset($d->reference[$field[1]]))
      {
       $dr=$d->getReferenceData($field[1]);

       if($d->reference[$field[1]]["type"]=="*:*")
        $dr=$dr->getReferenceData($dr->check("use", "to")[0]);

       $sort=Array();
       foreach($dr->check("properties", "title") as $title)
        $sort[]=$title.$field[2];

       $dr->sortOrder($sort);
       unset($dr);
      }
     }
    }


    $sort=implode(";", $d->sortOrder);

    if(request("sort")) //non default sort
     $vars["sort"]=$sort;
   }
   else
    $sort=false;

   //pager
   if(!isset($properties["pager"])||$properties["pager"])
   {
    if(!$d->getValue("show_total"))
     $d->setValue("show_total", 25);
    if((int)$d->getValue("show_total")!=25)
     $vars["show_total"]=(int)$d->getValue("show_total");

    if((int)$d->getValue("show_from"))
     $vars["show_from"]=(int)$d->getValue("show_from");
   }

   //get data set
   if(isset($properties["get"])&&$properties["get"]!==false)
   {
    if($d->extra=="all"&&($self=$d->getSelfReference())&&request("sort")===NULL)
    {
     if($d->field[$self]["valuetype"]=="default")
      $d->setValue($self, 0);

     $d->setShowFields(array_merge($showFields, Array("treedepth")));

     $d->setAction("tree");
     $d->setExtra("down");
    }

    //check if *:0/*:1 references are shown
    foreach($showFields as $field)
    {
     if(isset($d->reference[$field]))
     {
      if($d->reference[$field]["type"]=="*:0"||$d->reference[$field]["type"]=="*:1")
      {
       $dr=$d->getReferenceData($field);
       $dr->setShowFields($dr->check("properties", "title")); //only gather title fields

       if($properties["get"]==0)
        $properties["get"]=1;
      }
     }
    }

    if(!$d->get($properties["get"], true))
    {
     $this->e->add($d->e);
     return false;
    }

    if(!$d->vmax&&$d->action=="search"&&$d->extra=="index") //no results for index search
    {
     //retry with full text
     $d->setExtra("full");
     if(!$d->get(true, true))
     {
      $this->e->add($d->e);
      return false;
     }
    }
   }

   if($d->action=="tree"&&$d->extra=="down"&&$d->field["treedepth"]["value"])
    $treedepth=max($d->field["treedepth"]["value"])+1;
   else
    $treedepth=false;

   //get category rights
   if($rights)
   {
    $du=Data::cache("userRights", "view", "all");
    $du->clearValues();

    $du->setShowFields("nr", "catnr", "rights");
    $du->getReferenceData("nr")->setShowFields($du->getReferenceData("nr")->check("properties", "title"));
    unset($du->reference["catnr"]); //don't get category reference

    $dg=Data::cache("groupRights", "view", "all");
    $dg->clearValues();

    $dg->setShowFields("nr", "catnr", "rights");
    $dg->getReferenceData("nr")->setShowFields($dg->getReferenceData("nr")->check("properties", "title"));
    unset($dg->reference["catnr"]); //don't get category reference

    if($user) //user rights overview
    {
     //get user's users+groups rights
     if($user==$_config->get("rootnr")) //fake root rights
     {
      $rootrights=$du->rights2bin($du->getRootRights());
      for($c=0;$c<$d->vmax;$c++)
      {
       $du->setValue("nr", $user, $c);
       $du->setValue("catnr", $d->getValue("nr", $c), $c);
       $du->setValue("rights", $rootrights, $c);
      }

      $du->vmax=$d->vmax;
     }
     else
     {
      for($c=0;$c<$d->vmax;$c++)
      {
       $du->setValue("nr", $user, $c);

       $du->setValue("catnr", $d->getValue("nr", $c), $c);
       $du->setOperant("catnr", "OR", $c);
      }

      if(!$du->get())
       $this->e->add($du->e);
     }

     if($usergroups)
     {
      $c2=0;
      for($c=0;$c<$d->vmax;$c++)
      {
       foreach(array_keys($usergroups) as $group)
       {
        $dg->setValue("nr", $group, $c2);

        $dg->setValue("catnr", $d->getValue("nr", $c), $c2);
        $dg->setOperant("catnr", "OR", $c2);
        $c2++;
       }
      }

      if(!$dg->get())
       $this->e->add($dg->e);
     }

     //total rights
     $dt=clone $du;
     $values=Array();

     $du->reset();
     while($du->next())
     {
      if(!isset($values[$du->v["catnr"]]))
       $values[$du->v["catnr"]]=$du->v["rights"];
      else
       $values[$du->v["catnr"]]=$du->rights2bin($du->bin2rights($values[$du->v["catnr"]])|$du->bin2rights($du->v["rights"]));
     }

     $dg->reset();
     while($dg->next())
     {
      if(!isset($values[$dg->v["catnr"]]))
       $values[$dg->v["catnr"]]=$dg->v["rights"];
      else
       $values[$dg->v["catnr"]]=$dg->rights2bin($dg->bin2rights($values[$dg->v["catnr"]])|$dg->bin2rights($dg->v["rights"]));
     }

     $c=0;
     $dt->clearValues();
     foreach($values as $catnr => $r)
     {
      $dt->setValue("catnr", $catnr, $c);
      $dt->setValue("rights", $r, $c);
      $dt->setValue("nr", 0, $c);
      $c++;
     }
    }
    else //rights overview
    {
     //get all users+group rights
     for($c=0;$c<$d->vmax;$c++)
     {
      $du->setValue("catnr", $d->getValue("nr", $c), $c);
      $du->setOperant("catnr", "OR", $c);

      $dg->setValue("catnr", $d->getValue("nr", $c), $c);
      $dg->setOperant("catnr", "OR", $c);
     }

     if(!$du->get(1))
      $this->e->add($du->e);
     if(!$dg->get(1))
      $this->e->add($dg->e);
    }
   }

   if($id&&is_int(array_search("checkboxes", $properties))) //checkboxes flag set
   {
    $checkboxes=true;

    if(!($checked=request($id)))
     $checked=Array();
    else if(!is_array($checked))
     $checked=Array($checked);
   }
   else
    $checkboxes=false;

   //header
   if(!isset($properties["header"])||$properties["header"])
   {
    $this->tableRow($table, true, Array("class" => "data ".$d->module));

    if($checkboxes)
     $this->tableColumn($table, "", false, false, Array("class" => "checkall"));

    if($id)
     $this->tableColumn($table, "#", Array(false, array_merge($vars, Array("sort" => ($id."+"==$sort?$id."-":$id)))), $this->language->t("Sort on ID"), Array("class" => $d->field[$id]["type"]." ".$id));

    foreach($titles as $c => $field)
     $this->tableColumn($table, $d->title($field, true), Array(false, array_merge($vars, Array("sort" => ($field."+"==$sort?$field."-":$field)))), $this->language->t("Sort on ".$d->title($field)), Array("class" => $d->field[$field]["type"]." ".$field." title", "colspan" => ($c==0&&$treedepth?$treedepth:false)));

    foreach($fields as $field)
    {
     if($rights&&($field=="usernr"||$field=="groupnr"))
     {
      if($user&&$field=="groupnr")
      {
       foreach($usergroups as $group => $name)
        $this->tableColumn($table, $name);

       $this->tableColumn($table, $this->language->t("Total"));
      }
      else
       $this->tableColumn($table, $d->title($field, true));
     }
     else if($field=="updated"||$field=="added") //translation in jas set
      $this->tableColumn($table, $this->language->t($d->title($field)), Array(false, array_merge($vars, Array("sort" => ($field."-"==$sort?$field:$field."-")))), $this->language->t("Sort on ".$this->language->t($d->title($field))), Array("class" => $d->field[$field]["type"]." ".$field));
     else
      $this->tableColumn($table, $d->title($field, true), Array(false, array_merge($vars, Array("sort" => ($field."+"==$sort?$field."-":$field)))), $this->language->t("Sort on ".$d->title($field, true)), Array("class" => $d->field[$field]["type"]." ".$field));
    }

    $this->tableColumn($table, "");
   }

   //data
   while($d->next())
   {
    $this->tableRow($table);

    if($checkboxes)
     $this->tableColumn($table, $this->form->checkbox((in_array($d->v[$id], $checked)), false, $id."[]", $d->v[$id]));

    //fields
    if($id)
     $this->tableColumn($table, (is_array($d->v[$id])?$d->v[$id]["nr"]:$d->v[$id]), false, false, Array("class" => $d->field[$id]["type"]." ".$id));

    if($treedepth&&$d->v["treedepth"])
     for($c=0;$c<$d->v["treedepth"];$c++)
      $this->tableColumn($table, "", false, false, Array("class" => "treedepth"));

    foreach($titles as $c => $field)
    {
     if(isset($d->reference[$field])) //reference
     {
      $value="";
      $dr=$d->getReferenceData($field);
      foreach($dr->check("properties", "title") as $k)
       if(isset($d->v[$field][$k]))
        $value.=$d->v[$field][$k]." ";

      $this->tableColumn($table, $value, Array("data/".$d->module."/".$d->v("id")."/"), "View item: ".$d->v("title"), Array("class" => "reference ".$field." title", "colspan" => ($c==0&&$treedepth?($treedepth-$d->v["treedepth"]):false)));
     }
     else
      $this->tableColumn($table, $d->v[$field], Array("data/".$d->module."/".$d->v("id")."/"), "View item: ".$d->v("title"), Array("class" => $d->field[$field]["type"]." ".$field." title", "colspan" => ($c==0&&$treedepth?($treedepth-$d->v["treedepth"]):false)));
    }

    foreach($fields as $field)
    {
     if($rights&&($field=="usernr"||$field=="groupnr"))
     {
      if($user) //user rights overview
      {
       if($field=="usernr")
       {
        $du->reset();
        $values=Array();
        while($du->search("catnr", $d->v[$id]))
        {
         $this->_viewField($du, "rights", $value, $url, $class);
         $values[]=$this->link($value, $url);
        }

        $this->tableColumn($table, implode(", ", $values), false, "", Array("class" => "rights"));
       }
       else //groups
       {
        foreach($usergroups as $group => $name)
        {
         $dg->reset();
         $values=Array();
         while($dg->search("catnr", $d->v[$id]))
         {
          if($dg->v["nr"]==$group)
          {
           $this->_viewField($dg, "rights", $value, $url, $class);
           $values[]=$this->link($value, $url);
          }
         }

         $this->tableColumn($table, implode(", ", $values), false, "", Array("class" => "rights"));
        }

        //total
        $dt->reset();
        $values=Array();
        while($dt->search("catnr", $d->v[$id]))
        {
         $this->_viewField($dt, "rights", $value, $url, $class);
         $values[]=$this->link($value, $url);
        }

        $this->tableColumn($table, implode(", ", $values), false, "", Array("class" => "rights"));
       }
      }
      else //rights overview
      {
       if($field=="usernr")
       {
        $du->reset();
        $values=Array();
        while($du->search("catnr", $d->v[$id]))
        {
         $this->_viewField($du, "rights", $value, $url, $class);
         $values[]=$this->link($value, $url);
        }

        $this->tableColumn($table, implode(", ", $values), false, "", Array("class" => "rights"));
       }
       else //groupnr
       {
        $dg->reset();
        $values=Array();
        while($dg->search("catnr", $d->v[$id]))
        {
         $this->_viewField($dg, "rights", $value, $url, $class);
         $values[]=$this->link($value, $url);
        }

        $this->tableColumn($table, implode(", ", $values), false, "", Array("class" => "rights"));
       }
      }
     }
     else
     {
      $this->_viewField($d, $field, $value, $url, $class);

      if($url)
       $this->tableColumn($table, $value, $url, $this->language->t("Show: ".$value), Array("class" => $class));
      else
       $this->tableColumn($table, $value, false, false, Array("class" => $class));
     }
    }

    if($cms=$this->cmsOptions($d, false, true, $vars))
     $this->tableColumn($table, $this->icon("cms").$cms, false, false, Array("class" => "cms popup right"));
    else
     $this->tableColumn($table, "", false, false, Array("class" => "cms"));
   }

   //totals
   if(!isset($properties["footer"])||$properties["footer"])
   {
    $this->tableRow($table, false, Array("class" => "total"));

    if(!$d->vmax)
     $this->tableColumn($table, $this->language->t("No items"), false, false, Array("colspan" => sizeof($showFields)+($checkboxes?1:0)+($rights&&$user?sizeof($usergroups):0)));
    else
    {
     $this->tableColumn($table, $this->language->t("Total"), false, false, Array("colspan" => sizeof($showFields)-($checkboxes?0:1)+($rights&&$user?sizeof($usergroups):0)+($treedepth>1?$treedepth-1:0)));
     $this->tableColumn($table, $d->vmax." / ".$d->getValue("show_count"), false, false, Array("class" => "nr"));
    }

    if($cms=$this->cmsOptions($d, "add", false, $vars))
     $this->tableColumn($table, $cms["icon"], $cms["action"], $cms["title"], Array("class" => "cms"));
    else
     $this->tableColumn($table, "", false, false, Array("class" => "cms"));
   }

   //render table
   if(!isset($properties["render"])||$properties["render"])
    $r=$this->table($table, (isset($properties["class"])?$properties["class"]:false));


   //pager
   if(!isset($properties["pager"])||$properties["pager"])
   {
    if(($pages=ceil($d->getValue("show_count")/$d->getValue("show_total")))>1)
    {
     $this->tableRow($table."_pager");
     $current=ceil($d->getValue("show_from")/$d->getValue("show_total"))+1;


     if($d->getValue("show_from"))
      $this->tableColumn($table."_pager", $this->language->t("Previous"), Array(false, array_merge($vars, Array("show_from" => $d->getValue("show_from")-$d->getValue("show_total")))), $this->language->t("Previous page"), Array("class" => "previous"));
     else
      $this->tableColumn($table."_pager", $this->language->t("Previous"), false, false, Array("class" => "previous"));

     //per page dropdown
     $perpage="";
     $perpages=Array(10, 25, 50, 100, 250, 500, 1000);

     foreach($vars as $k => $v)
      if($k!="show_total")
       $perpage.=$this->form->input("hidden", $k, $v);

     $perpage.=$this->form->select(false, "show_total", $perpages, $d->getValue("show_total"), false, false, "onchange=\"this.form.submit();\"");

     if(!is_int(array_search("nopagerform", $properties))) //don't wrap pager in form
      $perpage=$this->form($perpage);

     $this->menuItem($table."_pager", $perpage, false, false, false, Array("class" => "perpage"));

     //page list
     $pagelist=Array();

     //determine "useful" interval
     $interval=ceil($pages/100)*10;

     //fill intervals
     for($c=$interval;$c<$pages;$c+=$interval)
      $pagelist[$c]=$c;

     //singles
     if($interval>=10)
     {
      $start=($current<7?1:$current-5);
      $end=($current+6>$pages?$pages:$current+5);

      for($c=$start;$c<$end;$c++)
       $pagelist[$c]=$c;
     }

     $pagelist[1]=$this->language->t("First");
     $pagelist[$pages]=$this->language->t("Last");
     ksort($pagelist);

     //render page list
     foreach($pagelist as $page => $title)
      $this->menuItem($table."_pager", "&nbsp; &nbsp; &nbsp; ".$title."&nbsp; &nbsp; &nbsp; ", Array(false, array_merge($vars, Array("show_from" => ($page-1)*$d->getValue("show_total")))), $this->language->t("Page: ". $page), false, ($current==$page?Array("selected"):false));


     $this->tableColumn($table."_pager", $current." / ".$pages."".$this->menu($table."_pager"), false, false, Array("class" => "pages"));

     if($d->getValue("show_from")+$d->getValue("show_total")<$d->getValue("show_count"))
      $this->tableColumn($table."_pager", $this->language->t("Next"), Array(false, array_merge($vars, Array("show_from" => $d->getValue("show_from")+$d->getValue("show_total")))), $this->language->t("Next page"), Array("class" => "next"));
     else
      $this->tableColumn($table."_pager", $this->language->t("Next"), false, false, Array("class" => "next"));


     $r.=$this->table($table."_pager", "pager");
    }
    else if(is_int(array_search("nopagerform", $properties))) //no pages
     $r.=$this->form->input("hidden", "show_total", $d->getValue("show_total")); //add show_total
   }

   if(!isset($properties["render"])||$properties["render"]||!isset($properties["pager"])||$properties["pager"])
    return $r;

   return true;
  }


  function viewItem(&$d, $properties=false)
  {
   //Properties:
   //- get: get data (true or depth (0, 1, 2, etc)
   //- show: what fields to show: normal, default, reference, all
   //- table: alternate table name
   //- render: render table
   //- class: table class (if table is rendered)

   if(!$properties)
    $properties=Array();

   $id=$d->getID();

   //get data
   if(isset($properties["get"])&&$properties["get"]!==false)
   {
    //determine show fields
    $fields=Array($id, $d->getRightsReference());

    //added+updated time ago
    if(isset($d->field["added"]))
    {
     $fields[]="added";
     $d->addMutilation("added", "timeAgo");
    }
    if(isset($d->field["updated"]))
    {
     $fields[]="updated";
     $d->addMutilation("updated", "timeAgo");
    }

    //limit amount of data large text fields show
    foreach($d->check("use", "show") as $field)
    {
     if($d->check("type", "text", $field)&&$d->field[$field]["max"]>65536) //don't show any data over 64k
      $d->addMutilation($field, "strip", 65536);

     if(($d->check("type", "text", $field)||$d->check("type", "html", $field))&&$d->field[$field]["max"]>1000) //scroll any data over 1k
      $d->addMutilation($field, "replace", "/^.*$/ms", "<div class=\"overflow\">\\0</div>");
    }

    //added id + rights fields
    $d->setShowFields(array_merge($fields, $d->check("use", "show")));

    //references only display title
    foreach(array_keys($d->reference) as $ref)
    {
     $dr=$d->getReferenceData($ref);
     if($d->reference[$ref]["type"]=="*:*")
     {
      $drm=$dr->getReferenceData($dr->check("use", "to")[0]);
      $drm->setShowFields($drm->check("properties", "title"));
      unset($drm);
     }
     else
      $dr->setShowFields($dr->check("properties", "title"));

     unset($dr);
    }

    if(!$d->get($properties["get"])||!$d->next())
    {
     $this->e->add($d->e);
     return false;
    }
   }
   else if(!$d->vmax) //no data
    return false;

   //determine table name
   if(isset($properties["table"]))
    $table=$properties["table"];
   else
    $table=$d->module."_".$d->v[$id].(isset($properties["show"])?"_".$properties["show"]:"");

   //get ignore list
   $ignore=Array();
   foreach(array_keys($d->fulltextIndex) as $field)
    $ignore[]=$field;

   foreach($d->check("type", "file") as $field)
    $ignore=array_merge($ignore, Array($field."Type", $field."Size", $field."Data", $field."Array"));

   foreach($d->check("type", "image") as $field)
    $ignore=array_merge($ignore, Array($field."ImageWidth", $field."ImageHeight", $field."PHPImageType", $field."Type", $field."Size", $field."Data", $field."Array"));

   //determine fields
   $showFields=Array();
   switch((isset($properties["show"])?$properties["show"]:false))
   {
    case "default":
    {
     $showFields[""]=Array();
     foreach($d->check("use", "show") as $field)
      if($d->check("properties", "default", $field))
       if(!isset($d->reference[$field]))
        if(!in_array($field, $ignore))
         $showFields[""][]=$field;

     break;
    }
    case "reference":
    {
     $showFields[""]=Array();
     foreach($d->check("use", "show") as $field)
      if(isset($d->reference[$field]))
       if(!in_array($field, $ignore))
        $showFields[""][]=$field;

     break;
    }
    case "normal": //all except default+reference
    {
     if($groups=$d->getGroups())
     {
      foreach($groups as $nr => $group)
       foreach($d->getGroupFields($nr) as $field)
        if($d->check("use", "show", $field))
         if(!$d->check("properties", "default", $field))
          if(!isset($d->reference[$field]))
           if(!in_array($field, $ignore))
            $showFields[$group][]=$field;
     }
     else //no groups
     {
      $showFields[$d->title]=Array();
      foreach($d->check("use", "show") as $field)
       if(!$d->check("properties", "default", $field))
        if(!isset($d->reference[$field]))
         if(!in_array($field, $ignore))
          $showFields[$d->title][]=$field;
     }

     break;
    }
    default:
    {
     if($groups=$d->getGroups())
     {
      foreach($groups as $nr => $group)
       foreach($d->getGroupFields($nr) as $field)
        if($d->check("use", "show", $field))
         if(!in_array($field, $ignore))
          $showFields[$group][]=$field;
     }
     else //no groups
     {
      $showFields[$d->title]=Array();
      foreach($d->check("use", "show") as $field)
       if(!in_array($field, $ignore))
        $showFields[$d->title][]=$field;
     }

     break;
    }
   }

   //clean up empty groups
   foreach($showFields as $group => $fields)
    if(!$fields)
     unset($showFields[$group]);

   if(!$showFields)
    return false;


   foreach($showFields as $group => $fields)
   {
    $this->tableRow($table, true);
    $this->tableColumn($table, $d->l->t($group), false, false, Array("colspan" => 2));

    $class="";
    foreach($fields as $field)
    {
     if($field=="r") //skip rights field
      continue;

     if($d->check("properties", "control", $field))
      continue;

     $this->_viewField($d, $field, $value, $url, $class);

     $this->tableRow($table, false, Array(($d->check("properties", "title", $field)?"hilite":false)));
     $this->tableColumn($table, $d->title($field, true), false, false, Array("class" => "key"));

     if($url)
      $this->tableColumn($table, $value, $url, $this->language->t("Show: ".$value), Array("class" => "value ".$class));
     else
      $this->tableColumn($table, $value, false, false, Array("class" => "value ".$class));
    }
   }

   //render table
   if(!isset($properties["render"])||$properties["render"])
    return $this->table($table, (isset($properties["class"])?$properties["class"]:false));

   return true;
  }

  function _viewField(&$d, $field, &$value, &$url, &$class)
  {
   $value="";
   $url=false;
   $class="";

   $id=$d->getID();

   if($field=="key"&&preg_match("/(^|_)pluginConfigs$/", $d->module)&&$d->v["plugin"]) //pluginConfigs key field
   {
    $value=$this->link($d->v[$field], Array("plugins/".$d->v["plugin"]."/frontend/", Array("config" => $d->v[$field])), $this->language->t("Show ".$d->v[$field]." plugin frontend"));
    $url=false;
   }
   else if(isset($d->reference[$field])) //reference
   {
    if($d->v[$field])
    {
     $dr=$d->getReferenceData($field);
     if($d->reference[$field]["type"]=="*:*") //many to many
     {
      $value=Array();
      $drm=$dr->getReferenceData($dr->check("use", "to")[0]);
      foreach($d->v[$field] as $k => $v)
      {
       $tmp="";
       foreach($drm->check("properties", "title") as $k)
        $tmp.=$v[$k]." ";

       if($tmp)
       {
        $tmp=rtrim(Mutilate::htmlDecodeS(Mutilate::stripTagsS($tmp)));
        $value[]=$this->link($tmp, Array("data/".$d->reference[$field]["module"]."/".$v[$drm->getID()]."/"), $this->language->t("Show: ".$tmp));
       }
      }

      $value=implode(", ", $value);
     }
     else if(strpos($d->reference[$field]["type"], ":*")) //0/1 to many
     {
      $value=Array();
      foreach($d->v[$field] as $k => $v)
      {
       $tmp="";
       foreach($dr->check("properties", "title") as $k)
        $tmp.=$v[$k]." ";

       if($tmp)
       {
        $tmp=rtrim(Mutilate::htmlDecodeS(Mutilate::stripTagsS($tmp)));
        $value[]=$this->link($tmp, Array("data/".$d->reference[$field]["module"]."/".$v[$dr->getID()]."/"), $this->language->t("Show: ".$tmp));
       }
      }

      $value=implode(", ", $value);
     }
     else //single values
     {
      foreach($dr->check("properties", "title") as $k)
       $value.=$d->v[$field][$k]." ";

      if($value)
      {
       $value=Mutilate::htmlDecodeS(Mutilate::stripTagsS($value));
       $url=$this->url("data/".$d->reference[$field]["module"]."/".$d->v[$field][$dr->getID()]."/");
      }
     }

     foreach($dr->check("properties", "title") as $k)
     {
      $class=$dr->field[$k]["type"];
      break;
     }

     unset($dr);
    }

    $class.=" ".$field;
   }
   else //normal fields
   {
    switch($d->field[$field]["type"])
    {
     case "boolean":
     {
      $value=$this->language->t(($d->v[$field]?"Yes":"No"));
      break;
     }
     case "enum":
     case "list":
     {
      if(array_type($d->listOptions[$field])=="associative"&&isset($d->listOptions[$field][$d->v[$field]]))
       $value=$d->listOptions[$field][$d->v[$field]]." (".$d->v[$field].")";
      else
       $value=$d->v[$field];

      break;
     }
     case "nr":
     {
      if(preg_match("/^\d+$/", $d->v[$field])&&!$d->check("properties", "identifier", $field))
       $value=Mutilate::humanReadableNumberS($d->v[$field]);
      else
       $value=$d->v[$field];

      break;
     }
     case "url":
     {
      if($d->v[$field])
       $value=$this->link($d->v[$field], $d->v[$field], $this->language->t("Open: ".$d->v[$field]), "online", Array("rel" => "external"));

      break;
     }
     case "plugin":
     {
      $value=$this->link($d->v[$field], Array("plugins/".$d->v[$field]."/"), $this->language->t("Show ".$d->v[$field]." plugin"));
      $url=false;
      break;
     }
     case "module":
     {
      $value=$this->link($d->v[$field], Array("modules/".$d->v[$field]."/"), $this->language->t("Show ".$d->v[$field]." items"));
      $url=false;
      break;
     }
     case "modules":
     {
      $value=Array();
      foreach($d->v[$field] as $v)
       $value[]=$this->link($v, Array("modules/".$v."/"), $this->language->t("Show ".$v." items"));

      $value=implode(", ", $value);
      $url=false;
      break;
     }
     case "localfile":
     {
      if($d->v[$field])
      {
       $value=$this->link($d->v[$field], Array("/files/".$d->module."/".$field."/".$d->v[$id]."/"), $this->language->t("Download: ".$d->v[$field]));
       $value.="<br/>".Mutilate::humanReadableFileSizeS(FS::sizeS($d->field[$field]["localpath"]["directory"].$d->v[$field]));
      }

      break;
     }
     case "file":
     {
      if($d->v[$field])
      {
       $value=$this->link($d->v[$field], Array("/files/".$d->module."/".$field."/".$d->v[$id]."/"), $this->language->t("Download: ".$d->v[$field]));

       if($d->check("properties", "md5")&&$d->v[$field."MD5"])
        $value.=" (".$this->link("MD5", Array("/files/".$d->module."/".$field."/".$d->v[$id]."/md5/"), $this->language->t("Download MD5 hash for: ".$d->v[$field])).")";

       $value.="<br/>".$d->v[$field."Type"];
       $value.="<br/>".Mutilate::humanReadableFileSizeS($d->v[$field."Size"]);
      }

      break;
     }
     case "image":
     {
      if($d->v[$field])
      {
       $value=$this->link("<img src=\"".$this->url("/images/".$d->module."/".$field."/".$d->v[$id]."/height/80/")."\" alt=\"".$d->v[$field]."\" class=\"preview\"/> ".$d->v[$field] , Array("/images/".$d->module."/".$field."/".$d->v[$id]."/"), $this->language->t("Download: ".$d->v[$field]));
       $value.="<br/> ".$d->v[$field."ImageWidth"]."x".$d->v[$field."ImageWidth"];
       $value.="<br/> ".$d->v[$field."Type"];
       $value.="<br/> ".Mutilate::humanReadableFileSizeS($d->v[$field."Size"]);
      }

      break;
     }
     case "rights":
     {
      if(is_array($d->v["nr"])) //show user/group with rights as tooltip
      {
       if($d->module=="userRights")
       {
        $value=$this->tooltip($d->v["nr"]["username"], $this->form->rights(false, $d->v[$field], false, false, "disabled"));
        $url=$this->url("rights/", Array("usernr" => $d->v["nr"]["nr"], "rights" => $d->v[$field], "nr[]" => $d->v["catnr"]));
       }
       else if($d->module=="groupRights")
       {
        $value=$this->tooltip($d->v["nr"]["name"], $this->form->rights(false, $d->v[$field], false, false, "disabled"));
        $url=$this->url("rights/", Array("groupnr" => $d->v["nr"]["nr"], "rights" => $d->v[$field], "nr[]" => $d->v["catnr"]));
       }
      }
      else if($d->module=="userRights"||$d->module=="groupRights") //show active rights
      {
       $keys=Array();
       foreach($d->rights as $type => $rights)
       {
        $values=Array();
        foreach($rights as $right => $key)
        {
         if(!in_array($key, $keys))
         {
          if($d->v[$field][$key])
           $values[]=strtoupper($d->l->t($right)[0]);

          $keys[]=$key;
         }
        }

        if($values)
         $value.=($value?" ":"").$this->icon(($type=="category"?"category":"data"), "small")." ".implode("", $values);
       }

       $value=$this->tooltip($value, $this->form->rights(false, $d->v[$field], false, false, "disabled"));
      }
      else //show rights
       $value=$this->form->rights(false, (isset($d->v[$field])?$d->v[$field]:$d->getValue($field, $d->vc-1)), false, false, "disabled");

      break;
     }
     default:
     {
      if(is_array($d->v[$field])) //list
       $value=implode(", ", $d->v[$field]);
      else //normal value
       $value=$d->v[$field];
     }
    }

    $class=$d->field[$field]["type"]." ".$field;
   }

   return true;
  }


  /*
   * CMS
   */

  function cms(&$d, $url=false, $config=false)
  {
   $cms=new CMS($config);
   $cms->setDataForm($d, $this->form);

   $content="";
   $submit=false;
   foreach($d->check("use", "show") as $field)
   {
    if($field=="r") //skip rights field
     continue;

    $content.=$cms->field($field);

    if($d->check("type", "submit", $field)) //submit field
     $submit=true;
   }

   if(!$submit) //no submit in form field
    $content.=$cms->formShowSubmit();

   if(isset($config["showCancel"])&&$config["showCancel"])
    $content.=$cms->formShowCancel();

   return $this->form($content, $url);
  }

  function cmsOptions(&$d, $options=false, $menu=false, $vars=Array())
  {
   if(!$options)
    $options="copy;edit;move;delete;undelete;owner;rights;order";

   $options=explode(";", $options);

   $cmsOptions=Array();

   if($d->action=="undelete"||$d->extra=="deleted")
   {
    if(in_array("undelete", $options)&&$d->validAction("undelete"))
     $cmsOptions["undelete"]=Array(
      "content" => $this->language->t("Undelete"),
      "action"  => ($d->r("undelete")?$this->url("cms/".$d->module."/undelete/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Undelete").": ".$d->v("title"),
      "icon"    => "undelete".($d->r("undelete")?"":"_disabled"),
     );
   }
   else
   {
    if(in_array("add", $options)&&$d->validAction("add"))
     $cmsOptions["add"]=Array(
      "content" => $this->language->t("Add"),
      "action"  => ($d->r("add")?$this->url("cms/".$d->module."/add/item/", $vars):false),
      "title"   => $this->language->t("Add: ".$d->title),
      "icon"    => "add".($d->r("add")?"":"_disabled"),
     );

    if(in_array("copy", $options)&&$d->validAction("copy"))
     $cmsOptions["copy"]=Array(
      "content" => $this->language->t("Copy"),
      "action"  => ($d->r("copy")?$this->url("cms/".$d->module."/copy/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Copy: ".$d->v("title")),
      "icon"    => "copy".($d->r("copy")?"":"_disabled"),
     );

    if(in_array("edit", $options)&&$d->validAction("edit"))
     $cmsOptions["edit"]=Array(
      "content" => $this->language->t("Edit"),
      "action"  => ($d->r("edit")?$this->url("cms/".$d->module."/edit/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Edit: ".$d->v("title")),
      "icon"    => "edit".($d->r("edit")?"":"_disabled"),
     );

    if(in_array("move", $options)&&$d->validAction("move"))
     $cmsOptions["move"]=Array(
      "content" => $this->language->t("Move"),
      "action"  => ($d->r("move")?$this->url("cms/".$d->module."/move/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Move: ".$d->v("title")),
      "icon"    => "move".($d->r("move")?"":"_disabled"),
     );

    if(in_array("delete", $options)&&$d->validAction("delete"))
     $cmsOptions["delete"]=Array(
      "content" => $this->language->t("Delete"),
      "action"  => ($d->r("delete")?$this->url("cms/".$d->module."/delete/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Delete: ".$d->v("title")),
      "icon"    => "delete".($d->r("delete")?"":"_disabled"),
     );

    if(in_array("owner", $options)&&$d->validAction("owner"))
     $cmsOptions["owner"]=Array(
      "content" => $this->language->t("Owner"),
      "action"  => ($d->r("owner")?$this->url("cms/".$d->module."/owner/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Change owner: ".$d->v("title")),
      "icon"    => "owner".($d->r("owner")?"":"_disabled"),
     );

    if(in_array("rights", $options)&&$d->validAction("rights"))
     $cmsOptions["rights"]=Array(
      "content" => $this->language->t("Rights"),
      "action"  => ($d->r("rights")?$this->url("cms/".$d->module."/rights/item/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Change rights: ".$d->v("title")),
      "icon"    => "rights".($d->r("rights")?"":"_disabled"),
     );

    if(in_array("order", $options)&&$d->validAction("order"))
    {
     $ordering=$d->check("properties", "order")[0];

     $cmsOptions["up"]=Array(
      "content" => $this->language->t("Up"),
      "action"  => ($d->r("edit")?$this->url(false, array_merge($vars, Array($d->getID() => $d->v("id"), "sort" => $ordering, "a" => "order", "e" => "up"))):false),
      "title"   => $this->language->t("Order up: ".$d->v("title")),
      "icon"    => "up".($d->r("edit")?"":"_disabled"),
     );

     $cmsOptions["down"]=Array(
      "content" => $this->language->t("Down"),
      "action"  => ($d->r("edit")?$this->url(false, array_merge($vars, Array($d->getID() => $d->v("id"), "sort" => $ordering, "a" => "order", "e" => "down"))):false),
      "title"   => $this->language->t("Order down: ".$d->v("title")),
      "icon"    => "down".($d->r("edit")?"":"_disabled"),
     );
    }

    if(in_array("lost", $options)&&$d->validAction("lost"))
     $cmsOptions["lost"]=Array(
      "content" => $this->language->t("Lost and found"),
      "action"  => ($d->r("rights")?$this->url("cms/".$d->module."/lost/andfound/".$d->v("id")."/"):false),
      "title"   => $this->language->t("Recover lost and found: ".$d->v("title")),
      "icon"    => "lostandfound".($d->r("lost")?"":"_disabled"),
     );

   }

   if(!$menu)
   {
    foreach($cmsOptions as $key => $option)
     $cmsOptions[$key]["icon"]=$this->icon($option["icon"]);

    if(sizeof($cmsOptions)==1)
     return current($cmsOptions);

    return $cmsOptions;
   }

   foreach($cmsOptions as $option)
    $this->menuItem("cms_".$d->module."_".$d->v("id"), $option["content"], $option["action"], $option["title"], $option["icon"], ($option["action"]?false:Array("disabled")));

   return $this->menu("cms_".$d->module."_".$d->v("id"));
  }


  /*
   * Support
   */

  function url($path=false, $args=false)
  {
   global $_path;

   if(is_array($args))
    $args=Mutilate::stripTagsS($args);
   else
    $args=Array();

   $questionmark=false; //no ?arguments=yes

   if($path===false) //current page
   {
    if($this->backend->rewrite) //path
     $url=$_path["HTML"]["url"]; //current url
    else //vars
    {
     $url=$_path["HTML"]["current"]; //current directory
     if($this->backend->pageVars)
     {
      $url.="?"; //add page vars
      foreach($this->backend->pageVars as $arg)
       $url.="p[]=".urlencode($arg)."&";

      $url=substr($url, 0, -1); //remove last &
      $questionmark=true;
     }
    }
   }
   else //different page
   {
    if($this->backend->rewrite) //path
     $url=FS::realPathS($_path["HTML"]["current"].$path); //add path
    else //vars
    {
     if(substr($path, 0, 1)!="/") //current path
     {
      $url=$_path["HTML"]["current"]; //current directory
      if($path=trim($path, "/")) //contains page vars
      {
       $url.="?"; //add page vars
       foreach(explode("/", $path) as $arg)
        $url.="p[]=".urlencode($arg)."&";

       $url=substr($url, 0, -1); //remove last &
       $questionmark=true;
      }
     }
     else //root path
     {
      $url=$_path["root"];
      $path=explode("/", trim($path, "/"));
      foreach($path as $entry => $directory)
      {
       if($this->backend->fs->exists($url.$directory."/", "directory")) //actual directory
        $url.=$directory."/"; //add directory
       else //no more folders
       {
        $url.="?"; //add page vars
        foreach(array_slice($path, $entry) as $arg)
         $url.="p[]=".urlencode($arg)."&";

        $url=substr($url, 0, -1); //remove last &
        $questionmark=true;
        break; //done
       }
      }
     }
    }
   }

   //(additional) URL arguments
   foreach($args as $k => $v)
   {
    if($questionmark) //questionmark already set
     $url.="&";
    else //no vars yet
    {
     $url.="?";
     $questionmark=true;
    }

    $url.=urlencode($k)."=".urlencode($v);
   }

   return $url;
  }


  function link($content, $action, $title=false, $icon=false, $properties=false)
  {
   if(is_array($action))
    $action=$this->url($action[0], (isset($action[1])?$action[1]:false));

   if($title===false)
    $title=Mutilate::stripTagsS($content);

   $title=Mutilate::htmlEncodeS($title);

   //Properties:
   //- class: link class
   //- style: link specific styling
   //- disabled: disable link (remove link and add disabled class)
   //- selected: selected link (class)
   //- rel: link relation

   if(!$properties)
    $properties=Array();

   if(is_int(array_search("disabled", $properties))) //disabled flag set
    if(preg_match("/^[0-9a-z\-]+$/i", $icon)) //icon
     $icon.="_disabled";

   //class
   $class="";
   if(isset($properties["class"]))
    $class.=$properties["class"];

   if(is_int(array_search("disabled", $properties))) //disabled flag set
   {
    $disabled=true;
    $class.=($class?" ":"")."disabled";
   }
   else
    $disabled=false;

   if(is_int(array_search("selected", $properties))) //selected flag set
    $class.=($class?" ":"")."selected";

   //style
   $style="";
   if(isset($properties["style"]))
    $style=$properties["style"];

   //rel
   $rel="";
   if(isset($properties["rel"]))
    $rel=$properties["rel"];

   //render link
   $content=($icon?$this->icon($icon, (stristr($class, "small")?"small":false))." ":"").$content;

   if($action&&!$disabled)
    $r="<a href=\"".$action."\"".($class?" class=\"".$class."\"":"").($style?" style=\"".$style."\"":"").($rel?" rel=\"".$rel."\"":"")." title=\"".$title."\">".$content."</a>";
   else
    $r=$content;

   return $r;
  }


  function form($content, $url=false)
  {
   if(!$url) //current page
   {
    $r=$this->form->start()."\n";

    if(!$this->backend->rewrite) //vars
     foreach($this->backend->pageVars as $page)
      $r.=$this->form->input("hidden", "p[]", $page);
   }
   else //different page
   {
    $url=explode("?", $url, 2); //split path and args

    $r=$this->form->start($url[0])."\n"; //path

    if(isset($url[1])) //vars
    {
     foreach(explode("&", $url[1]) as $var)
     {
      $var=explode("=", urldecode($var), 2);
      if(isset($var[1])) //key=value
       $r.=$this->form->input("hidden", $var[0], $var[1])."\n";
      else //key
       $r.=$this->form->input("hidden", $var[0])."\n";
     }
    }
   }

   $r.=$content;
   $r.=$this->form->end()."\n";

   return $r;
  }


  function tooltip($tool, $tip)
  {
   $r="<div class=\"tool\">".$tool;
   $r.="<div class=\"tip\">".$tip;
   $r.="</div>";
   $r.="</div>\n";

   return $r;
  }

  function field($title, $input, $options=false)
  {
   if(!$title)
    $title="&nbsp; ";

   if(!$input)
    $input="&nbsp; ";

   $r="<div class=\"field\">";
   $r.="<div class=\"title\">".$title."</div>";
   $r.="<div class=\"input\">".$input."</div>";

   if($options!==false)
    $r.="<div class=\"options\">".$options."</div>";

   $r.="</div>\n";

   return $r;
  }

  function confirm($properties=false)
  {
   //Properties:
   //- disabled: disable item (remove link and add disabled class)

   if(!$properties)
    $properties=Array();

   if(is_int(array_search("disabled", $properties))) //disabled flag set
    $disabled=true;
   else
    $disabled=false;

   $buttons=$this->form->button("s", $this->icon("enable", "small")." ".$this->language->t("Yes"), false, false, "submit".($disabled?" disabled":""))." ";

   if(is_int(array_search("history", $properties))) //history flag set
   {
    $hs=new History();

    if($hs->getTotal()&&($page=$hs->previous())) //history in use
     $buttons.=$this->form->button("", $this->icon("disable", "small")." ".$this->language->t("No"), false, false, "onclick=\"document.location='".$page."'\"");
    else //no history, use browser history
     $buttons.=$this->form->button("", $this->icon("disable", "small")." ".$this->language->t("No"), false, false, "onclick=\"history.back();\"");
   }
   else
    $buttons.=$this->form->button("", $this->icon("disable", "small")." ".$this->language->t("No"), false, false, "submit");

   return $this->field($this->language->t("Confirm"), $buttons);
  }


  function icon($icon, $class=false)
  {
   global $_path, $_jasPath;

   if(preg_match("/^[0-9a-z\-\_]+$/i", $icon)&&$this->backend->fs->exists($_jasPath["project"]["images"]."icons/".$icon.".png")) //icon
    $file=$this->backend->fs->realPath($_path["HTML"]["current"].$_jasPath["project"]["images"]."icons/".$icon.".png");
   else if($this->backend->fs->exists($icon)) //local file
    $file=$this->backend->fs->realPath($_path["HTML"]["current"].$icon);
   else
    return false;

   return "<img src=\"".$file."\" class=\"icon".($class?" ".$class:"")."\" alt=\"".ucfirst($icon)."\"/>";
  }

 }
?>
