<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/language.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."mutilate.php");

 class Backend_Language extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);
  }

  function main($section=false)
  {
   global $_session, $_config, $_path;

   //section
   if(!parent::main($section))
    return false;

   $_session->frontend();

   //reload translations
   if(request("reloadTranslations")) //actual reloading already done in backend
    $this->e->add("Translations reloaded", "note");

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   if($this->section=="errors")
   {
    $this->backend->setTitle($this->icon("language_dim"),
     $this->link($this->language->t("Language"), Array("language/")),
     $this->link($this->language->t("Errors"), Array("language/errors/")));

    //errors
    $total=0;
    $from=false;
    $to=false;
    foreach($this->language->errors as $original => $languages)
    {
     foreach($languages as $language => $errors)
     {
      if($original!=$from||$language!=$to)
      {
       $this->tableRow("language", true);
       $this->tableColumn("language", $this->language->t("Text"));
       $this->tableColumn("language", strtoupper($original)." => ".strtoupper($language));
       $this->tableColumn("language", $this->language->t("URL"));
      }

      ksort($errors);
      foreach($errors as $text => $error)
      {
       $total++;
       $this->tableRow("language");
       $this->tableColumn("language", $text);
       $this->tableColumn("language", strtoupper(implode(", ", $error["sets"])));
       $this->tableColumn("language", $this->link("", $error["url"], $error["url"], "link"));
      }
     }
    }

    if(isset($this->table["language"]))
    {
     $this->tableRow("language", false, Array("class" => "total"));

     if(!$total)
      $this->tableColumn("language", $this->language->t("No translation errors"), false, false, Array("colspan" => 3));
     else
     {
      $this->tableColumn("language", $this->language->t("Total"), false, false, Array("colspan" => 2));
      $this->tableColumn("language", $total, false, false, Array("class" => "nr"));
     }
    }
    else
    {
     $this->tableRow("language", true);
     $this->tableColumn("language", $this->language->t("Text"));
     $this->tableColumn("language", "");
     $this->tableColumn("language", $this->language->t("URL"));

     $this->tableRow("language");
     $this->tableColumn("language", $this->language->t("No translation errors"), false, false, Array("colspan" => 3));
    }
   }
   else
   {
    $this->backend->setTitle($this->icon("language_dim"),
     $this->link($this->language->t("Language"), Array("language/")));

    //files
    $total=0;
    $from=false;
    $to=false;
    foreach($this->language->files as $original => $languages)
    {
     foreach($languages as $language => $files)
     {
      if($original!=$from||$language!=$to)
      {
       $this->tableRow("language", true);
       $this->tableColumn("language", $this->language->t("File"));
       $this->tableColumn("language", strtoupper($original)." => ".strtoupper($language));
       $this->tableColumn("language", $this->language->t("Updated"));
      }

      ksort($files);
      foreach($files as $set => $file)
      {
       $total++;
       $this->tableRow("language");
       $this->tableColumn("language", $file);
       $this->tableColumn("language", strtoupper($set));
       $this->tableColumn("language", Mutilate::timeAgoS(filemtime($file), "U"));
      }
     }
    }

    if(isset($this->table["language"]))
    {
     $this->tableRow("language", false, Array("class" => "total"));

     if(!$total)
      $this->tableColumn("language", $this->language->t("No translations"), false, false, Array("colspan" => 3));
     else
     {
      $this->tableColumn("language", $this->language->t("Total"), false, false, Array("colspan" => 2));
      $this->tableColumn("language", $total, false, false, Array("class" => "nr"));
     }
    }
    else
    {
     $this->tableRow("language", true);
     $this->tableColumn("language", $this->language->t("File"));
     $this->tableColumn("language", "");
     $this->tableColumn("language", $this->language->t("Updated"));

     $this->tableRow("language");
     $this->tableColumn("language", $this->language->t("No translations"), false, false, Array("colspan" => 3));
    }
   }

   $r="<div class=\"twothird\">\n";
   $r.=$this->table("language");
   $r.="</div>\n";


   //translations
   $this->tableRow("translations", true);
   $this->tableColumn("translations", "", false, false, Array("colspan" => 3));
   if(!isset($this->language->translations["raw"]))
   {
    $this->tableRow("translations");
    $this->tableColumn("translations", $this->language->t("None"), false, false, Array("colspan" => 3));
   }
   else
   {
    $translations=Array();
    foreach(array_keys($this->language->translations["raw"]) as $from)
    {
     foreach(array_keys($this->language->translations["raw"][$from]) as $to)
     {
      foreach(array_keys($this->language->translations["raw"][$from][$to]) as $set)
      {
       if(!isset($translations[$from.$to]))
       {
        $translations[$from.$to]["from"]=$from;
        $translations[$from.$to]["to"]=$to;
        $translations[$from.$to]["sets"]=1;
        $translations[$from.$to]["count"]=sizeof($this->language->translations["raw"][$from][$to][$set]);
       }
       else
       {
        $translations[$from.$to]["sets"]++;
        $translations[$from.$to]["count"]+=sizeof($this->language->translations["raw"][$from][$to][$set]);
       }
      }
     }
    }

    foreach($translations as $translation)
    {
     $this->tableRow("translations");
     $this->tableColumn("translations", strtoupper($translation["from"]." => ".$translation["to"]));
     $this->tableColumn("translations", $this->language->t($translation["count"]." translations in ".$translation["sets"]." sets"));
    }
   }

   //actions
   if($this->section=="errors")
    $this->menuItem("actions", $this->language->t("Translation files"), $this->url("language/"), $this->language->t("Translation files"), "file");
   else
    $this->menuItem("actions", $this->language->t("Translation errors"), $this->url("language/errors/"), $this->language->t("Translation errors"), "error");

   $this->menuItem("actions", $this->language->t("Reload translations"), $this->url(false, Array("reloadTranslations" => "true")), $this->language->t("Reload translations"), "reload");
   $this->menuItem("actions", $this->language->t("Configuration"), $this->url("configuration/"), $this->language->t("Configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("language_dim", "small")." ".$this->language->t("Translations")."</h3>\n";
   $r.=$this->table("translations");
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }
 }
?>
