<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/projects.plugins.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/plugins.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."jas.php");

 class Backend_Projects_Plugins extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->data=false;
   $this->jasRootUser=false;
  }


  function data()
  {
   global $_session;

   $this->data=Data::cache("empty", "add", "item", Array("skipCheckTables" => true));

   $this->data->addField("enabled", "list", "", "", false, 1000000, false, "Existing plugins");

   $this->data->addField("plugins", "multilist", "all", "", false, 1000000, false, "Plugins");

   $this->data->setRequestValues();

   $project=$this->data->getValue("backendProject");

   $_session->frontend();

   $enabled=Array();
   $plugins=Array();
   foreach(Backend_Plugins::plugins() as $plugin => $v)
   {
    $plugins[$plugin]=$v["title"];

    if($v["enabled"])
     $enabled[]=$plugin;
   }

   $this->data->listOptions("plugins", $plugins);
   $this->data->listOptions("enabled", $enabled);

   if($this->data->getValue("plugins"))
    $this->data->setValue("plugins", array_unique(array_merge($this->data->listOptions["enabled"], $this->data->getValue("plugins"))));
   else
    $this->data->setValue("plugins", $this->data->listOptions["enabled"]);

   $_session->backend();
  }


  function status(&$text)
  {
   $this->data();

   if(!$this->data->listOptions["enabled"])
   {
    $text="No plugins enabled";
    return "";
   }

   $text=sizeof($this->data->listOptions["enabled"])." plugin(s) enabled";
   return "Done";
  }

  function main($submit=true)
  {
   global $_session, $_config, $_Backend_Plugin_List;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status
   if(!$this->jasRootUser)
   {
    $this->e->add("Project plugins can only be setup by the JAS root user", "error");
    return false;
   }

   $this->data();

   $_session->frontend();

   if(request("s")&&$submit)
   {
    $success=true;
    foreach(Backend_Plugins::plugins() as $plugin => $v)
    {
     if(in_array($plugin, $this->data->getValue("plugins")))
     {
      $p=new Plugin($plugin);

      //enabled plugin
      if(!$p->isEnabled())
      {
       $_config->su(true);
       if(!$p->setStatus(true))
       {
        $_config->su();
        $this->e->add($p->e);
        $_session->backend();
        return false;
       }
       else
       {
        $_config->su();

        if(isset($_Backend_Plugin_List[$_session->get("project")][$plugin])) //update cached plugin list
         $_Backend_Plugin_List[$_session->get("project")][$plugin]["enabled"]=true;
       }
      }

      //setup plugin
      $_config->su(true);
      $p->initClass("setup", false, false);

      if(!$p->call("setup", "init")||$p->e->any("fatal"))
      {
       $_config->su();
       $this->e->add($p->e);
       $_session->backend();
       return false;
      }

      if(!$p->call("setup", "exec")||$p->e->any("fatal"))
      {
       $_config->su();
       $this->e->add($p->e);
       $_session->backend();
       return false;
      }

      $p->call("setup", "clear");
      $_config->su();
     }
    }

    if($success)
    {
     $this->e->add("Plugins for project: \"".$_session->get("project")."\" successfully created", "note");
     $_session->backend();
     return "next";
    }
    else
    {
     $_session->backend();
     return false;
    }
   }

   $_session->backend();
   return true;
  }


  /*
   * Output
   */

  function output()
  {
   if(!$this->jasRootUser)
    return "&nbsp; ";

   $cms=new CMS();
   $cms->l->read("jas");
   $cms->setDataForm($this->data, $this->form);

   $plugins=Array();
   foreach($this->data->listOptions["plugins"] as $k => $dir)
    $plugins[$k]=(in_array($k, $this->data->listOptions["enabled"])?"<i>".$dir."</i>":$dir);

   $this->data->listOptions("plugins", $plugins);

   $form="";
   foreach($this->data->check("use", "form") as $field)
    $form.=$cms->field($field);

   $form.=$this->confirm(Array("history", (!$this->jasRootUser?"disabled":false)));

   $r=$this->form($form, $this->url("projects/setup/plugins/"));

   return $r;
  }

 }
?>
