<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/rights.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class Backend_Rights extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->nr=false;
   $this->module="categories";
  }

  function main($nr=false)
  {
   global $_session, $_config;

   $_session->frontend();

   //modify quick categories list url
   if($quick=$this->backend->component("backend/categories/quick")) //categories list found
    $quick->path=false; //current page

   $d=Data::cache($this->module, "rights", "item");

   if(!$d->check("properties", "rights"))
   {
    $d->e->clear();
    $d->e->add($d->title." module doesn't have an rights reference", "error");
   }

   if($d->e->any("fatal"))
    $this->e->add($d->e);
   else
   {
    $id=$d->getID();

    if(request("confirm"))
    {
     if($d->field[$id]["valuetype"]!="request")
      $this->e->add("No ".$d->title." items selected to change the rights", "warning");
     else if(!$d->getValue("usernr")&&!$d->getValue("groupnr"))
      $this->e->add("No user/group selected to change the rights", "warning");
     else
      $this->e->add("Are you sure you wish to change the rights for ".$d->vmax." ".$d->title." item(s)?", "warning");
    }
    else if(request("s")&&$d->field[$id]["valuetype"]=="request") //values submitted
    {
     //set page as CMS in history
     $hs=new History();
     if(!$hs->cms())
      $this->e->add($hs->e);

     $d->submit();
     $this->e->add($d->e);
    }

    if($nr)
    {
     $this->nr=(int)$nr;
     $this->section="item";
    }
    else
    {
     $this->section="list";

     //set inline search options
     if($titles=$d->check("properties", "title"))
      $this->backend->search("data", $this->module, "title", $titles[0]);

     //search redirect
     if(request("search"))
     {
      $d->setAction("search");
      if($d->fulltextIndex) //full text index exists
       $d->setExtra("index");
      else //normal search
       $d->setExtra("full");

      $d->setShowFields();
      $d->setValue("show_total", 2);
      $d->setRequestValue("search");

      if($d->get()&&$d->vmax==1) //only 1 entry
       Headers::redirectS($this->url("rights/".$d->getValue($d->getID())."/"));
     }
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");

   $this->backend->setTitle($this->icon("rights_dim"),
    $this->link($this->language->t("Rights"), Array("rights/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
    $d->setValue($field, $catnr);

   $list=$this->viewList($d, Array("checkboxes", "nopagerform", "get" => 2, "rights"));

   if($this->backend->rewrite)
    $list=preg_replace("!href=\"([^\"]*)data/".$d->module."/(\d+)/\"!", "href=\"\\1rights/\\2/\"", $list);
   else
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data&p\[\]=".$d->module."(&p\[\]=\d+)\"!", "href=\"\\1p[]=rights\\2\"", $list);

   $r="<div class=\"twothird rights\">\n";
   $r.=$list;
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   $r.="<div class=\"third rights\">\n";

   if(request("search"))
    $r.=$this->form->input("hidden", "search", strip_tags(request("search")));
   else if(request("catnr"))
    $r.=$this->form->input("hidden", "catnr", strip_tags(request("catnr")));

   //rights submit
   $r.="<h3>".$this->icon("rights_dim", "small")." ".$this->language->t("Rights")."</h3>\n";

   $cms=new CMS();
   $cms->setDataForm($d, $this->form);

   if(!isset($d->field["usernr"]))
    $d->moduleObject->initRightsFields();

   $d->clearValues();
   $d->setDefaultValue("usernr");
   $d->setDefaultValue("groupnr");
   $d->setRequestValues();

   if(request("confirm")&&request($d->getID()))
   {
    $r.=str_replace("\n", "", $cms->field("usernr"));
    $r.=str_replace("\n", "", $cms->field("groupnr"));
    $r.=$this->form->rights("rights", request("rights"));
    $r.=$this->confirm((!$d->r("rights")?Array("disabled"):false));
   }
   else
   {
    $r.=str_replace("\n", "", $cms->field("usernr"));
    $r.=str_replace("\n", "", $cms->field("groupnr"));
    $r.=$this->form->rights("rights", request("rights"));
    $r.=$this->field($this->language->t("Items"), $this->form->button("confirm", $this->icon("rights", "small")." ".$this->language->t("Rights"), false, false, "submit".($d->r("rights")?"":" disabled")));
   }

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add;edit;move;owner;rights;delete", true)."\n";
   $r.="</div>\n";

   //make full page part of the form
   $r=$this->form($r);

   $_session->backend();

   return $r;
  }

  function output_list(){return $this->output_default();}

  function output_item()
  {
   global $_session;

   $_session->frontend();

   //set default title
   $this->backend->setTitle($this->icon("rights_dim"),
    $this->link($this->language->t("Rights"), Array("rights/")));

   $d=Data::cache($this->module, "view", "item");
   $d->setValue($d->getID(), $this->nr);

   $r="<div class=\"twothird rights\">\n";
   $r.=$this->viewItem($d, Array("show" => "normal", "get" => 1))."<br/>";

   //set title after viewItem got data
   $this->backend->setTitle($this->icon("rights_dim"),
    $this->link($this->language->t("Rights"), Array("rights/")),
    $this->link(Mutilate::stripS(strip_tags($d->v("title")), 24), $this->url()));

   //categories rights
   $du=Data::cache("userRights", "view", "all");
   $du->clearValues();

   $du->setShowFields("nr", "rights");
   $du->getReferenceData("nr")->setShowFields($du->getReferenceData("nr")->check("properties", "title"));

   $dg=Data::cache("groupRights", "view", "all");
   $dg->clearValues();

   $dg->setShowFields("nr", "rights");
   $dg->getReferenceData("nr")->setShowFields($dg->getReferenceData("nr")->check("properties", "title"));

   $du->setValue("catnr", $this->nr);
   $dg->setValue("catnr", $this->nr);

   if(!$du->get(1))
    $this->e->add($du->e);

   if(!$dg->get(1))
    $this->e->add($dg->e);

   if($du->vmax)
   {
    $this->tableRow("item", true);
    $this->tableColumn("item", $this->icon("user_dim")." ".$this->language->t("Users"));
    $this->tableColumn("item", $this->language->t("Rights"));

    while($du->next())
    {
     $this->tableRow("item", false, Array("class" => "userrights"));
     $this->tableColumn("item", $du->getReferenceData("nr")->v("title"), Array("users/".$du->getReferenceData("nr")->v("id")."/"), $this->language->t("View ".$du->getReferenceData("nr")->v("title")));
     $this->tableColumn("item", $this->form->rights(false, $du->v["rights"], false, false, "disabled"));
    }
   }

   if($dg->vmax)
   {
    $this->tableRow("item", true);
    $this->tableColumn("item", $this->icon("group_dim")." ".$this->language->t("Groups"));
    $this->tableColumn("item", $this->language->t("Rights"));

    while($dg->next())
    {
     $this->tableRow("item", false, Array("class" => "grouprights"));
     $this->tableColumn("item", $dg->getReferenceData("nr")->v("title"), Array("users/?catnr=".$dg->getReferenceData("nr")->v("id")), $this->language->t("View ".$dg->getReferenceData("nr")->v("title")));
     $this->tableColumn("item", $this->form->rights(false, $dg->v["rights"], false, false, "disabled"));
    }
   }

   if(!isset($this->table["item"]))
   {
    $_session->backend();
    return false;
   }

   $r.=$this->table("item");
   $r.="</div>\n";


   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "default"))."\n";

   $r.="<h3>".$this->icon("link_dim", "small")." ".$this->language->t("Reference")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "reference"))."\n";

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "add;edit;move;owner;rights;delete", true)."\n";
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }


 }
?>
