

 /*****************************************************************************
 * Filename: script/calendar.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


if(typeof(onload)!='function')
 alert("Script not loaded: functions.js");
else
 onload("initDates();");

function initDates()
{
 var tmp=document.getElementsByTagName("input");

 dates=Array();
 for(var c=0;c<tmp.length;c++)
  if(tmp[c].type=="text"&&tmp[c].className.match(/(^|\s)(date|datetime)(\s|$)/))
   if(!tmp[c].disabled)
    dates[c]=new Calendar(tmp[c], c);
}


/*
 * Date format support
 * */

Date.prototype.days=Array("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday");
Date.prototype.months=Array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");
Date.prototype.monthsDays=Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

Date.prototype.getDayOfYear=function()
{
 var days=this.getDate()-1;
 for(var c=0;c<this.getMonth();c++)
 {
  days+=this.monthsDays[c]; //add days for the month

  if(this.getFullYear()%4==0&&c==1) //leap year
   days++; //add additional day for February
 }

 return days;
}

Date.prototype.set=function(date, format)
{
 var formatConvert={
  "d": "(\\d{2})", //days
  "j": "(\\d{1,2})",
  "m": "(\\d{2})", //months
  "n": "(\\d{1,2})",
  "Y": "(\\d{4})", //years
  "y": "(\\d{2})",
  "H": "(\\d{2})", //hours
  "G": "(\\d{1,2})",
  "i": "(\\d{2})", //minutes
  "s": "(\\d{2})", //seconds
  "W": "(\\d{1,2})", //weeks
  "z": "(\\d{1,3})" //days (per year)
 };

 var regexp=format;
 for(key in formatConvert)
  regexp=regexp.replace(key, formatConvert[key]);

 regexp=new RegExp(regexp);

 var keys;
 if(!(keys=format.match(/[a-z]/ig)))
  return false;

 var values;
 if(!(values=date.match(regexp)))
  return false;

 //merge values and keys
 values.shift();
 var tmp=Array();
 for(var c=0;c<keys.length;c++)
  tmp[keys[c]]=values[c];

 values=tmp;

 if(values["d"]!=undefined)
  this.setDate(values["d"]);
 else if(values["j"]!=undefined)
  this.setDate(values["j"]);

 if(values["m"]!=undefined)
  this.setMonth(values["m"]-1);
 else if(values["n"]!=undefined)
  this.setMonth(values["n"]-1);

 if(values["Y"]!=undefined)
  this.setFullYear(values["Y"]);
 else if(values["y"]!=undefined)
 {
  var year=parseInt(values["y"]);
  if(year>=50) //we use it from 1950 till 2049
   year+=1900;
  else
   year+=2000;

  this.setFullYear(year);
 }

 if(values["H"]!=undefined)
  this.setHours(values["H"]);
 else if(values["G"]!=undefined)
  this.setHours(values["G"]);

 if(values["i"]!=undefined)
  this.setMinutes(values["i"]);

 if(values["s"]!=undefined)
  this.setSeconds(values["s"]);

 if(values["W"]!=undefined) //week number
 {
  var date=(values["W"]-1)*7; //determine day of the year
  date=new Date(this.getFullYear(), 0, date);

  var weekday;
  if(new Date(year, 11, 31).format("W")==53) //53 weeks year
  {
   if((weekday=new Date(date.toString()).format("N"))>1) //not monday
    date.setDate(date.getDate()-(weekday-1)); //round date to previous monday
  }
  else //52 weeks year
  {
   if((weekday=new Date(date.toString()).format("N"))>1) //not monday
    date.setDate(date.getDate()+(8-weekday)); //round date to previous monday
  }

  if(values["d"]==undefined&&values["j"]==undefined) //day not set
   this.setDate(date.format("d")); //determine day of the month

  if(values["m"]==undefined&&values["n"]==undefined) //month not set
   this.setMonth(date.format("m")-1); //determine month

  this.setFullYear(date.format("Y")); //ovewrite year in case start of the week falls in another year
 }

 if(values["z"]!=undefined) //numberic day of the year
 {
  var date=parseInt(values["z"])+1; //determine day of the year
  date=new Date(this.getFullYear(), 0, date);

  if(values["d"]==undefined&&values["j"]==undefined) //day not set
   this.setDate(date.format("d")); //determine day of the month

  if(values["m"]==undefined&&values["n"]==undefined) //month not set
   this.setMonth(date.format("m")-1); //determine month

  this.setFullYear(date.format("Y")); //ovewrite year in case start of the week falls in another year
 }

 return true;
}

Date.prototype.format=function(format)
{
 var value
 var output="";

 //output date in PHP formatted string
 for(var c=0;c<format.length;c++)
 {
  if(format[c]=="\\"&&(c+1)<format.length) //backslashed characters are output normally
  {
   output+=format[c+1];
   c++; //skip to next

   continue;
  }

  switch(format[c])
  {
   case "a": //am
   case "A": //AM
   {
    if(this.getHours()<12)
     value="AM";
    else
     value="PM";

    if(format[c]=="a")
     value=value.toLowerCase();

    output+=value;
    break;
   }
   case "s": //second: 01
   {
    value=this.getSeconds();

    if(value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }
   case "i": //minute: 01
   {
    value=this.getMinutes();

    if(value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }
   case "g": //hour: 1 (12h)
   case "h": //hour: 01 (12h)
   case "G": //hour: 1 (24h)
   case "H": //hour: 01 (24h)
   {
    value=this.getHours();

    if((format[c]=="g"||format[c]=="h")&&value>12) //convert 12 hour clock
     value=value-12;

    if((format[c]=="h"||format[c]=="H")&&value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }

   case "z": //day of the year: 1
   {
    value=this.getDayOfYear();

    output+=value;
    break;
   }

   case "j": //day: 1
   case "d": //day: 01
   {
    value=this.getDate();

    if(format[c]=="d"&&value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }

   case "N": //weekday, 1 = Mon, 2 = Tue ... 7 = Sun
   {
    value=this.getDay();

    if(value==0) //set Sunday to end of the week
     value=7;

    output+=value;
    break;
   }
   case "w": //weekday, 0 = Sun, 1 = Mon ... 6 = Sat
   {
    value=this.getDay();

    output+=value;
    break;
   }
   case "J": //day: M
   case "D": //day: Mon
   case "l": //day: Monday
   {
    value=this.days[this.getDay()];

    if(format[c]=="J") //only first character
     value=value.toString().substr(0, 1);
    else if(format[c]=="D") //only first 3 characters
     value=value.toString().substr(0, 3);

    output+=value;
    break;
   }
   case "S": //day: st
   {
    value=this.getDate();

    if(value%10>3) //determine st, nd, rd, th
     value="th";
    else if(value%10==3)
     value="rd";
    else if(value%10==2)
     value="nd";
    else if(value%10==1)
     value="st";

    output+=value;
    break;
   }

   case "W": //week: 1
   {
    var weekday=(this.getDay()==0?7:this.getDay()); //weekday, 1 = Mon, 2 = Tue ... 7 = Sun

    value=(this.getDayOfYear()-(weekday+1))+10; //ordinal date - weekday + 10
    value=Math.floor(value/7); //divide by 7 ignoring remainder

    //determine begin/end of year oddities
    var dowFirstLastYear=new Date(this.getFullYear()-1, 0, 1); //day of the week of first day of last year
    dowFirstLastYear=(dowFirstLastYear.getDay()==0?7:dowFirstLastYear.getDay()); //weekday, 1 = Mon, 2 = Tue ... 7 = Sun

    var dowFirstThisYear=new Date(this.getFullYear(), 0, 1); //day of the week of first day of this year
    dowFirstThisYear=(dowFirstThisYear.getDay()==0?7:dowFirstThisYear.getDay()); //weekday, 1 = Mon, 2 = Tue ... 7 = Sun

    var dowFirstNextYear=new Date(this.getFullYear()+1, 0, 1); //day of the week of first day of next year
    dowFirstNextYear=(dowFirstNextYear.getDay()==0?7:dowFirstNextYear.getDay()); //weekday, 1 = Mon, 2 = Tue ... 7 = Sun


    if(dowFirstThisYear==3&&dowFirstNextYear==4) //this year starts on Wednesday, next year on Thursday
    {
     value+=1; //add 1 week

     if(value>52) //more than 52 weeks: is in next year
      value-=52;
    }

    if(dowFirstThisYear==4) //this year starts on Thursday
    {
     value+=1; //add 1 week
    }
    else //this year doesn't start on Thursday
    {
     if(value>52) //more than 52 weeks: is in next year
      value-=52;
    }

    if(dowFirstLastYear==3&&dowFirstThisYear==5) //last year starts on Wednesday, this year on Friday
    {
     if(value==0) //week 0: is previous week
      value+=53;
    }

    if(dowFirstThisYear==3&&dowFirstNextYear==5) //this year starts on Wednesday, next year on Friday
    {
     value+=1; //add 1 week
    }

    if(dowFirstLastYear==4) //last year starts on Thursday
    {
     if(value==0) //week 0: is previous week
      value+=53;
    }
    else //last year doesn't start on Thursday
    {
     if(value==0) //week 0: is previous week
      value+=52;
    }

    if(value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }

   case "n": //month: 1
   case "m": //month: 01
   {
    value=this.getMonth()+1;

    if(format[c]=="m"&&value.toString().length<2) //add leading zero
     value="0"+value;

    output+=value;
    break;
   }
   case "M": //month: Jan
   case "F": //month: January
   {
    value=this.months[this.getMonth()];

    if(format[c]=="M") //only first 3 characters
     value=value.toString().substr(0, 3);

    output+=value;
    break;
   }
   case "t": //days in month: 28 - 31
   {
    value=this.monthsDays[this.getMonth()];

    if(this.getFullYear()%4==0&&value==28) //leap year
     value++; //add additional day for February

    output+=value;
    break;
   }

   case "y": //year: 99
   case "Y": //year: 2001
   {
    value=this.getFullYear();

    if(format[c]=="y") //remove first 2 characters
     value=value.toString().substr(2);

    output+=value;
    break;
   }
   case "L": //leap year: 1 or 0
   {
    if(this.getFullYear()%4==0) //leap year
     value=1;
    else
     value=0;

    output+=value;
    break;
   }

   case "O": //timezone: +0200
   case "P": //timezone: +02:00
   {
    var negative=false;
    value=this.getTimezoneOffset()/60;

    if(value<0) //get offset and flip prefix
     value=0-value;
    else //flip prefix
     negative=true;

    if(value.toString().length<2) //add leading zero
     value="0"+value;

    if(format[c]=="O") //add 00
     value+="00";
    else //add :00
     value+=":00";

    if(negative) //add - prefix
     value="-"+value;
    else //add + prefix
     value="+"+value;

    output+=value;
    break;
   }
   case "I": //DST
   {
    nodst=new Date(this.getFullYear(), 1, 1);

    if(this.getTimezoneOffset()==nodst.getTimezoneOffset()) //offset matched non DST offset, note in case of a timezone without DST this should also match (and thus not come up with DST)
     value=0; //no DST
    else //offsets don't match
     value=1; //DST

    output+=value;
    break;
   }

   default:
   {
    output+=format[c];
    break;
   }
  }
 }

 return output;
}


/*
 *
 * Calendar class
 * */

function Calendar(input, id)
{
 this.input=input;
 this.id=id;

 this.init=init;

 this.generateCalendar=generateCalendar;
 this.update=update;

 this.show=show;
 this.hide=hide;
 this._hide=_hide;
 this.active={"input": false, "calendar": false};

 this.setDate=setDate;
 this.inputChanged=inputChanged;
 this.previousMonth=previousMonth;
 this.nextMonth=nextMonth;

 this.addEvent=addEvent;
 this.removeEvent=removeEvent;

 //default config
 this.config=Array();
 this.config["hideTimeout"]=100;
 this.config["showWeekend"]=true;
 this.config["weekdays"]=Array("Monday", "Tuesday", "Wednesday", "Thursday", "Friday");

 this.config["dateForm"]=true;
 this.config["timeForm"]=true;

 this.config["format"]=Array();
 this.config["format"]["header"]="F Y";
 this.config["format"]["time"]="G:i:s";
 this.config["format"]["weekdays"]="D";
 this.config["format"]["days"]="j";
 this.config["format"]["weeks"]="W";
 this.config["format"]["input"]="H:i:s d-m-Y";

 this.date=new Date();
 this.init();

 function init()
 {
  //calendar
  this.calendar=document.createElement("div");
  this.calendar.className="calendar";
  this.calendar.style.display="none";
  this.calendar.style.position="absolute";
  this.input.parentNode.insertBefore(this.calendar, this.input.nextSibling);

  //get input date format from placeholder (HTML5) attribute
  if(this.input.placeholder)
  {
   this.config["format"]["input"]=this.input.placeholder;

   var time;
   if(this.input.placeholder.match(/[siHG]+/)&&(time=this.input.placeholder.match(/([siHG][\s\:\-\_]*)+/))) //time found in date format
    this.config["format"]["time"]=time[0];
   else //no time, don't display it
    this.config["format"]["time"]="";
  }

  //set date
  if(this.input.value)
   this.date.set(this.input.value, this.config["format"]["input"]);

  //add events
  this.addEvent(this.input, "change", this.inputChanged);
  this.addEvent(this.input, "focus", this.show);
  this.addEvent(this.input, "blur", this.hide);
  this.addEvent(this.calendar, "mouseover", this.show);
  this.addEvent(this.calendar, "mouseout", this.hide);

  //setup screen
  this.update();

  return true;
 }


//Displaying

 function generateCalendar()
 {
  //lookup week days
  var weekdays=Array();
  for(var c=0;c<this.config["weekdays"].length;c++)
  {
   var found=false;
   for(var x=0;x<this.date.days.length;x++)
   {
    if(this.config["weekdays"][c]==this.date.days[x])
    {
     weekdays.push(x); //store weekday number
     found=true;
     break;
    }
   }

   if(!found)
    alert("Unknown week day: "+this.config["weekdays"][c]);
  }

  //lookup weekend days
  if(this.config["showWeekend"])
   var start=weekdays[weekdays.length-1];
   for(var x=start;x<start+this.date.days.length;x++)
    if(!this.config["weekdays"].in_array(this.date.days[x%7]))
     weekdays.push(x); //store weekday number

  var startDate=new Date(this.date.toString());
  startDate.setDate(1); //set to first of the month

  //align at start of weekdays
  if(startDate.getDay()==0&&weekdays[0]==1)
   startDate.setDate(1+weekdays[0]-7);
  else
   startDate.setDate(1+weekdays[0]-startDate.getDay());

  var numberOfWeeks=7;
  for(var c=0;c<7;c++)
  {
   var tmp=new Date(startDate.toString());
   tmp.setDate(startDate.getDate()+(c*7));

   if((tmp.getMonth()+(tmp.getFullYear()*12))>(this.date.getMonth()+(this.date.getFullYear()*12)))
   {
    numberOfWeeks=c;
    break;
   }
  }


  //render
  var calendar="";
  var colspan=0;

  //determine colspan
  colspan+=(this.config["format"]["weeks"]?1:0); //show week numbering
  colspan+=weekdays.length; //show week days

  //create calendar
  calendar="<table class=\"calendar\">\n";

  //header
  calendar+=" <tr class=\"header\">";
  calendar+="<td class=\"previous\" onclick=\"dates["+this.id+"].previousMonth();\">&lt;&lt;</td>";

  if(this.config["format"]["header"])
  {
   var value="";

   if(!this.config["dateForm"])
    value=this.date.format(this.config["format"]["header"]);
   else
   {
    for(var c=0;c<this.config["format"]["header"].length;c++)
    {
     switch(this.config["format"]["header"][c])
     {
      case "j":
      case "d":
      case "n":
      case "m":
      case "M":
      case "F":
      case "y":
      case "Y":
      case "W":
      {
       var current=this.date.format(this.config["format"]["header"][c]);

       var form;
       if(this.config["format"]["header"][c]=="M"||this.config["format"]["header"][c]=="F")
        form="<select class=\""+this.config["format"]["header"][c]+"\" onchange=\"dates["+this.id+"].setDate(this.options[this.selectedIndex].value, 'n')\">";
       else
        form="<select class=\""+this.config["format"]["header"][c]+"\" onchange=\"dates["+this.id+"].setDate(this.options[this.selectedIndex].value, '"+this.config["format"]["header"][c]+"')\">";

       var start, end;
       if(this.config["format"]["header"][c]=="j"||this.config["format"]["header"][c]=="d")
       {
        start=1;
        end=this.date.format("t");
       }
       else if(this.config["format"]["header"][c]=="n"||this.config["format"]["header"][c]=="m"||this.config["format"]["header"][c]=="M"||this.config["format"]["header"][c]=="F")
       {
        start=1;
        end=12;
       }
       else if(this.config["format"]["header"][c]=="W")
       {
        start=1;
        end=53;
       }
       else
       {
        start=1950;
        end=2050;
       }

       for(var x=start;x<=end;x++)
       {
        if((this.config["format"]["header"][c]=="d"||this.config["format"]["header"][c]=="m"||this.config["format"]["header"][c]=="y")&&x.toString().length<2) //add leading zero
         x="0"+x;

        if(this.config["format"]["header"][c]=="y") //remove first 2 characters
         x=x.toString().substr(2);

        var text;
        if(this.config["format"]["header"][c]=="M"||this.config["format"]["header"][c]=="F")
        {
         text=this.date.months[x-1];

         if(this.config["format"]["header"][c]=="M")
          text=text.toString().substr(0, 3);
        }
        else
         text=x;

        form+="<option value=\""+x+"\""+(text==current?" selected=\"selected\"":"")+">"+text;
       }

       form+="</select>";

       value+=form;
       break;
      }
      default:
      {
       value+=this.date.format(this.config["format"]["header"][c]);
       break;
      }
     }
    }
   }

   calendar+="<td colspan=\""+(colspan-2)+"\">"+value+"</td>";
  }

  calendar+="<td class=\"next\" onclick=\"dates["+this.id+"].nextMonth();\">&gt;&gt;</td>";
  calendar+"</tr>\n";

  calendar+=" <tr class=\"weekdays\">";

  if(this.config["format"]["weeks"])
   calendar+="<td class=\"weeknr\">&nbsp; </td>"; //week number

  for(var c=0;c<weekdays.length;c++)
  {
   var tmp=new Date(startDate.toString());
   tmp.setDate(startDate.getDate()+c);

   var classes=Array();
   if(this.config["weekdays"].in_array(tmp.format("l")))
    classes.push("week");
   else
    classes.push("weekend");

   calendar+="<td class=\""+classes.join(" ")+"\">"+tmp.format(this.config["format"]["weekdays"])+"</td>";
  }

  calendar+="</tr>\n";


  //body
  var today=new Date();
  for(var c=0;c<numberOfWeeks;c++)
  {
   var tmp=new Date(startDate.toString());
   tmp.setDate(startDate.getDate()+(c*7));

   if(tmp.format("W-Y")==today.format("W-Y"))
    calendar+=" <tr class=\"days thisweek\">";
   else
    calendar+=" <tr class=\"days\">";

   if(this.config["format"]["weeks"])
    calendar+="<td class=\"weeknr\">"+tmp.format(this.config["format"]["weeks"])+"</td>";

   for(var x=0;x<weekdays.length;x++)
   {
    var tmp=new Date(startDate.toString());
    tmp.setDate(startDate.getDate()+(c*7)+x);

    var classes=Array();
    if(this.config["weekdays"].in_array(tmp.format("l")))
     classes.push("week");
    else
     classes.push("weekend");

    if(tmp.getMonth()<this.date.getMonth())
     classes.push("previous");
    else if(tmp.getMonth()>this.date.getMonth())
     classes.push("next");
    else
     classes.push("current");

    if(tmp.format("d-m-Y")==this.date.format("d-m-Y"))
     classes.push("selected");

    if(tmp.format("d-m-Y")==today.format("d-m-Y"))
     classes.push("today");

    calendar+="<td class=\""+classes.join(" ")+"\" onclick=\"dates["+this.id+"].setDate('"+tmp.format("d-m-Y")+"', 'd-m-Y');\">"+tmp.format(this.config["format"]["days"])+"</td>";
   }
  }
  calendar+="</tr>\n";

  //time
  if(this.config["format"]["time"])
  {
   var value="";

   if(!this.config["timeForm"])
    value=this.date.format(this.config["format"]["time"]);
   else
   {
    for(var c=0;c<this.config["format"]["time"].length;c++)
    {
     switch(this.config["format"]["time"][c])
     {
      case "s":
      case "i":
      case "G":
      case "H":
      {
       var current=this.date.format(this.config["format"]["time"][c]);

       var form="<select class=\""+this.config["format"]["time"][c]+"\" onchange=\"dates["+this.id+"].setDate(this.options[this.selectedIndex].value, '"+this.config["format"]["time"][c]+"')\">";

       var start, end;
       if(this.config["format"]["time"][c]=="H"||this.config["format"]["time"][c]=="G")
       {
        start=0;
        end=24;
       }
       else
       {
        start=0;
        end=60;
       }

       for(var x=start;x<end;x++)
       {
        if(this.config["format"]["time"][c]!="G"&&x.toString().length<2) //add leading zero
         x="0"+x;

        form+="<option value=\""+x+"\""+(x==current?" selected=\"selected\"":"")+">"+x;
       }

       form+="</select>";

       value+=form;
       break;
      }
      default:
      {
       value+=this.date.format(this.config["format"]["time"][c]);
       break;
      }
     }
    }
   }

   calendar+=" <tr class=\"time\"><td colspan=\""+colspan+"\">"+value+"</td></tr>\n";
  }

  calendar+="</table>";

  return calendar;
 }


 function update()
 {
  this.calendar.innerHTML=this.generateCalendar();
 }

 function show(e)
 {
  var calendar, type=false;

  if(e==undefined)
   calendar=this;
  else //find parent with calendar reference
  {
   var node=(e.target!=undefined?e.target:e.srcElement);

   if(node.nodeName.match(/input/i))
    type="input";
   else
    type="calendar";

   while(true)
   {
    if(node.calendar!=undefined)
    {
     calendar=node.calendar;
     break;
    }

    if(!node.parentNode)
     return false;

    node=node.parentNode;
   }
  }

  if(type)
   calendar.active[type]=true;

  calendar.calendar.style.display="block";
 }

 function hide(e)
 {
  var calendar, type=false;

  if(e==undefined)
   calendar=this;
  else //find parent with calendar reference
  {
   var node=(e.target!=undefined?e.target:e.srcElement);

   if(node.nodeName.match(/select/i)) //ignore select hide events (google+ie)
    return false;

   if(node.nodeName.match(/input/i))
    type="input";
   else
    type="calendar";

   while(true)
   {
    if(node.calendar!=undefined)
    {
     calendar=node.calendar;
     break;
    }

    if(!node.parentNode)
     return false;

    node=node.parentNode;
   }
  }

  if(type)
   calendar.active[type]=false;

  //delayed hide to ensure subsequent show events are handled
  setTimeout("dates["+calendar.id+"]._hide()", calendar.config["hideTimeout"]);
 }

 function _hide()
 {
  for(key in this.active) //check if no elements are active
   if(this.active[key])
    return false;

  this.calendar.style.display="none";
  return true;
 }


 //Navigation

 function setDate(date, format)
 {
  this.date.set(date, format);
  this.input.value=this.date.format(this.config["format"]["input"]);

  this.update();

  if(this.input.onchange)
   this.input.onchange();

  this.active["calendar"]=false; //hide calendar on change (for mobile devices)
  this.hide();
 }

 function inputChanged(e)
 {
  var input=e.target;
  var calendar=e.target.calendar;

  if(input.value)
  {
   calendar.date.set(input.value, calendar.config["format"]["input"]);
   input.value=calendar.date.format(calendar.config["format"]["input"]);
  }

  calendar.update();
 }

 function previousMonth()
 {
  this.date.setMonth(this.date.getMonth()-1);
  this.update();
 }

 function nextMonth()
 {
  this.date.setMonth(this.date.getMonth()+1);
  this.update();
 }


 //Events

 function addEvent(obj, event, action)
 {
  obj.calendar=this;

  if(window.addEventListener)
   obj.addEventListener(event, action, false);
  else if(window.attachEvent)
   obj.attachEvent("on"+event, action);
 }

 function removeEvent(obj, event, action)
 {
  obj.calendar=undefined;

  if(window.addEventListener)
   obj.removeEventListener(event, action, false);
  else if(window.attachEvent)
   obj.detachEvent("on"+event, action);
 }


}
